@@ -1,4 +1,4 @@
-## Copyright (C) 1996, 1997, 2002, 2005, 2007 John W. Eaton
+## Copyright (C) 1996, 1997, 2002, 2005, 2007, 2008 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -23,8 +23,17 @@
 
 ## Author: jwe
 
+## Deprecated in version 3.0
+
 function retval = is_scalar (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "is_scalar is obsolete and will be removed from a future version of Octave; please use isscalar instead");
+  endif
+
   retval = isscalar (varargin{:});
 
 endfunction
