@@ -1,5 +1,5 @@
 ## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2004, 2005,
-##               2006, 2007 John W. Eaton
+##               2006, 2007, 2008, 2009 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -44,10 +44,19 @@ function y = fliplr (x)
   endif
 
   if (ndims (x) > 2)
-    error ("fliplr: Only works with 2-D arrays")
+    error ("fliplr: Only works with 2-D arrays");
   endif
 
   nc = columns (x);
   y = x (:, nc:-1:1);
 
 endfunction
+
+%!assert((fliplr ([1, 2; 3, 4]) == [2, 1; 4, 3]
+%! && fliplr ([1, 2; 3, 4; 5, 6]) == [2, 1; 4, 3; 6, 5]
+%! && fliplr ([1, 2, 3; 4, 5, 6]) == [3, 2, 1; 6, 5, 4]));
+
+%!error fliplr();
+
+%!error fliplr (1, 2);
+
