@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1999, 2000, 2003, 2007 John W. Eaton
+Copyright (C) 1999, 2000, 2003, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -29,10 +29,6 @@ along with Octave; see the file COPYING.  If not, see
 #include <iostream>
 #include <string>
 
-#ifndef NPOS
-#define NPOS std::string::npos
-#endif
-
 static bool
 looks_like_octave_copyright (const std::string& s)
 {
@@ -139,20 +135,26 @@ int
 main (int argc, char **argv)
 {
   std::string name;
+  std::string file_name;
 
-  if (argc != 2)
+  if (argc != 3)
     {
-      std::cerr << "usage: gethelp name\n";
+      std::cerr << "usage: gethelp name file-name\n";
       return 1;
     }
   else
-    name = argv[1];
+    {
+      name = argv[1];
+      file_name = argv[2];
+    }
 
   std::string help_text = extract_help_text ();  
 
   if (! help_text.empty ())
     {
-      std::cout << "" << name << "\n" << help_text;
+      std::cout << "" << name << "\n"
+		<< "@c " << file_name << "\n"
+		<< help_text;
 
       if (help_text[help_text.length () - 1] != '\n')
 	std::cout << "\n";
