@@ -1,4 +1,4 @@
-## Copyright (C) 2004, 2005, 2006, 2007 John W. Eaton
+## Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -19,19 +19,27 @@
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} delete (@var{file})
 ## @deftypefnx {Function File} {} delete (@var{h})
-## Delete the named file or figure handle.
+## Delete the named file or graphics handle.
 ## @end deftypefn
 
-## PKG_ADD: mark_as_command delete
-
 ## Author: jwe
 
 function delete (arg)
 
   if (nargin == 1)
     if (ischar (arg))
-      unlink (arg);
-    elseif (ishandle (arg))
+      files = glob (arg).';
+      if (isempty (files))
+        warning ("delete: no such file: %s", arg);
+      endif
+      for i = 1:length (files)
+        file = files{i};
+        [err, msg] = unlink (file);
+        if (err)
+          warning ("delete: %s: %s", file, msg);
+        endif
+      endfor
+    elseif (all (ishandle (arg(:))))
       ## Delete a graphics object.
       __go_delete__ (arg);
     else
