@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2007 John W. Eaton
+## Copyright (C) 2005, 2007, 2008, 2009 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -16,7 +16,10 @@
 ## along with Octave; see the file COPYING.  If not, see
 ## <http://www.gnu.org/licenses/>.
 
+## -*- texinfo -*-
+## @deftypefn {Function File} {@var{h} =} __line__ (@var{p}, @dots{})
 ## Undocumented internal function.
+## @end deftypefn
 
 ## __line__ (p, x, y, z)
 ## Create line object from x, y, and z with parent p.
@@ -42,8 +45,7 @@ function h = __line__ (p, varargin)
     num_data_args = 0;
   endif
 
-  if (rem (nvargs - num_data_args, 2) == 0)
-  else
+  if (rem (nvargs - num_data_args, 2) != 0)
     print_usage ("line");
   endif
 
