@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2006, 2007 John W. Eaton
+## Copyright (C) 2005, 2006, 2007, 2008, 2009 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -18,9 +18,9 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} hold @var{args}
-## Tell Octave to `hold' the current data on the plot when executing
+## Tell Octave to `hold' the current data on the graph when executing
 ## subsequent plotting commands.  This allows you to execute a series of
-## plot commands and have all the lines end up on the same figure.  The
+## plot commands and have all the lines end up on the same graph.  The
 ## default is for each new plot command to clear the plot device first.
 ## For example, the command
 ##
@@ -32,20 +32,30 @@
 ## turns the hold state on.  An argument of @code{"off"} turns the hold
 ## state off, and @code{hold} with no arguments toggles the current hold
 ## state.
+##
+## @deftypefnx {Function File} {} hold (@var{h}, @dots{})
+## 
+## Applies to a specific axis or axes, associated with the handle(s), 
+## @var{h}.
+## 
 ## @end deftypefn
 
-## PKG_ADD: mark_as_command hold
-
 function hold (varargin)
 
-  [h, varargin] = __plt_get_axis_arg__ ("hold", varargin{:});
+  if (nargin > 0 && numel (varargin{1}) == 1 && ishandle (varargin{1}(1))
+      && strcmp (get (varargin{1}, "type"), "axes"))
+    [h, varargin, nargs] = __plt_get_axis_arg__ ("hold", varargin{:});
+  elseif (nargin > 0 && numel (varargin{1}) > 1 && ishandle (varargin{1}(1)))
+    print_usage ();
+  else
+    h = gcf ();
+    nargs = numel (varargin);
+  endif
 
   hold_state = get (h, "nextplot");
 
-  nargs = numel (varargin);
-
   if (nargs == 0)
-    if (strcmp (hold_state, "add"))
+    if (strcmpi (hold_state, "add"))
       hold_state = "replace";
     else
       hold_state = "add";
@@ -53,9 +63,9 @@ function hold (varargin)
   elseif (nargs == 1)
     state = varargin{1};
     if (ischar (state))
-      if (strcmp ("off", state))
+      if (strcmpi (state, "off"))
 	hold_state = "replace";
-      elseif (strcmp ("on", state))
+      elseif (strcmpi (state, "on"))
 	hold_state = "add";
       else
 	print_usage ();
@@ -65,6 +75,45 @@ function hold (varargin)
     print_usage ();
   endif
 
+  if (isfigure (h))
+    if (isempty (get (h, "currentaxes")))
+      set (h, "currentaxes", __go_axes__ (h))
+    endif
+    axes_objs = findobj (h, "type", "axes");
+    h = [h; axes_objs];
+  endif
+
   set (h, "nextplot", hold_state);
 
 endfunction
+
+%!demo
+%! clf
+%! A = rand (100);
+%! [X, Y] = find (A > 0.9);
+%! imshow (A)
+%! hold on
+%! plot (X, Y, 'o')
+%! hold off
+
+%!demo
+%! clf
+%! hold on
+%! imagesc(1./hilb(4));
+%! plot (1:4, "-s")
+%! hold off
+
+%!demo
+%! clf
+%! hold on
+%! imagesc(1./hilb(2));
+%! imagesc(1./hilb(4));
+%! hold off
+
+%!demo
+%! clf
+%! hold on
+%! plot (1:4, "-s")
+%! imagesc(1./hilb(4));
+%! hold off
+
