@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2006, 2007 John W. Eaton
+## Copyright (C) 2005, 2006, 2007, 2008 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -25,7 +25,7 @@
 function retval = ishold ()
 
   if (nargin == 0)
-    retval = strcmp (get (gca (), "nextplot"), "add");
+    retval = strcmpi (get (gca (), "nextplot"), "add");
   else
     print_usage ();
   endif
