@@ -29,17 +29,22 @@
 
 function retval = loglog (varargin)
 
-  newplot ();
+  [h, varargin] = __plt_get_axis_arg__ ("loglog", varargin{:});
 
-  ## [h, varargin] = __plt_get_axis_arg__ ("loglog", varargin{:});
-  h = gca ();
+  oldh = gca ();
+  unwind_protect
+    axes (h);
+    newplot ();
 
-  set (h, "xscale", "log", "yscale", "log");
+    set (h, "xscale", "log", "yscale", "log");
 
-  tmp = __plt__ ("loglog", h, varargin{:});
+    tmp = __plt__ ("loglog", h, varargin{:});
 
-  if (nargout > 0)
-    retval = tmp;
-  endif
+    if (nargout > 0)
+      retval = tmp;
+    endif
+  unwind_protect_cleanup
+    axes (oldh);
+  end_unwind_protect
 
 endfunction
