@@ -1,4 +1,4 @@
-## Copyright (C) 2007 David Bateman
+## Copyright (C) 2007, 2008, 2009 David Bateman
 ##
 ## This file is part of Octave.
 ##
@@ -85,7 +85,7 @@ function hlist = __pie__ (varargin)
     if (iscell (arg))
       labels = arg;
       have_labels = true;
-      if (! size_equal (x, labels))
+      if (numel (x) != numel (labels))
 	error ("pie: mismatch in number of labels and data");
       endif
     elseif (isnumeric (arg))
