@@ -1,5 +1,5 @@
 ## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2003,
-##               2004, 2005, 2006, 2007 John W. Eaton
+##               2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -22,14 +22,12 @@
 ## @deftypefnx {Function File} {} ylabel (@var{string})
 ## @deftypefnx {Function File} {} zlabel (@var{string})
 ## @deftypefnx {Function File} {} xlabel (@var{h}, @var{string})
-## Specify x, y, and z axis labels for the current figure. If @var{h} is
+## Specify x, y, and z axis labels for the current figure.  If @var{h} is
 ## specified then label the axis defined by @var{h}.
 ## @seealso{plot, semilogx, semilogy, loglog, polar, mesh, contour,
-## bar, stairs,  ylabel, title}
+## bar, stairs, title}
 ## @end deftypefn
 
-## PKG_ADD: mark_as_command xlabel
-
 ## Author: jwe
 
 function retval = xlabel (varargin)
@@ -49,7 +47,7 @@ function retval = xlabel (varargin)
   end_unwind_protect
 
   if (nargout > 0)
-    retval = h;
+    retval = tmp;
   endif
 
 endfunction
