@@ -1,11 +1,11 @@
-## Copyright (C) 2000 Paul Kienzle
+## Copyright (C) 2000, 2006, 2007, 2008, 2009 Paul Kienzle
 ##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,19 +13,18 @@
 ## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with Octave; see the file COPYING.  If not, write to the Free
-## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301, USA.
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {@var{pp} = } mkpp (@var{x}, @var{p})
-## @deftypefnx {Function File} {@var{pp} = } mkpp (@var{x}, @var{p}, @var{d})
+## @deftypefn {Function File} {@var{pp} =} mkpp (@var{x}, @var{p})
+## @deftypefnx {Function File} {@var{pp} =} mkpp (@var{x}, @var{p}, @var{d})
 ## 
 ## Construct a piece-wise polynomial structure from sample points
-## @var{x} and coefficients @var{p}.  The ith row of @var{p},
+## @var{x} and coefficients @var{p}.  The i-th row of @var{p},
 ## @code{@var{p} (@var{i},:)}, contains the coefficients for the polynomial
 ## over the @var{i}-th interval, ordered from highest to 
-## lowest. There must be one row for each interval in @var{x}, so 
+## lowest.  There must be one row for each interval in @var{x}, so 
 ## @code{rows (@var{p}) == length (@var{x}) - 1}.  
 ##
 ## You can concatenate multiple polynomials of the same order over the 
@@ -34,7 +33,7 @@
 ## * (length (@var{x}) - 1)}.
 ##
 ## @var{d} specifies the shape of the matrix @var{p} for all except the
-## last dimension. If @var{d} is not specified it will be computed as
+## last dimension.  If @var{d} is not specified it will be computed as
 ## @code{round (rows (@var{p}) / (length (@var{x}) - 1))} instead.
 ##
 ## @seealso{unmkpp, ppval, spline}
@@ -52,7 +51,7 @@ function pp = mkpp (x, P, d)
     d = round (rows (P) / pp.n); 
   endif
   pp.d = d;
-  if (pp.n*d != rows (P))
+  if (pp.n * prod (d) != rows (P))
     error ("mkpp: num intervals in x doesn't match num polynomials in P");
   endif
 endfunction
@@ -64,4 +63,5 @@ endfunction
 %! b=t(1:4,:);
 %! pp = mkpp(x, [m(:),b(:)]);
 %! xi=linspace(0,pi,50);
-%! plot(x,t,"x;control;",xi,ppval(pp,xi),";interp;");
+%! plot(x,t,"x",xi,ppval(pp,xi));
+%! legend("control","interp");
