@@ -1,4 +1,4 @@
-## Copyright (C) 2004, 2005, 2007 David Bateman & Andy Adler
+## Copyright (C) 2004, 2005, 2007, 2008, 2009 David Bateman and Andy Adler
 ##
 ## This file is part of Octave.
 ##
@@ -20,7 +20,7 @@
 ## @deftypefn {Function File} {@var{p} =} colperm (@var{s})
 ## Returns the column permutations such that the columns of
 ## @code{@var{s} (:, @var{p})} are ordered in terms of increase number
-## of non-zero elements. If @var{s} is symmetric, then @var{p} is chosen
+## of non-zero elements.  If @var{s} is symmetric, then @var{p} is chosen
 ## such that @code{@var{s} (@var{p}, @var{p})} orders the rows and
 ## columns with increasing number of non zeros elements.
 ## @end deftypefn
@@ -31,7 +31,7 @@ function p = colperm (s)
     print_usage ();
   endif
 
-  [i, j] = spfind (s);
+  [i, j] = find (s);
   idx = find (diff ([j; Inf]) != 0);
   [dummy, p] = sort (idx - [0; idx(1:(end-1))]);
 endfunction
