@@ -1,4 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 1998, 2000, 2002, 2005, 2006, 2007
+## Copyright (C) 1995, 1996, 1997, 1998, 2000, 2002, 2005, 2006, 2007, 2009
 ##               Kurt Hornik
 ##
 ## This file is part of Octave.
@@ -21,13 +21,11 @@
 ## @deftypefn {Function File} {} cloglog (@var{x})
 ## Return the complementary log-log function of @var{x}, defined as
 ##
-## @iftex
 ## @tex
 ## $$
 ## {\rm cloglog}(x) = - \log (- \log (x))
 ## $$
 ## @end tex
-## @end iftex
 ## @ifnottex
 ## @example
 ## cloglog(x) = - log (- log (@var{x}))
