@@ -1,5 +1,5 @@
 ## Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2002, 2004, 2005,
-##               2006, 2007 Kurt Hornik
+##               2006, 2007, 2008, 2009 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -25,14 +25,12 @@
 ## a variable, the (@var{i}, @var{j})-th entry of
 ## @code{cov (@var{x}, @var{y})} is the covariance between the @var{i}-th
 ## variable in @var{x} and the @var{j}-th variable in @var{y}.
-## @iftex
 ## @tex
 ## $$
 ## \sigma_{ij} = {1 \over N-1} \sum_{i=1}^N (x_i - \bar{x})(y_i - \bar{y})
 ## $$
 ## where $\bar{x}$ and $\bar{y}$ are the mean values of $x$ and $y$.
 ## @end tex
-## @end iftex
 ## If called with one argument, compute @code{cov (@var{x}, @var{x})}.
 ## @end deftypefn
 
@@ -57,12 +55,23 @@ function c = cov (x, y)
     if (rows (y) != n)
       error ("cov: x and y must have the same number of observations");
     endif
-    x = x - ones (n, 1) * sum (x) / n;
-    y = y - ones (n, 1) * sum (y) / n;
+    x = center (x, 1);
+    y = center (y, 1);
     c = conj (x' * y / (n - 1));
   elseif (nargin == 1)
-    x = x - ones (n, 1) * sum (x) / n;
+    x = center (x, 1);
     c = conj (x' * x / (n - 1));
   endif
 
 endfunction
+
+%!test
+%! x = rand (10);
+%! cx1 = cov (x);
+%! cx2 = cov (x, x);
+%! assert(size (cx1) == [10, 10] && size (cx2) == [10, 10] && norm(cx1-cx2) < 1e1*eps);
+
+%!error cov ();
+
+%!error cov (1, 2, 3);
+
