@@ -1,11 +1,12 @@
-## Copyright (C) 1996, 1997 John W. Eaton
+## Copyright (C) 1996, 1997, 1998, 1999, 2000, 2004, 2005, 2006, 2007,
+##               2008, 2009 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,15 +14,14 @@
 ## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with Octave; see the file COPYING.  If not, write to the Free
-## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301, USA.
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} median (@var{x})
+## @deftypefn {Function File} {} median (@var{x}, @var{dim})
 ## If @var{x} is a vector, compute the median value of the elements of
-## @var{x}.
-## @iftex
+## @var{x}.  If the elements of @var{x} are sorted, the median is defined
+## as
 ## @tex
 ## $$
 ## {\rm median} (x) =
@@ -29,8 +29,7 @@
 ##           (x(N/2)+x(N/2+1))/2, & $N$ even.}
 ## $$
 ## @end tex
-## @end iftex
-## @ifinfo
+## @ifnottex
 ##
 ## @example
 ## @group
@@ -39,9 +38,10 @@
 ##             (x(N/2) + x((N/2)+1))/2,  N even
 ## @end group
 ## @end example
-## @end ifinfo
+## @end ifnottex
 ## If @var{x} is a matrix, compute the median value for each
-## column and return them in a row vector.
+## column and return them in a row vector.  If the optional @var{dim}
+## argument is given, operate along this dimension.
 ## @seealso{std, mean}
 ## @end deftypefn
 
@@ -92,3 +92,19 @@ function retval = median (a, dim)
   endif
 
 endfunction
+
+%!test
+%! x = [1, 2, 3, 4, 5, 6];
+%! x2 = x';
+%! y = [1, 2, 3, 4, 5, 6, 7];
+%! y2 = y';
+%! 
+%! assert((median (x) == median (x2) && median (x) == 3.5
+%! && median (y) == median (y2) && median (y) == 4
+%! && median ([x2, 2*x2]) == [3.5, 7]
+%! && median ([y2, 3*y2]) == [4, 12]));
+
+%!error median ();
+
+%!error median (1, 2, 3);
+
