@@ -1,5 +1,5 @@
 ## Copyright (C) 1995, 1996, 1997, 1998, 2000, 2002, 2004, 2005, 2006,
-##               2007 Kurt Hornik
+##               2007, 2009 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -41,7 +41,7 @@
 ## Author: KH <Kurt.Hornik@wu-wien.ac.at>
 ## Description: Compute variance
 
-function y = var (x, opt, dim)
+function retval = var (x, opt, dim)
 
   if (nargin < 1 || nargin > 3)
     print_usage ();
@@ -56,19 +56,13 @@ function y = var (x, opt, dim)
     opt = 0;
   endif
 
-  sz = size (x);
-  if (prod (sz) < 1)
-    error ("var: x must not be empty");
-  elseif (sz(dim) == 1)
-    y = 0;
+  n = size (x, dim);
+  if (n == 1)
+    retval = zeros (sz);
+  elseif (numel (x) > 0)
+    retval = sumsq (center (x, dim), dim) / (n + opt - 1);
   else
-    rng = ones (1, length (sz));
-    rng (dim) = sz (dim);
-    if (opt == 0)
-      y = sumsq (x - repmat(mean (x, dim), rng), dim) / (sz(dim) - 1);
-    else
-      y = sumsq (x - repmat(mean (x, dim), rng), dim) / sz(dim);
-    endif
+    error ("var: x must not be empty");
   endif
 
 endfunction
