@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2006, 2007 Paul Kienzle
+## Copyright (C) 2005, 2006, 2007, 2009 Paul Kienzle
 ##
 ## This file is part of Octave.
 ##
@@ -24,7 +24,7 @@
 ## @deftypefnx {Function File} {} fail (@var{code},'warning',@var{pattern})
 ##
 ## Return true if @var{code} fails with an error message matching
-## @var{pattern}, otherwise produce an error. Note that @var{code}
+## @var{pattern}, otherwise produce an error.  Note that @var{code}
 ## is a string and if @var{code} runs successfully, the error produced is:
 ##
 ## @example
@@ -34,8 +34,10 @@
 ## If the code fails with a different error, the message produced is:
 ##
 ## @example
+## @group
 ##           expected <pattern>
 ##           but got <text of actual error>
+## @end group
 ## @end example
 ##
 ## The angle brackets are not part of the output.
@@ -48,8 +50,6 @@
 
 ## Author: Paul Kienzle <pkienzle@users.sf.net>
 
-## PKG_ADD mark_as_command fail
-
 function ret = fail (code, pattern, warning_pattern)
 
   if (nargin < 1 || nargin > 3)
@@ -60,7 +60,7 @@ function ret = fail (code, pattern, warning_pattern)
   test_warning = (nargin > 1 && strcmp (pattern, "warning"));
   if (nargin == 3)
     pattern = warning_pattern;
-  elseif (nargin == 1 || (nargin==2 && test_warning))
+  elseif (nargin == 1 || (nargin == 2 && test_warning))
     pattern = "";
   endif
 
@@ -75,20 +75,24 @@ function ret = fail (code, pattern, warning_pattern)
   endif  
 
   if (test_warning)
-    ## perform the warning test
-    lastwarn ();  # clear old warnings
-    state = warning ("query", "quiet"); # make sure warnings are turned on
+    ## Perform the warning test.
+    ## Clear old warnings.
+    lastwarn ();
+    ## Make sure warnings are turned on.
+    state = warning ("query", "quiet");
     warning ("on", "quiet");
     try
       ## printf("lastwarn before %s: %s\n",code,lastwarn);
       evalin ("caller", sprintf ("%s;", code));
       ## printf("lastwarn after %s: %s\n",code,lastwarn);
-      err = lastwarn ();  # retrieve new warnings
+      ## Retrieve new warnings.
+      err = lastwarn ();
       warning (state.state, "quiet");
       if (isempty (err))
         msg = sprintf ("expected warning <%s> but got none", pattern); 
       else
-        err([1:9, end]) = [];  # transform "warning: ...\n" to "..."
+	## Transform "warning: ...\n" to "...".
+        err([1:9, end]) = [];
         if (! isempty (regexp (err, pattern, "once")))
 	  return;
 	endif
@@ -97,12 +101,13 @@ function ret = fail (code, pattern, warning_pattern)
     catch
       warning (state.state, "quiet");
       err = lasterr;
-      err([1:7, end]) = [];  # transform "error: ...\n", to "..."
+      ## Transform "error: ...\n", to "...".
+      err([1:7, end]) = [];
       msg = sprintf ("expected warning <%s> but got error <%s>", pattern, err);
     end_try_catch
       
   else
-    ## perform the error test
+    ## Perform the error test.
     try
       evalin ("caller", sprintf ("%s;", code));
       msg = sprintf ("expected error <%s> but got none", pattern);
@@ -118,8 +123,9 @@ function ret = fail (code, pattern, warning_pattern)
     end_try_catch
   endif
 
-  ## if we get here, then code didn't fail or error didn't match
+  ## If we get here, then code didn't fail or error didn't match.
   error (msg);
+
 endfunction
 
 %!fail ('[1,2]*[2,3]','nonconformant')
