@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1998 A. S. Hodel
+Copyright (C) 1998, 1999, 2000, 2002, 2003, 2004, 2005, 2006, 2007,
+              2008, 2009 A. S. Hodel
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -32,7 +32,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "config.h"
 
 #include <cfloat>
-#include <cmath>
 
 #include <iostream>
 #include <iomanip>
@@ -40,6 +39,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "CmplxQRP.h"
 #include "dbleQR.h"
 #include "f77-fcn.h"
+#include "lo-math.h"
 #include "quit.h"
 
 #include "defun-dld.h"
@@ -208,7 +208,7 @@ DEFUN_DLD (qz, args, nargout,
   "-*- texinfo -*-\n\
 @deftypefn {Loadable Function} {@var{lambda} =} qz (@var{a}, @var{b})\n\
 Generalized eigenvalue problem @math{A x = s B x},\n\
-@var{QZ} decomposition. There are three ways to call this function:\n\
+@var{QZ} decomposition.  There are three ways to call this function:\n\
 @enumerate\n\
 @item @code{lambda = qz(A,B)}\n\
 \n\
@@ -218,14 +218,14 @@ Computes the generalized eigenvalues\n\
 $\\lambda$\n\
 @end tex\n\
 @end iftex\n\
-@ifinfo\n\
+@ifnottex\n\
 @var{lambda}\n\
-@end ifinfo\n\
+@end ifnottex\n\
 of @math{(A - s B)}.\n\
 @item @code{[AA, BB, Q, Z, V, W, lambda] = qz (A, B)}\n\
 \n\
 Computes qz decomposition, generalized eigenvectors, and \n\
-        generalized eigenvalues of @math{(A - sB)}\n\
+generalized eigenvalues of @math{(A - sB)}\n\
 @iftex\n\
 @tex\n\
 $$ AV = BV{ \\rm diag }(\\lambda) $$\n\
@@ -233,52 +233,54 @@ $$ W^T A = { \\rm diag }(\\lambda)W^T B $$\n\
 $$ AA = Q^T AZ, BB = Q^T BZ $$\n\
 @end tex\n\
 @end iftex\n\
-@ifinfo\n\
+@ifnottex\n\
 @example\n\
 @group\n\
-        A*V = B*V*diag(lambda)\n\
-        W'*A = diag(lambda)*W'*B\n\
-        AA = Q'*A*Z, BB = Q'*B*Z\n\
+\n\
+    A*V = B*V*diag(lambda)\n\
+    W'*A = diag(lambda)*W'*B\n\
+    AA = Q'*A*Z, BB = Q'*B*Z\n\
+\n\
 @end group\n\
 @end example\n\
-@end ifinfo\n\
+@end ifnottex\n\
 with @var{Q} and @var{Z} orthogonal (unitary)= @var{I}\n\
 \n\
 @item @code{[AA,BB,Z@{, lambda@}] = qz(A,B,opt)}\n\
 \n\
 As in form [2], but allows ordering of generalized eigenpairs\n\
-        for (e.g.) solution of discrete time algebraic Riccati equations.\n\
-        Form 3 is not available for complex matrices, and does not compute\n\
-        the generalized eigenvectors @var{V}, @var{W}, nor the orthogonal matrix @var{Q}.\n\
+for (e.g.) solution of discrete time algebraic Riccati equations.\n\
+Form 3 is not available for complex matrices, and does not compute\n\
+the generalized eigenvectors @var{V}, @var{W}, nor the orthogonal matrix @var{Q}.\n\
 @table @var\n\
 @item opt\n\
- for ordering eigenvalues of the GEP pencil.  The leading  block\n\
-             of the revised pencil contains all eigenvalues that satisfy:\n\
+for ordering eigenvalues of the GEP pencil.  The leading block\n\
+of the revised pencil contains all eigenvalues that satisfy:\n\
 @table @code\n\
 @item \"N\"\n\
- = unordered (default) \n\
+= unordered (default) \n\
 \n\
 @item \"S\"\n\
 = small: leading block has all |lambda| <=1 \n\
 \n\
 @item \"B\"\n\
- = big: leading block has all |lambda >= 1 \n\
+= big: leading block has all |lambda| >= 1 \n\
 \n\
 @item \"-\"\n\
- = negative real part: leading  block has all eigenvalues\n\
-                  in the open left half-plant\n\
+= negative real part: leading block has all eigenvalues\n\
+in the open left half-plane\n\
 \n\
 @item \"+\"\n\
- = nonnegative real part:  leading block has all eigenvalues\n\
-                  in the closed right half-plane\n\
-@end  table\n\
+= non-negative real part: leading block has all eigenvalues\n\
+in the closed right half-plane\n\
+@end table\n\
 @end table\n\
 @end enumerate\n\
 \n\
 Note: qz performs permutation balancing, but not scaling (see balance).\n\
-      Order of output arguments was selected for compatibility with MATLAB\n\
+Order of output arguments was selected for compatibility with @sc{matlab}\n\
 \n\
-@seealso{balance, dare, eig, schur}\n\
+@seealso{balance, eig, schur}\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -290,7 +292,7 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
 
   if (nargin < 2 || nargin > 3 || nargout > 7)
     {
-      print_usage ("qz");
+      print_usage ();
       return retval;
     }
   else if (nargin == 3 && (nargout < 3 || nargout > 4))
@@ -304,11 +306,11 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
 #endif
 
   // Determine ordering option
-  std::string ord_job;
+  volatile char ord_job = 0;
   static double safmin;
 
   if (nargin == 2)
-    ord_job = "N";
+    ord_job = 'N';
   else if (!args(2).is_string ())
     {
       error ("qz: argument 3 must be a string");
@@ -316,13 +318,15 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
     }
   else
     {
-      ord_job = args(2).string_value ();
+      std::string tmp = args(2).string_value ();
 
-      if (ord_job[0] != 'N'
-	  && ord_job[0] != 'S'
-	  && ord_job[0] != 'B'
-	  && ord_job[0] != '+'
-	  && ord_job[0] != '-')
+      if (! tmp.empty ())
+	ord_job = tmp[0];
+
+      if (! (ord_job == 'N' || ord_job == 'n'
+	     || ord_job == 'S' || ord_job == 's'
+	     || ord_job == 'B' || ord_job == 'b'
+	     || ord_job == '+' || ord_job == '-'))
 	{
 	  error ("qz: invalid order option");
 	  return retval;
@@ -473,12 +477,6 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
 		 nn, ilo, ihi, lscale.fortran_vec (),
 		 rscale.fortran_vec (), work.fortran_vec (), info
 		 F77_CHAR_ARG_LEN (1)));
-
-      if (f77_exception_encountered)
-	{
-	  error ("unrecoverable error in qz (bal)");
-	  return retval;
-	}
     }
 
   // Since we just want the balancing matrices, we can use dggbal
@@ -499,12 +497,6 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
       if (compq == 'V')
 	std::cout << "qz: balancing done; QQ=" << std::endl << QQ << std::endl;
 #endif
-
-    if (f77_exception_encountered)
-      {
-	error ("unrecoverable error in qz (bal-L)");
-	return retval;
-      }
   }
 
   // then right
@@ -522,12 +514,6 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
       if (compz == 'V')
 	std::cout << "qz: balancing done; ZZ=" << std::endl << ZZ << std::endl;
 #endif
-
-      if (f77_exception_encountered)
-	{
-	  error ("unrecoverable error in qz (bal-R)");
-	  return retval;
-	}
     }
 
   static char qz_job;
@@ -601,12 +587,6 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
 		 F77_CHAR_ARG_LEN (1)
 		 F77_CHAR_ARG_LEN (1)));
 
-      if (f77_exception_encountered)
-	{
-	  error ("unrecoverable error in qz (dgghrd)");
-	  return retval;
-	}
-
       // check if just computing generalized eigenvalues or if we're
       // actually computing the decomposition
 
@@ -622,16 +602,10 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
 		 F77_CHAR_ARG_LEN (1)
 		 F77_CHAR_ARG_LEN (1)
 		 F77_CHAR_ARG_LEN (1)));
-
-      if (f77_exception_encountered)
-	{
-	  error ("unrecoverable error in qz (dhgeqz)");
-	  return retval;
-	}
     }
 
   // order the QZ decomposition?
-  if (ord_job[0] != 'N')
+  if (! (ord_job == 'N' || ord_job == 'n'))
     {
       if (complex_case)
 	{
@@ -642,20 +616,23 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
       else
 	{
 #ifdef DEBUG_SORT
-	  std::cout << "qz: ordering eigenvalues: ord_job = " << ord_job[0] << std::endl;
+	  std::cout << "qz: ordering eigenvalues: ord_job = "
+		    << ord_job << std::endl;
 #endif
 
 	  // declared static to avoid vfork/long jump compiler complaints
 	  static sort_function sort_test;
-	  sort_test = NULL;
+	  sort_test = 0;
 
-	  switch (ord_job[0])
+	  switch (ord_job)
 	    {
 	    case 'S':
+	    case 's':
 	      sort_test = &fin;
 	      break;
 
 	    case 'B':
+	    case 'b':
 	      sort_test = &fout;
 	      break;
 
@@ -775,7 +752,7 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
 		    }
 #endif
 
-		  // XXX FIXME XXX -- probably should be using
+		  // FIXME -- probably should be using
 		  // fortran_vec instead of &aa(jj,jj) here.
 
 		  double scale1, scale2, wr1, wr2, wi;
@@ -876,7 +853,7 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
     {
       char side = (nargout == 5 ? 'R' : 'B');	// which side to compute?
       char howmny = 'B';  // compute all of them and backtransform
-      octave_idx_type *select = NULL; // dummy pointer; select is not used.
+      octave_idx_type *select = 0; // dummy pointer; select is not used.
       octave_idx_type m;
 
       if (complex_case)
@@ -902,12 +879,6 @@ Note: qz performs permutation balancing, but not scaling (see balance).\n\
 		     F77_CHAR_ARG_LEN (1)
 		     F77_CHAR_ARG_LEN (1)));
 
-	  if (f77_exception_encountered)
-	    {
-	      error ("unrecoverable error in qz (dtgevc)");
-	      return retval;
-	    }
-
 	  // now construct the complex form of VV, WW
 	  int jj = 0;
 
