@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007 David Bateman
+Copyright (C) 2004, 2005, 2006, 2007, 2009 David Bateman
 Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Andy Adler
 
 This file is part of Octave.
@@ -56,13 +56,9 @@ DEFBINOP (div, sparse_complex_matrix, scalar)
   octave_value retval;
 
   if (d == 0.0)
-    {
-      gripe_divide_by_zero ();
+    gripe_divide_by_zero ();
 
-      retval = octave_value (v1.complex_matrix_value () / d);
-    }
-  else
-    retval = octave_value (v1.sparse_complex_matrix_value () / d);
+  retval = octave_value (v1.sparse_complex_matrix_value () / d);
 
   return retval;
 }
@@ -90,7 +86,7 @@ DEFBINOP (ldiv, sparse_complex_matrix, scalar)
       if (d == 0.0)
 	gripe_divide_by_zero ();
 
-      return octave_value (v2.scalar_value () / d);
+      return octave_value (SparseComplexMatrix (1, 1, v2.scalar_value () / d));
     }
   else
     {
@@ -121,13 +117,9 @@ DEFBINOP (el_div, sparse_complex_matrix, scalar)
   octave_value retval;
 
   if (d == 0.0)
-    {
-      gripe_divide_by_zero ();
+    gripe_divide_by_zero ();
 
-      retval = octave_value (v1.complex_matrix_value () / d);
-    }
-  else
-    retval = octave_value (v1.sparse_complex_matrix_value () / d);
+  retval = octave_value (v1.sparse_complex_matrix_value () / d);
 
   return retval;
 }
