@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2003, 2004, 2005, 2006, 2007, 2008 John W. Eaton
+              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -28,6 +28,7 @@ along with Octave; see the file COPYING.  If not, see
 #include <cstdarg>
 #include <cstring>
 
+#include <iostream>
 #include <sstream>
 #include <string>
 
@@ -86,16 +87,7 @@ static std::string Vlast_warning_id;
 static std::string Vlast_error_id;
 
 // The last file in which an error occured
-static std::string Vlast_error_file;
-
-// The last function in which an error occured
-static std::string Vlast_error_name;
-
-// The last line in a function at which an error occured
-static int Vlast_error_line = -1;
-
-// The last column in a function at which an error occured
-static int Vlast_error_column = -1;
+static Octave_map Vlast_error_stack;
 
 // Current error state.
 //
@@ -149,6 +141,26 @@ initialize_warning_options (const std::string& state)
   warning_options.assign ("state", state);
 }
 
+static Octave_map
+initialize_last_error_stack (void)
+{
+  static bool initialized = false;
+
+  static string_vector sv (4);
+
+  if (! initialized)
+    {
+      sv[0] = "file";
+      sv[1] = "name";
+      sv[2] = "line";
+      sv[3] = "column";
+
+      initialized = true;
+    }
+
+  return Octave_map (dim_vector (0, 1), sv);
+}
+
 // Warning messages are never buffered.
 
 static void
@@ -200,75 +212,55 @@ verror (bool save_last_error, std::ostream& os,
   if (! buffer_error_messages)
     flush_octave_stdout ();
 
-  bool to_beep_or_not_to_beep_p = Vbeep_on_error && ! error_state;
-
-  std::ostringstream output_buf;
-
-  if (to_beep_or_not_to_beep_p)
-    output_buf << "\a";
-
-  if (name)
-    output_buf << name << ": ";
-
-  octave_vformat (output_buf, fmt, args);
-
-  output_buf << std::endl;
-
   // FIXME -- we really want to capture the message before it
   // has all the formatting goop attached to it.  We probably also
   // want just the message, not the traceback information.
 
-  std::string msg_string = output_buf.str ();
+  std::ostringstream output_buf;
+
+  octave_vformat (output_buf, fmt, args);
+
+  std::string base_msg = output_buf.str ();
+
+  bool to_beep_or_not_to_beep_p = Vbeep_on_error && ! error_state;
+
+  std::string msg_string;
+
+  if (to_beep_or_not_to_beep_p)
+    msg_string = "\a";
+
+  if (name)
+    msg_string += std::string (name) + ": ";
+
+  msg_string += base_msg + "\n";
 
   if (! error_state && save_last_error)
     {
       // This is the first error in a possible series.
 
       Vlast_error_id = id;
-      Vlast_error_message = msg_string;
+      Vlast_error_message = base_msg;
 
-      Vlast_error_line = -1;
-      Vlast_error_column = -1;
-      Vlast_error_name = std::string ();
-      Vlast_error_file = std::string ();
+      octave_user_code *fcn = octave_call_stack::caller_user_code ();
 
-      if (curr_statement)
+      if (fcn)
 	{
-	  octave_function *fcn
-	    = octave_call_stack::caller_user_script_or_function ();
+	  octave_idx_type curr_frame = -1;
 
-	  if (fcn)
-	    {
-	      Vlast_error_file = fcn->fcn_file_name ();
-	      Vlast_error_name = fcn->name();
-	      Vlast_error_line = curr_statement->line ();
-	      Vlast_error_column = curr_statement->column ();
-	    }
+	  Vlast_error_stack = octave_call_stack::backtrace (0, curr_frame);
 	}
+      else
+	Vlast_error_stack = initialize_last_error_stack ();
     }
 
   if (buffer_error_messages)
     {
-      std::string tmp = msg_string;
+      if (error_message_buffer)
+	msg_string = "error: " + msg_string;
+      else
+	error_message_buffer = new std::ostringstream ();
 
-      if (! error_message_buffer)
-	{
-	  error_message_buffer = new std::ostringstream ();
-
-	  // FIXME -- this is ugly, but it prevents
-	  //
-	  //   eval ("error (\"msg\")", "error (lasterr ())");
-	  //
-	  // from printing `error: ' twice.  Assumes that the NAME we
-	  // have been given doesn't contain `:'.
-
-	  size_t pos = msg_string.find (':');
-
-	  if (pos != NPOS && pos < Vlast_error_message.length () - 2)
-	    tmp = msg_string.substr (pos+2);
-	}
-
-      *error_message_buffer << tmp;
+      *error_message_buffer << msg_string;
     }
   else
     {
@@ -420,67 +412,32 @@ pr_where_1 (const char *fmt, ...)
 }
 
 static void
-pr_where (const char *name, bool print_code = true)
+pr_where (const char *who)
 {
-  if (curr_statement)
+  octave_idx_type curr_frame = -1;
+
+  Octave_map stk = octave_call_stack::backtrace (0, curr_frame);
+
+  octave_idx_type nframes_to_display = stk.numel ();
+
+  if (nframes_to_display > 0)
     {
-      std::string nm;
+      pr_where_1 ("%s: called from\n", who);
 
-      int l = -1;
-      int c = -1;
+      Cell names = stk.contents ("name");
+      Cell lines = stk.contents ("line");
+      Cell columns = stk.contents ("column");
 
-      octave_function *fcn
-	= octave_call_stack::caller_user_script_or_function ();
-
-      if (fcn)
+      for (octave_idx_type i = 0; i < nframes_to_display; i++)
 	{
-	  nm = fcn->fcn_file_name ();
+	  octave_value name = names(i);
+	  octave_value line = lines(i);
+	  octave_value column = columns(i);
 
-	  if (nm.empty ())
-	    nm = fcn->name ();
+	  std::string nm = name.string_value ();
 
-	  if (curr_statement)
-	    {
-	      l = curr_statement->line ();
-	      c = curr_statement->column ();
-	    }
-	}
-
-      if (nm.empty ())
-	{
-	  if (l > 0 && c > 0)
-	    pr_where_1 ("%s: near line %d, column %d:", name, l, c);
-	}
-      else
-	{
-	  if (l > 0 && c > 0)
-	    pr_where_1 ("%s: in %s near line %d, column %d:",
-			name, nm.c_str (), l, c);
-	  else
-	    pr_where_1 ("%s: in %s", name, nm.c_str ());
-	}
-
-      if (print_code)
-	{
-	  // FIXME -- Note that the column number is probably
-	  // not going to mean much here since the code is being
-	  // reproduced from the parse tree, and we are only showing
-	  // one statement even if there were multiple statements on
-	  // the original source line.
-
-	  std::ostringstream output_buf;
-
-	  output_buf << std::endl;
-
-	  tree_print_code tpc (output_buf, ">>> ");
-
-	  curr_statement->accept (tpc);
-
-	  output_buf << std::endl;
-
-	  std::string msg = output_buf.str ();
-
-	  pr_where_1 ("%s", msg.c_str ());
+	  pr_where_1 ("    %s at line %d column %d\n", nm.c_str (),
+		      line.int_value (), column.int_value ());
 	}
     }
 }
@@ -494,15 +451,15 @@ error_2 (const char *id, const char *fmt, va_list args)
 
   if ((interactive || forced_interactive)
       && Vdebug_on_error && init_state == 0
-      && octave_call_stack::caller_user_script_or_function ())
+      && octave_call_stack::caller_user_code ())
     {
       unwind_protect_bool (Vdebug_on_error);
       Vdebug_on_error = false;
 
-      pr_where ("error");
-
       error_state = 0;
 
+      pr_where ("error");
+
       do_keyboard (octave_value_list ());
 
       unwind_protect::run ();
@@ -643,19 +600,19 @@ warning_1 (const char *id, const char *fmt, va_list args)
     }
   else if (warn_opt == 1)
     {
-      if (curr_sym_tab != top_level_sym_tab
+      vwarning ("warning", id, fmt, args);
+
+      if (! symbol_table::at_top_level ()
 	  && Vbacktrace_on_warning
 	  && ! warning_state
 	  && ! discard_warning_messages)
-	pr_where ("warning", false);
-
-      vwarning ("warning", id, fmt, args);
+	pr_where ("warning");
 
       warning_state = 1;
 
       if ((interactive || forced_interactive)
 	  && Vdebug_on_warning
-	  && octave_call_stack::caller_user_script_or_function ())
+	  && octave_call_stack::caller_user_code ())
 	{
 	  unwind_protect_bool (Vdebug_on_warning);
 	  Vdebug_on_warning = false;
@@ -830,10 +787,10 @@ handle_message (error_fun f, const char *id, const char *msg,
 DEFUN (rethrow, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} rethrow (@var{err})\n\
-Reissues a previous error as defined by @var{err}. @var{err} is a structure\n\
-that must contain at least the 'message' and 'identifier' fields. @var{err}\n\
+Reissues a previous error as defined by @var{err}.  @var{err} is a structure\n\
+that must contain at least the 'message' and 'identifier' fields.  @var{err}\n\
 can also contain a field 'stack' that gives information on the assumed\n\
-location of the error. Typically @var{err} is returned from\n\
+location of the error.  Typically @var{err} is returned from\n\
 @code{lasterror}.\n\
 @seealso{lasterror, lasterr, error}\n\
 @end deftypefn")
@@ -854,14 +811,17 @@ location of the error. Typically @var{err} is returned from\n\
 	      std::string msg = err.contents("message")(0).string_value ();
 	      std::string id = err.contents("identifier")(0).string_value ();
 	      int len = msg.length();
+
 	      std::string file;
 	      std::string nm;
 	      int l = -1;
 	      int c = -1;
 
+	      Octave_map err_stack = initialize_last_error_stack ();
+
 	      if (err.contains ("stack"))
 		{
-		  Octave_map err_stack = err.contents("stack")(0).map_value ();
+		  err_stack = err.contents("stack")(0).map_value ();
 
 		  if (err_stack.numel () > 0)
 		    {
@@ -893,11 +853,11 @@ location of the error. Typically @var{err} is returned from\n\
 		rethrow_error (id.c_str (), "%s", tmp_msg);
 	      delete [] tmp_msg;
 
-	      // FIXME: Need to restore the stack as rethrow_error sets it?
-	      Vlast_error_file = file;
-	      Vlast_error_name = nm;
-	      Vlast_error_line = l;
-	      Vlast_error_column = c;
+	      // FIXME -- is this the right thing to do for
+	      // Vlast_error_stack?  Should it be saved and restored
+	      // with unwind_protect?
+
+	      Vlast_error_stack = err_stack;
 
 	      if (err.contains ("stack"))
 		{
@@ -990,17 +950,17 @@ function h () nargin == 1 || error (\"nargin != 1\"); end\n\
 calling the function @code{f} will result in a list of messages that\n\
 can help you to quickly locate the exact location of the error:\n\
 \n\
-@smallexample\n\
+@example\n\
 @group\n\
 f ()\n\
 error: nargin != 1\n\
-error: evaluating index expression near line 1, column 30\n\
-error: evaluating binary operator `||' near line 1, column 27\n\
-error: called from `h'\n\
-error: called from `g'\n\
-error: called from `f'\n\
+error: called from:\n\
+error:   error at line -1, column -1\n\
+error:   h at line 1, column 27\n\
+error:   g at line 1, column 15\n\
+error:   f at line 1, column 15\n\
 @end group\n\
-@end smallexample\n\
+@end example\n\
 \n\
 If the error message ends in a new line character, Octave will print the\n\
 message but will not display any traceback messages as it returns\n\
@@ -1031,7 +991,7 @@ error: nargin != 1\n\
 
       if (! error_state)
 	{
-	  if (arg1.find ('%') == NPOS)
+	  if (arg1.find ('%') == std::string::npos)
 	    {
 	      id = arg1;
 
@@ -1044,13 +1004,42 @@ error: nargin != 1\n\
       else
 	return retval;
     }
+  else if (nargin == 1 && args(0).is_map ())
+    {
+      octave_value_list tmp;
+
+      Octave_map m = args(0).map_value ();
+
+      if (m.numel () == 1)
+	{
+	  if (m.contains ("message"))
+	    {
+	      Cell c = m.contents ("message");
+
+	      if (! c.is_empty () && c(0).is_string ())
+		nargs(0) = c(0).string_value ();
+	    }
+
+	  if (m.contains ("identifier"))
+	    {
+	      Cell c = m.contents ("identifier");
+
+	      if (! c.is_empty () && c(0).is_string ())
+		id = c(0).string_value ();
+	    }
+
+	  // FIXME -- also need to handle "stack" field in error
+	  // structure, but that will require some more significant
+	  // surgery on handle_message, error_with_id, etc.
+	}
+    }
 
   handle_message (error_with_id, id.c_str (), "unspecified error", nargs);
 
   return retval;
 }
 
-DEFCMD (warning, args, nargout,
+DEFUN (warning, args, nargout,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} warning (@var{template}, @dots{})\n\
 @deftypefnx {Built-in Function} {} warning (@var{id}, @var{template}, @dots{})\n\
@@ -1355,7 +1344,7 @@ warning named by @var{id} is handled as if it were an error instead.\n\
 
 	  if (! error_state)
 	    {
-	      if (arg1.find ('%') == NPOS)
+	      if (arg1.find ('%') == std::string::npos)
 		{
 		  id = arg1;
 
@@ -1413,15 +1402,19 @@ initialize_default_warning_state (void)
   disable_warning ("Octave:str-to-num");
   disable_warning ("Octave:string-concat");
   disable_warning ("Octave:variable-switch-label");
+  disable_warning ("Octave:int-convert-nan");
+  disable_warning ("Octave:int-convert-non-int-val");
+  disable_warning ("Octave:int-convert-overflow");
+  disable_warning ("Octave:int-math-overflow");
 }
 
 DEFUN (lasterror, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {@var{err} =} lasterror (@var{err})\n\
 @deftypefnx {Built-in Function} {} lasterror ('reset')\n\
-Returns or sets the last error message. Called without any arguments\n\
+Returns or sets the last error message.  Called without any arguments\n\
 returns a structure containing the last error message, as well as other\n\
-information related to this error. The elements of this structure are:\n\
+information related to this error.  The elements of this structure are:\n\
 \n\
 @table @asis\n\
 @item 'message'\n\
@@ -1429,9 +1422,9 @@ The text of the last error message\n\
 @item 'identifier'\n\
 The message identifier of this error message\n\
 @item 'stack'\n\
-A structure containing information on where the message occurred. This might\n\
-be an empty structure if this in the case where this information can not\n\
-be obtained. The fields of this structure are:\n\
+A structure containing information on where the message occurred.  This might\n\
+be an empty structure if this in the case where this information cannot\n\
+be obtained.  The fields of this structure are:\n\
 \n\
 @table @asis\n\
 @item 'file'\n\
@@ -1446,8 +1439,8 @@ An optional field with the column number at which the error occurred\n\
 @end table\n\
 \n\
 The @var{err} structure may also be passed to @code{lasterror} to set the\n\
-information about the last error. The only constraint on @var{err} in that\n\
-case is that it is a scalar structure. Any fields of @var{err} that match\n\
+information about the last error.  The only constraint on @var{err} in that\n\
+case is that it is a scalar structure.  Any fields of @var{err} that match\n\
 the above are set to the value passed in @var{err}, while other fields are\n\
 set to their default values.\n\
 \n\
@@ -1470,28 +1463,7 @@ their default values.\n\
       err.assign ("message", Vlast_error_message);
       err.assign ("identifier", Vlast_error_id);
 
-      if (! (Vlast_error_file.empty() && Vlast_error_name.empty() &&
-	     Vlast_error_line < 0 && Vlast_error_column < 0))
-	{
-	  Octave_map err_stack;
-
-	  err_stack.assign ("file", Vlast_error_file);
-	  err_stack.assign ("name", Vlast_error_name);
-	  err_stack.assign ("line", Vlast_error_line);
-	  err_stack.assign ("column", Vlast_error_column);
-
-	  err.assign ("stack", octave_value (err_stack));
-	}
-      else
-	{
-	  string_vector sv(4);
-	  sv[0] = "file"; 
-	  sv[1] = "name";
-	  sv[2] = "line";
-	  sv[3] = "column";
-	  err.assign ("stack", octave_value (Octave_map (dim_vector (0,1), 
-							 sv)));
-	}
+      err.assign ("stack", octave_value (Vlast_error_stack));
 
       if (nargin == 1)
 	{
@@ -1501,10 +1473,8 @@ their default values.\n\
 		{
 		  Vlast_error_message = std::string();
 		  Vlast_error_id = std::string();
-		  Vlast_error_file = std::string();
-		  Vlast_error_name = std::string();
-		  Vlast_error_line = -1;
-		  Vlast_error_column = -1;
+
+		  Vlast_error_stack = initialize_last_error_stack ();
 		}
 	      else
 		error("lasterror: unrecognized string argument");
@@ -1571,10 +1541,11 @@ their default values.\n\
 		{
 		  Vlast_error_message = new_error_message;
 		  Vlast_error_id = new_error_id;
-		  Vlast_error_file = new_error_file;
-		  Vlast_error_name = new_error_name;
-		  Vlast_error_line = new_error_line;
-		  Vlast_error_column = new_error_column;
+
+		  octave_idx_type curr_frame = -1;
+
+		  Vlast_error_stack
+		    = octave_call_stack::backtrace (0, curr_frame);
 		}
 	    }
 	  else
@@ -1757,6 +1728,30 @@ to enter the debugger when a warning is encountered.\n\
   return SET_INTERNAL_VARIABLE (debug_on_warning);
 }
 
+std::string
+last_error_message (void)
+{
+  return Vlast_error_message;
+}
+
+std::string
+last_error_id (void)
+{
+  return Vlast_error_id;
+}
+
+std::string
+last_warning_message (void)
+{
+  return Vlast_warning_message;
+}
+
+std::string
+last_warning_id (void)
+{
+  return Vlast_warning_id;
+}
+
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
