@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2003, 2004, 2005, 2006, 2007, 2008 John W. Eaton
+              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -44,6 +44,7 @@ along with Octave; see the file COPYING.  If not, see
 #include <cstdio>
 
 #include <iostream>
+#include <stack>
 #include <vector>
 
 #ifdef HAVE_UNISTD_H
@@ -62,6 +63,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "file-stat.h"
 #include "lo-ieee.h"
 #include "oct-env.h"
+#include "oct-locbuf.h"
 
 #include "defun.h"
 #include "file-io.h"
@@ -74,7 +76,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-stream.h"
 #include "oct-strstrm.h"
 #include "pager.h"
-#include "so-array.h"
 #include "sysdep.h"
 #include "utils.h"
 #include "variables.h"
@@ -150,7 +151,7 @@ fopen_mode_to_ios_mode (const std::string& mode_arg)
 
       size_t pos = mode.find ('W');
 
-      if (pos != NPOS)
+      if (pos != std::string::npos)
 	{
 	  warning ("fopen: treating mode \"W\" as equivalent to \"w\"");
 	  mode[pos] = 'w';
@@ -158,7 +159,7 @@ fopen_mode_to_ios_mode (const std::string& mode_arg)
 
       pos = mode.find ('R');
 
-      if (pos != NPOS)
+      if (pos != std::string::npos)
 	{
 	  warning ("fopen: treating mode \"R\" as equivalent to \"r\"");
 	  mode[pos] = 'r';
@@ -166,7 +167,7 @@ fopen_mode_to_ios_mode (const std::string& mode_arg)
 
       pos = mode.find ('z');
 
-      if (pos != NPOS)
+      if (pos != std::string::npos)
 	{
 #if defined (HAVE_ZLIB)
 	  mode.erase (pos, 1);
@@ -334,7 +335,7 @@ If there are no more characters to read, @code{fgetl} returns @minus{}1.\n\
 
 	  std::string tmp = os.getl (len_arg, err, who);
 
-	  if (! err)
+	  if (! (error_state || err))
 	    {
 	      retval(1) = tmp.length ();
 	      retval(0) = tmp;
@@ -358,7 +359,7 @@ If @var{len} is omitted, @code{fgets} reads until the next newline\n\
 character.\n\
 \n\
 If there are no more characters to read, @code{fgets} returns @minus{}1.\n\
-@seealso{fread, fscanf}\n\
+@seealso{fputs, fopen, fread, fscanf}\n\
 @end deftypefn")
 {
   static std::string who = "fgets";
@@ -382,7 +383,7 @@ If there are no more characters to read, @code{fgets} returns @minus{}1.\n\
 
 	  std::string tmp = os.gets (len_arg, err, who);
 
-	  if (! err)
+	  if (! (error_state || err))
 	    {
 	      retval(1) = tmp.length ();
 	      retval(0) = tmp;
@@ -414,12 +415,12 @@ do_stream_open (const std::string& name, const std::string& mode,
 	{
 	  std::string fname = file_ops::tilde_expand (name);
 
+	  file_stat fs (fname);
+
 	  if (! (md & std::ios::out
 		 || octave_env::absolute_pathname (fname)
 		 || octave_env::rooted_relative_pathname (fname)))
 	    {
-	      file_stat fs (fname);
-
 	      if (! fs.exists ())
 		{
 		  std::string tmp = octave_env::make_absolute
@@ -433,49 +434,52 @@ do_stream_open (const std::string& name, const std::string& mode,
 		    }
 		}
 	    }
+	  
+	  if (! fs.is_dir ())
+	    {
+	      std::string tmode = mode;
 
-	  std::string tmode = mode;
+	      // Use binary mode if 't' is not specified, but don't add
+	      // 'b' if it is already present.
 
-	  // Use binary mode if 't' is not specified, but don't add
-	  // 'b' if it is already present.
+	      size_t bpos = tmode.find ('b');
+	      size_t tpos = tmode.find ('t');
 
-	  size_t bpos = tmode.find ('b');
-	  size_t tpos = tmode.find ('t');
-
-	  if (bpos == NPOS && tpos == NPOS)
-	    tmode += 'b';
+	      if (bpos == std::string::npos && tpos == std::string::npos)
+		tmode += 'b';
 
 #if defined (HAVE_ZLIB)
-	  size_t pos = tmode.find ('z');
+	      size_t pos = tmode.find ('z');
 
-	  if (pos != NPOS)
-	    {
-	      tmode.erase (pos, 1);
+	      if (pos != std::string::npos)
+		{
+		  tmode.erase (pos, 1);
 
-	      gzFile fptr = ::gzopen (fname.c_str (), tmode.c_str ());
+		  gzFile fptr = ::gzopen (fname.c_str (), tmode.c_str ());
 
-	      if (fptr)
-		retval = octave_zstdiostream::create (fname, fptr, md, flt_fmt);
+		  if (fptr)
+		    retval = octave_zstdiostream::create (fname, fptr, md, flt_fmt);
+		  else
+		    {
+		      using namespace std;
+		      retval.error (::strerror (errno));
+		    }
+		}
 	      else
-		{
-		  using namespace std;
-		  retval.error (::strerror (errno));
-		}
-	    }
-	  else
 #endif
-	    {
-	      FILE *fptr = ::fopen (fname.c_str (), tmode.c_str ());
-
-	      retval = octave_stdiostream::create (fname, fptr, md, flt_fmt);
-
-	      if (! fptr)
 		{
-		  using namespace std;
-		  retval.error (::strerror (errno));
-		}
-	    }
+		  FILE *fptr = ::fopen (fname.c_str (), tmode.c_str ());
 
+		  retval = octave_stdiostream::create (fname, fptr, md, flt_fmt);
+
+		  if (! fptr)
+		    {
+		      using namespace std;
+		      retval.error (::strerror (errno));
+		    }
+		}
+
+	    }
 	}
     }
 
@@ -613,7 +617,7 @@ Cray floating format.\n\
 @noindent\n\
 however, conversions are currently only supported for @samp{native}\n\
 @samp{ieee-be}, and @samp{ieee-le} formats.\n\
-@seealso{fclose, fread, fseek}\n\
+@seealso{fclose, fgets, fputs, fread, fseek, ferror, fprintf, fscanf, ftell, fwrite}\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -748,11 +752,11 @@ DEFUN (fseek, args, ,
 Set the file pointer to any location within the file @var{fid}.\n\
 \n\
 The pointer is positioned @var{offset} characters from the @var{origin},\n\
-which may be one of the predefined variables @code{SEEK_CUR} (current\n\
-position), @code{SEEK_SET} (beginning), or @code{SEEK_END} (end of\n\
-file) or strings \"cof\", \"bof\" or \"eof\". If @var{origin} is omitted,\n\
-@code{SEEK_SET} is assumed.  The offset must be zero, or a value returned\n\
-by @code{ftell} (in which case @var{origin} must be @code{SEEK_SET}).\n\
+which may be one of the predefined variables @w{@code{SEEK_CUR}} (current\n\
+position), @w{@code{SEEK_SET}} (beginning), or @w{@code{SEEK_END}} (end of\n\
+file) or strings \"cof\", \"bof\" or \"eof\".  If @var{origin} is omitted,\n\
+@w{@code{SEEK_SET}} is assumed.  The offset must be zero, or a value returned\n\
+by @code{ftell} (in which case @var{origin} must be @w{@code{SEEK_SET}}).\n\
 \n\
 Return 0 on success and -1 on error.\n\
 @seealso{ftell, fopen, fclose}\n\
@@ -810,6 +814,7 @@ DEFUN (fprintf, args, nargout,
 @deftypefn {Built-in Function} {} fprintf (@var{fid}, @var{template}, @dots{})\n\
 This function is just like @code{printf}, except that the output is\n\
 written to the stream @var{fid} instead of @code{stdout}.\n\
+If @var{fid} is omitted, the output is written to @code{stdout}.\n\
 @seealso{printf, sprintf, fread, fscanf, fopen, fclose}\n\
 @end deftypefn")
 {
@@ -921,6 +926,7 @@ DEFUN (fputs, args, ,
 Write a string to a file with no formatting.\n\
 \n\
 Return a non-negative number on success and EOF on error.\n\
+@seealso{scanf, sscanf, fread, fprintf, fgets, fscanf}\n\
 @end deftypefn")
 {
   static std::string who = "fputs";
@@ -1025,7 +1031,7 @@ converted.\n\
 DEFUN (fscanf, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {[@var{val}, @var{count}] =} fscanf (@var{fid}, @var{template}, @var{size})\n\
-@deftypefnx {Built-in Function} {[@var{v1}, @var{v2}, @dots{}, @var{count}] = } fscanf (@var{fid}, @var{template}, \"C\")\n\
+@deftypefnx {Built-in Function} {[@var{v1}, @var{v2}, @dots{}, @var{count}] =} fscanf (@var{fid}, @var{template}, \"C\")\n\
 In the first form, read from @var{fid} according to @var{template},\n\
 returning the result in the matrix @var{val}.\n\
 \n\
@@ -1068,7 +1074,7 @@ conversions is returned in @var{count}\n\
 See the Formatted Input section of the GNU Octave manual for a\n\
 complete description of the syntax of the template string.\n\
 @end ifclear\n\
-@seealso{scanf, sscanf, fread, fprintf}\n\
+@seealso{scanf, sscanf, fread, fprintf, fgets, fputs}\n\
 @end deftypefn")
 {
   static std::string who = "fscanf";
@@ -1112,8 +1118,11 @@ complete description of the syntax of the template string.\n\
 		    {
 		      octave_value tmp = os.scanf (args(1), size, count, who);
 
-		      retval(1) = count;
-		      retval(0) = tmp;
+		      if (! error_state)
+			{
+			  retval(1) = count;
+			  retval(0) = tmp;
+			}
 		    }
 		}
 	      else
@@ -1130,7 +1139,7 @@ complete description of the syntax of the template string.\n\
 DEFUN (sscanf, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {[@var{val}, @var{count}] =} sscanf (@var{string}, @var{template}, @var{size})\n\
-@deftypefnx {Built-in Function} {[@var{v1}, @var{v2}, @dots{}, @var{count}] = } sscanf (@var{string}, @var{template}, \"C\")\n\
+@deftypefnx {Built-in Function} {[@var{v1}, @var{v2}, @dots{}, @var{count}] =} sscanf (@var{string}, @var{template}, \"C\")\n\
 This is like @code{fscanf}, except that the characters are taken from the\n\
 string @var{string} instead of from a stream.  Reaching the end of the\n\
 string is treated as an end-of-file condition.\n\
@@ -1192,15 +1201,18 @@ string is treated as an end-of-file condition.\n\
 
 		      octave_value tmp = os.scanf (args(1), size, count, who);
 
-		      // FIXME -- is this the right thing to do?
-		      // Extract error message first, because getting
-		      // position will clear it.
-		      std::string errmsg = os.error ();
+		      if (! error_state)
+			{
+			  // FIXME -- is this the right thing to do?
+			  // Extract error message first, because getting
+			  // position will clear it.
+			  std::string errmsg = os.error ();
 
-		      retval(3) = os.tell () + 1;
-		      retval(2) = errmsg;
-		      retval(1) = count;
-		      retval(0) = tmp;
+			  retval(3) = os.tell () + 1;
+			  retval(2) = errmsg;
+			  retval(1) = count;
+			  retval(0) = tmp;
+			}
 		    }
 		  else
 		    ::error ("%s: format must be a string", who.c_str ());
@@ -1222,7 +1234,7 @@ string is treated as an end-of-file condition.\n\
 DEFUN (scanf, args, nargout,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {[@var{val}, @var{count}] =} scanf (@var{template}, @var{size})\n\
-@deftypefnx {Built-in Function} {[@var{v1}, @var{v2}, @dots{}, @var{count}]] = } scanf (@var{template}, \"C\")\n\
+@deftypefnx {Built-in Function} {[@var{v1}, @var{v2}, @dots{}, @var{count}]] =} scanf (@var{template}, \"C\")\n\
 This is equivalent to calling @code{fscanf} with @var{fid} = @code{stdin}.\n\
 \n\
 It is currently not useful to call @code{scanf} in interactive\n\
@@ -1730,7 +1742,7 @@ For example,\n\
 @example\n\
 @group\n\
 fid = popen (\"ls -ltr / | tail -3\", \"r\");\n\
-while (isstr (s = fgets (fid)))\n\
+while (ischar (s = fgets (fid)))\n\
   fputs (stdout, s);\n\
 endwhile\n\
      @print{} drwxr-xr-x  33 root  root  3072 Feb 15 13:28 etc\n\
@@ -1895,6 +1907,29 @@ system-dependent error message.\n\
   return retval;
 }
 
+#if defined (HAVE_MKSTEMPS)
+// Prototype for mkstemps in libiberty
+extern "C" int mkstemps (char *pattern, int suffix_len);
+#endif
+
+#if ! defined (HAVE_MKSTEMP) && ! defined (HAVE_MKSTEMPS) && defined (_MSC_VER)
+# if defined (HAVE_FCNTL_H)
+#  include <fcntl.h>
+# endif
+# if defined (HAVE_SYS_STAT_H)
+#  include <sys/stat.h>
+# endif
+int mkstemp (char *tmpl)
+{
+  int ret=-1;
+  mktemp (tmpl);
+  ret = open (tmpl, O_RDWR | O_BINARY | O_CREAT | O_EXCL | _O_SHORT_LIVED,
+	      _S_IREAD | _S_IWRITE);
+  return ret;
+}
+#define HAVE_MKSTEMP 1
+#endif
+
 DEFUN (mkstemp, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {[@var{fid}, @var{name}, @var{msg}] =} mkstemp (@var{template}, @var{delete})\n\
@@ -1904,7 +1939,7 @@ must be @code{XXXXXX} and these are replaced with a string that makes the\n\
 filename unique.  The file is then created with mode read/write and\n\
 permissions that are system dependent (on GNU/Linux systems, the permissions\n\
 will be 0600 for versions of glibc 2.0.7 and later).  The file is opened\n\
-with the @code{O_EXCL} flag.\n\
+with the @w{@code{O_EXCL}} flag.\n\
 \n\
 If the optional argument @var{delete} is supplied and is true,\n\
 the file will be deleted automatically when Octave exits, or when\n\
@@ -1923,7 +1958,7 @@ error message.\n\
   retval(1) = std::string ();
   retval(0) = -1;
 
-#if defined (HAVE_MKSTEMP)
+#if defined (HAVE_MKSTEMP) || defined (HAVE_MKSTEMPS)
 
   int nargin = args.length ();
 
@@ -1936,7 +1971,11 @@ error message.\n\
 	  OCTAVE_LOCAL_BUFFER (char, tmp, tmpl8.size () + 1);
 	  strcpy (tmp, tmpl8.c_str ());
 
+#if defined (HAVE_MKSTEMP)
 	  int fd = mkstemp (tmp);
+#else
+	  int fd = mkstemps (tmp, 0);
+#endif
 
 	  if (fd < 0)
 	    {
