@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-              2003, 2004, 2005, 2006, 2007 John W. Eaton
+              2003, 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -39,10 +39,6 @@ along with Octave; see the file COPYING.  If not, see
 #include <sstream>
 #include <string>
 
-#ifdef HAVE_HDF5
-#include <hdf5.h>
-#endif
-
 #include "byte-swap.h"
 #include "data-conv.h"
 #include "file-ops.h"
@@ -54,6 +50,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-time.h"
 #include "quit.h"
 #include "str-vec.h"
+#include "oct-locbuf.h"
 
 #include "Cell.h"
 #include "defun.h"
@@ -74,9 +71,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "version.h"
 #include "dMatrix.h"
 
-#ifdef HAVE_HDF5
 #include "ls-hdf5.h"
-#endif
 #include "ls-mat-ascii.h"
 #include "ls-mat4.h"
 #include "ls-mat5.h"
@@ -132,149 +127,22 @@ gripe_file_open (const std::string& fcn, const std::string& file)
     error ("%s: unable to open file `%s'", fcn.c_str (), file.c_str ());
 }
 
-// FIXME -- shouldn't this be implemented in terms of other
-// functions that are already available?
-
-// Install a variable with name NAME and the value specified TC in the
-// symbol table.  If FORCE is TRUE, replace any existing definition
-// for NAME.  If GLOBAL is TRUE, make the variable global.
-//
-// Assumes TC is defined.
+// Install a variable with name NAME and the value VAL in the
+// symbol table.  If GLOBAL is TRUE, make the variable global.
 
 static void
-install_loaded_variable (int force, const std::string& name,
+install_loaded_variable (const std::string& name,
 			 const octave_value& val,
-			 int global, const std::string& doc)
+			 bool global, const std::string& /*doc*/)
 {
-  // Is there already a symbol by this name?  If so, what is it?
-
-  symbol_record *lsr = curr_sym_tab->lookup (name);
-
-  bool is_undefined = true;
-  bool is_variable = false;
-  bool is_function = false;
-  bool is_global = false;
-
-  if (lsr)
-    {
-      is_undefined = ! lsr->is_defined ();
-      is_variable = lsr->is_variable ();
-      is_function = lsr->is_function ();
-      is_global = lsr->is_linked_to_global ();
-    }
-
-  symbol_record *sr = 0;
-
   if (global)
     {
-      if (is_global || is_undefined)
-	{
-	  if (force || is_undefined)
-	    {
-	      lsr = curr_sym_tab->lookup (name, true);
-	      link_to_global_variable (lsr);
-	      sr = lsr;
-	    }
-	  else
-	    {
-	      warning ("load: global variable name `%s' exists",
-		       name.c_str ());
-	      warning ("use `load -force' to overwrite");
-	    }
-	}
-      else if (is_function)
-	{
-	  if (force)
-	    {
-	      lsr = curr_sym_tab->lookup (name, true);
-	      link_to_global_variable (lsr);
-	      sr = lsr;
-	    }
-	  else
-	    {
-	      warning ("load: `%s' is currently a function in this scope",
-		       name.c_str ());
-	      warning ("`load -force' will load variable and hide function");
-	    }
-	}
-      else if (is_variable)
-	{
-	  if (force)
-	    {
-	      lsr = curr_sym_tab->lookup (name, true);
-	      link_to_global_variable (lsr);
-	      sr = lsr;
-	    }
-	  else
-	    {
-	      warning ("load: local variable name `%s' exists",
-		       name.c_str ());
-	      warning ("use `load -force' to overwrite");
-	    }
-	}
-      else
-	error ("load: unable to load data for unknown symbol type");
+      symbol_table::clear (name);
+      symbol_table::mark_global (name);
+      symbol_table::global_varref (name) = val;
     }
   else
-    {
-      if (is_global)
-	{
-	  if (force || is_undefined)
-	    {
-	      lsr = curr_sym_tab->lookup (name, true);
-	      link_to_global_variable (lsr);
-	      sr = lsr;
-	    }
-	  else
-	    {
-	      warning ("load: global variable name `%s' exists",
-		       name.c_str ());
-	      warning ("use `load -force' to overwrite");
-	    }
-	}
-      else if (is_function)
-	{
-	  if (force)
-	    {
-	      lsr = curr_sym_tab->lookup (name, true);
-	      link_to_global_variable (lsr);
-	      sr = lsr;
-	    }
-	  else
-	    {
-	      warning ("load: `%s' is currently a function in this scope",
-		       name.c_str ());
-	      warning ("`load -force' will load variable and hide function");
-	    }
-	}
-      else if (is_variable || is_undefined)
-	{
-	  if (force || is_undefined)
-	    {
-	      lsr = curr_sym_tab->lookup (name, true);
-	      sr = lsr;
-	    }
-	  else
-	    {
-	      warning ("load: local variable name `%s' exists",
-		       name.c_str ());
-	      warning ("use `load -force' to overwrite");
-	    }
-	}
-      else
-	error ("load: unable to load data for unknown symbol type");
-    }
-
-  if (sr)
-    {
-      sr->define (val);
-      sr->document (doc);
-      return;
-    }
-  else
-    error ("load: unable to load variable `%s'", name.c_str ());
-
-  return;
+    symbol_table::varref (name) = val;
 }
 
 // Return TRUE if NAME matches one of the given globbing PATTERNS.
@@ -451,7 +319,7 @@ get_file_format (const std::string& fname, const std::string& orig_fname,
 }
 
 octave_value
-do_load (std::istream& stream, const std::string& orig_fname, bool force,
+do_load (std::istream& stream, const std::string& orig_fname,
 	 load_save_format format, oct_mach_info::float_format flt_fmt,
 	 bool list_only, bool swap, bool verbose,
 	 const string_vector& argv, int argv_idx, int argc, int nargout)
@@ -461,6 +329,7 @@ do_load (std::istream& stream, const std::string& orig_fname, bool force,
   Octave_map retstruct;
 
   std::ostringstream output_buf;
+  std::list<std::string> symbol_names;
 
   octave_idx_type count = 0;
 
@@ -472,7 +341,7 @@ do_load (std::istream& stream, const std::string& orig_fname, bool force,
       std::string name;
       std::string doc;
 
-      switch (format)
+      switch (format.type)
 	{
 	case LS_ASCII:
 	  name = read_ascii_data (stream, orig_fname, global, tc, count);
@@ -538,9 +407,10 @@ do_load (std::istream& stream, const std::string& orig_fname, bool force,
 			    << std::setiosflags (std::ios::right)
 			    << std::setw (7) << tc.rows ()
 			    << std::setw (7) << tc.columns ()
-			    << "   ";
+			    << "   " << name << "\n";
 			}
-		      output_buf << name << "\n";
+		      else
+			symbol_names.push_back (name);
 		    }
 		  else
 		    {
@@ -552,7 +422,7 @@ do_load (std::istream& stream, const std::string& orig_fname, bool force,
 			    retstruct.assign (name, tc);
 			}
 		      else
-			install_loaded_variable (force, name, tc, global, doc);
+			install_loaded_variable (name, tc, global, doc);
 		    }
 		}
 
@@ -576,12 +446,28 @@ do_load (std::istream& stream, const std::string& orig_fname, bool force,
 
   if (list_only && count)
     {
-      std::string msg = output_buf.str ();
+      if (verbose)
+	{
+	  std::string msg = output_buf.str ();
 
-      if (nargout > 0)
-	retval = msg;
+	  if (nargout > 0)
+	    retval = msg;
+	  else
+	    octave_stdout << msg;
+	}
       else
-	octave_stdout << msg;
+	{
+	  if (nargout  > 0)
+	    retval = Cell (string_vector (symbol_names));
+	  else
+	    {
+	      string_vector names (symbol_names);
+
+	      names.list_in_columns (octave_stdout);
+
+	      octave_stdout << "\n";
+	    }
+	}
     }
   else if (retstruct.nfields () != 0)
     retval = retstruct;
@@ -614,9 +500,10 @@ find_file_to_load (const std::string& name, const std::string& orig_name)
     }
 
   size_t dot_pos = fname.rfind (".");
-  size_t sep_pos = fname.find_last_of (file_ops::dir_sep_chars);
+  size_t sep_pos = fname.find_last_of (file_ops::dir_sep_chars ());
     
-  if (dot_pos == NPOS || (sep_pos != NPOS && dot_pos < sep_pos))
+  if (dot_pos == std::string::npos
+      || (sep_pos != std::string::npos && dot_pos < sep_pos))
     {
       // Either no '.' in name or no '.' appears after last directory
       // separator.
@@ -655,7 +542,7 @@ HDF5 load and save are not available, as this Octave executable was\n\
 not linked with the HDF5 library."
 #endif /* ! HAVE HDF5 */
 
-DEFCMD (load, args, nargout,
+DEFUN (load, args, nargout,
   "-*- texinfo -*-\n\
 @deffn {Command} load options file v1 v2 @dots{}\n\
 Load the named variables @var{v1}, @var{v2}, @dots{}, from the file\n\
@@ -693,7 +580,8 @@ returned.  Otherwise, @code{load} returns a structure with members\n\
  corresponding to the names of the variables in the file.\n\
 \n\
 The @code{load} command can read data stored in Octave's text and\n\
-binary formats, and @sc{Matlab}'s binary format.  It will automatically\n\
+binary formats, and @sc{matlab}'s binary format.  If compiled with zlib\n\
+support, it can load gzip-compressed files as well.  It will automatically\n\
 detect the type of file and do conversion from different floating point\n\
 formats (currently only IEEE big and little endian, though other formats\n\
 may added in the future).\n\
@@ -703,7 +591,7 @@ Valid options for @code{load} are listed in the following table.\n\
 @table @code\n\
 @item -force\n\
 The @samp{-force} option is accepted but ignored for backward\n\
-compatibility. Octave now overwrites variables currently in memory with\n\
+compatibility.  Octave now overwrites variables currently in memory with\n\
 the same name as those found in the file.\n\
 \n\
 @item -ascii\n\
@@ -721,7 +609,7 @@ Force Octave to assume the file is in Octave's binary format.\n\
 @itemx -v6\n\
 @itemx -7\n\
 @itemx -v7\n\
-Force Octave to assume the file is in @sc{Matlab}'s version 6 or 7 binary\n\
+Force Octave to assume the file is in @sc{matlab}'s version 6 or 7 binary\n\
 format.\n\
 \n\
 @item -V4\n\
@@ -729,7 +617,7 @@ format.\n\
 @itemx -4\n\
 @itemx -mat4-binary\n\
 Force Octave to assume the file is in the binary format written by\n\
-@sc{Matlab} version 4.\n\
+@sc{matlab} version 4.\n\
 \n\
 @item -hdf5\n\
 Force Octave to assume the file is in HDF5 format.\n\
@@ -766,7 +654,6 @@ Force Octave to assume the file is in Octave's text format.\n\
 
   load_save_format format = LS_UNKNOWN;
 
-  bool force = true;
   bool list_only = false;
   bool verbose = false;
 
@@ -857,7 +744,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 	  // that we can get additional input?  I'm afraid that we
 	  // can't fix this using std::cin only.
 
-	  retval = do_load (std::cin, orig_fname, force, format, flt_fmt,
+	  retval = do_load (std::cin, orig_fname, format, flt_fmt,
 			    list_only, swap, verbose, argv, i, argc,
 			    nargout);
 	}
@@ -887,7 +774,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 
 	  if (hdf5_file.file_id >= 0)
 	    {
-	      retval = do_load (hdf5_file, orig_fname, force, format,
+	      retval = do_load (hdf5_file, orig_fname, format,
 				flt_fmt, list_only, swap, verbose,
 				argv, i, argc, nargout);
 
@@ -904,14 +791,10 @@ Force Octave to assume the file is in Octave's text format.\n\
 	{
 	  i++;
 
-	  std::ios::openmode mode = std::ios::in;
+	  // Always open in binary mode and handle various
+	  // line-endings explicitly.
+	  std::ios::openmode mode = std::ios::in | std::ios::binary;
 
-	  // Open in binary mode in any case, to fix annoying bug that
-	  // text-mode opened streams cannot be seekg'ed/tellg'ed with
-	  // mingw32 (See http://oldwiki.mingw.org/index.php/Known%20Problems )
-	  // The CR/LF issues are handled in ls-ascii-helper.cc
-	  mode |= std::ios::binary;
-	  
 #ifdef HAVE_ZLIB
 	  if (use_zlib)
 	    {
@@ -937,7 +820,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 			}
 		    }
 
-		  retval = do_load (file, orig_fname, force, format,
+		  retval = do_load (file, orig_fname, format,
 				    flt_fmt, list_only, swap, verbose,
 				argv, i, argc, nargout);
 
@@ -971,7 +854,7 @@ Force Octave to assume the file is in Octave's text format.\n\
 			}
 		    }
 
-		  retval = do_load (file, orig_fname, force, format,
+		  retval = do_load (file, orig_fname, format,
 				    flt_fmt, list_only, swap, verbose,
 				    argv, i, argc, nargout);
 
@@ -1030,9 +913,9 @@ glob_pattern_p (const std::string& pattern)
 static void
 do_save (std::ostream& os, const octave_value& tc,
 	 const std::string& name, const std::string& help,
-	 int global, load_save_format fmt, bool save_as_floats)
+	 bool global, load_save_format fmt, bool save_as_floats)
 {
-  switch (fmt)
+  switch (fmt.type)
     {
     case LS_ASCII:
       save_ascii_data (os, tc, name, global, 0);
@@ -1043,8 +926,8 @@ do_save (std::ostream& os, const octave_value& tc,
       break;
 
     case LS_MAT_ASCII:
-    case LS_MAT_ASCII_LONG:
-      if (! save_mat_ascii_data (os, tc, fmt == LS_MAT_ASCII ? 8 : 16))
+      if (! save_mat_ascii_data (os, tc, fmt.opts & LS_MAT_ASCII_LONG ? 16 : 8, 
+                                 fmt.opts & LS_MAT_ASCII_TABS))
 	warning ("save: unable to save %s in ASCII format", name.c_str ());
       break;
 
@@ -1075,46 +958,70 @@ do_save (std::ostream& os, const octave_value& tc,
 // Save the info from SR on stream OS in the format specified by FMT.
 
 void
-do_save (std::ostream& os, symbol_record *sr, load_save_format fmt,
-	 bool save_as_floats)
+do_save (std::ostream& os, const symbol_table::symbol_record& sr,
+	 load_save_format fmt, bool save_as_floats)
 {
-  if (! sr->is_variable ())
+  octave_value val = sr.varval ();
+
+  if (val.is_defined ())
     {
-      error ("save: can only save variables, not functions");
-      return;
+      std::string name = sr.name ();
+      std::string help;
+      bool global = sr.is_global ();
+
+      do_save (os, val, name, help, global, fmt, save_as_floats);
+    }
+}
+
+// save fields of a scalar structure STR matching PATTERN on stream OS
+// in the format specified by FMT.
+
+static size_t
+save_fields (std::ostream& os, const Octave_map& m,
+	     const std::string& pattern,
+	     load_save_format fmt, bool save_as_floats)
+{
+  glob_match pat (pattern);
+  
+  size_t saved = 0;
+
+  for (Octave_map::const_iterator p = m.begin (); p != m.end (); p++)
+    {
+      std::string empty_str;
+
+      if (pat.match(p->first))
+        {
+          do_save (os, p->second(0), p->first, empty_str,
+		   0, fmt, save_as_floats);
+
+          saved++;
+        }
     }
 
-  octave_value tc = sr->def ();
-
-  if (tc.is_defined ())
-    {
-      std::string name = sr->name ();
-      std::string help = sr->help ();
-
-      int global = sr->is_linked_to_global ();
-
-      do_save (os, tc, name, help, global, fmt, save_as_floats);
-    }
+  return saved;
 }
 
 // Save variables with names matching PATTERN on stream OS in the
 // format specified by FMT.
 
-static int
+static size_t
 save_vars (std::ostream& os, const std::string& pattern,
 	   load_save_format fmt, bool save_as_floats)
 {
-  Array<symbol_record *> vars = curr_sym_tab->glob
-    (pattern, symbol_record::USER_VARIABLE, SYMTAB_ALL_SCOPES);
+  std::list<symbol_table::symbol_record> vars = symbol_table::glob (pattern);
 
-  int saved = vars.length ();
+  size_t saved = 0;
 
-  for (int i = 0; i < saved; i++)
+  typedef std::list<symbol_table::symbol_record>::const_iterator const_vars_iterator;
+
+  for (const_vars_iterator p = vars.begin (); p != vars.end (); p++)
     {
-      do_save (os, vars(i), fmt, save_as_floats);
+      do_save (os, *p, fmt, save_as_floats);
 
       if (error_state)
 	break;
+
+      saved++;
     }
 
   return saved;
@@ -1128,6 +1035,8 @@ parse_save_options (const string_vector &argv,
   string_vector retval;
   int argc = argv.length ();
 
+  bool do_double = false, do_tabs = false;
+
   for (int i = 0; i < argc; i++)
     {
       if (argv[i] == "-append")
@@ -1138,6 +1047,14 @@ parse_save_options (const string_vector &argv,
 	{
 	  format = LS_MAT_ASCII;
 	}
+      else if (argv[i] == "-double")
+	{
+	  do_double = true;
+	}
+      else if (argv[i] == "-tabs")
+	{
+	  do_tabs = true;
+	}
       else if (argv[i] == "-text" || argv[i] == "-t")
 	{
 	  format = LS_ASCII;
@@ -1196,6 +1113,22 @@ parse_save_options (const string_vector &argv,
         retval.append (argv[i]);
     }
 
+  if (do_double)
+    {
+      if (format == LS_MAT_ASCII)
+	format.opts |= LS_MAT_ASCII_LONG;
+      else
+	warning ("save: \"-double\" option only has an effect with \"-ascii\"");
+    }
+
+  if (do_tabs)
+    {
+      if (format == LS_MAT_ASCII)
+	format.opts |= LS_MAT_ASCII_TABS;
+      else
+	warning ("save: \"-tabs\" option only has an effect with \"-ascii\"");
+    }
+
   return retval;
 }
 
@@ -1221,7 +1154,7 @@ parse_save_options (const std::string &arg, load_save_format &format,
 void
 write_header (std::ostream& os, load_save_format format)
 {
-  switch (format)
+  switch (format.type)
     {
     case LS_BINARY:
       {
@@ -1312,14 +1245,55 @@ save_vars (const string_vector& argv, int argv_idx, int argc,
     {
       save_vars (os, "*", fmt, save_as_floats);
     }
+  else if (argv[argv_idx] == "-struct")
+    {
+      if (++argv_idx >= argc) 
+        {
+          error ("save: missing struct name");
+          return;
+        }
+
+      std::string struct_name = argv[argv_idx];
+
+      if (! symbol_table::is_variable (struct_name))
+        {
+          error ("save: no such variable: `%s'", struct_name.c_str ());
+          return;
+        }
+
+      octave_value struct_var = symbol_table::varref (struct_name);
+
+      if (! struct_var.is_map () || struct_var.numel () != 1) 
+        {
+          error ("save: `%s' is not a scalar structure",
+		 struct_name.c_str ());
+          return;
+        }
+      Octave_map struct_var_map = struct_var.map_value ();
+
+      ++argv_idx;
+
+      if (argv_idx < argc) 
+        {
+          for (int i = argv_idx; i < argc; i++)
+            {
+              if (! save_fields (os, struct_var_map, argv[i], fmt,
+				 save_as_floats))
+                {
+                  warning ("save: no such field `%s.%s'", 
+			   struct_name.c_str (), argv[i].c_str ());
+                }
+            }
+        }
+      else
+	save_fields (os, struct_var_map, "*", fmt, save_as_floats);
+    }
   else
     {
       for (int i = argv_idx; i < argc; i++)
 	{
 	  if (! save_vars (os, argv[i], fmt, save_as_floats))
-	    {
-	      warning ("save: no such variable `%s'", argv[i].c_str ());
-	    }
+	    warning ("save: no such variable `%s'", argv[i].c_str ());
 	}
     }
 }
@@ -1330,43 +1304,36 @@ dump_octave_core (std::ostream& os, const char *fname, load_save_format fmt,
 {
   write_header (os, fmt);
 
-  Array<symbol_record *> vars = curr_sym_tab->glob
-    ("*", symbol_record::USER_VARIABLE, SYMTAB_ALL_SCOPES);
-
-  int num_to_save = vars.length ();
+  std::list<symbol_table::symbol_record> vars
+    = symbol_table::all_variables (symbol_table::top_scope (), 0);
 
   double save_mem_size = 0;
 
-  for (int i = 0; i < num_to_save; i++)
+  typedef std::list<symbol_table::symbol_record>::const_iterator const_vars_iterator;
+
+  for (const_vars_iterator p = vars.begin (); p != vars.end (); p++)
     {
-      symbol_record *sr = vars(i);
+      octave_value val = p->varval ();
 
-      if (sr->is_variable ())
+      if (val.is_defined ())
 	{
-	  octave_value tc = sr->def ();
+	  std::string name = p->name ();
+	  std::string help;
+	  bool global = p->is_global ();
 
-	  if (tc.is_defined ())
+	  double val_size = val.byte_size () / 1024;
+
+	  // FIXME -- maybe we should try to throw out the largest first...
+
+	  if (Voctave_core_file_limit < 0
+	      || save_mem_size + val_size < Voctave_core_file_limit)
 	    {
-	      double tc_size = tc.byte_size () / 1024;
+	      save_mem_size += val_size;
 
-	      // FIXME -- maybe we should try to throw out the
-	      // largest first...
+	      do_save (os, val, name, help, global, fmt, save_as_floats);
 
-	      if (Voctave_core_file_limit < 0
-		  || save_mem_size + tc_size < Voctave_core_file_limit)
-		{
-		  save_mem_size += tc_size;
-
-		  std::string name = sr->name ();
-		  std::string help = sr->help ();
-
-		  int global = sr->is_linked_to_global ();
-
-		  do_save (os, tc, name, help, global, fmt, save_as_floats);
-
-		  if (error_state)
-		    break;
-		}
+	      if (error_state)
+		break;
 	    }
 	}
     }
@@ -1472,15 +1439,19 @@ This option is not available, as this Octave executable was not linked with\n\
 the zlib library."
 #endif /* ! HAVE ZLIB */
 
-DEFCMD (save, args, ,
+DEFUN (save, args, ,
   "-*- texinfo -*-\n\
 @deffn {Command} save options file @var{v1} @var{v2} @dots{}\n\
+@deffnx {Command} save options file -struct @var{STR} @var{f1} @var{f2} @dots{}\n\
 Save the named variables @var{v1}, @var{v2}, @dots{}, in the file\n\
-@var{file}.  The special filename @samp{-} can be used to write the\n\
+@var{file}.  The special filename @samp{-} may be used to write the\n\
 output to your terminal.  If no variable names are listed, Octave saves\n\
-all the variables in the current scope.  Valid options for the\n\
-@code{save} command are listed in the following table.  Options that\n\
-modify the output format override the format specified by\n\
+all the variables in the current scope.\n\
+If the @code{-struct} modifier is used, fields @var{f1} @var{f2} @dots{}\n\
+of the scalar structure @var{STR} are saved as if they were variables\n\
+with corresponding names.\n\
+Valid options for the @code{save} command are listed in the following table.\n\
+Options that modify the output format override the format specified by \n\
 @code{default_save_options}.\n\
 \n\
 If save is invoked using the functional form\n\
@@ -1509,7 +1480,7 @@ values to be saved can be represented in single precision.\n\
 @itemx -v7\n\
 @itemx -7\n\
 @itemx -mat7-binary\n\
-Save the data in @sc{Matlab}'s v7 binary data format.\n"
+Save the data in @sc{matlab}'s v7 binary data format.\n"
 
 HAVE_ZLIB_HELP_STRING
 
@@ -1519,13 +1490,13 @@ HAVE_ZLIB_HELP_STRING
 @itemx -6\n\
 @itemx -mat\n\
 @itemx -mat-binary\n\
-Save the data in @sc{Matlab}'s v6 binary data format.\n\
+Save the data in @sc{matlab}'s v6 binary data format.\n\
 \n\
 @item -V4\n\
 @itemx -v4\n\
 @itemx -4\n\
 @itemx -mat4-binary\n\
-Save the data in the binary format written by @sc{Matlab} version 4.\n\
+Save the data in the binary format written by @sc{matlab} version 4.\n\
 \n\
 @item -hdf5\n\
 Save the data in HDF5 format.\n\
@@ -1542,7 +1513,7 @@ values to be saved can be represented in single precision.\n\
 \n\
 @item -zip\n\
 @itemx -z\n\
-Use the gzip algorithm to compress the file. This works equally on files that\n\
+Use the gzip algorithm to compress the file.  This works equally on files that\n\
 are compressed with gzip outside of octave, and gzip can equally be used to\n\
 convert the files for backward compatibility.\n"
 
@@ -1563,13 +1534,16 @@ Match zero or more characters.\n\
 Match the list of characters specified by @var{list}.  If the first\n\
 character is @code{!} or @code{^}, match all characters except those\n\
 specified by @var{list}.  For example, the pattern @samp{[a-zA-Z]} will\n\
-match all lower and upper case alphabetic characters. \n\
+match all lower and upper case alphabetic characters.  \n\
+\n\
+Wildcards may also be used in the field names specifications when using\n\
+the @code{-struct} modifier (but not in the struct name itself).\n\
 \n\
 @item -text\n\
 Save the data in Octave's text data format.\n\
 @end table\n\
 \n\
-Except when using the @sc{Matlab} binary data file format, saving global\n\
+Except when using the @sc{matlab} binary data file format, saving global\n\
 variables also saves the global status of the variable, so that if it is\n\
 restored at a later time using @samp{load}, it will be restored as a\n\
 global variable.\n\
@@ -1826,7 +1800,7 @@ DEFUN (octave_core_file_options, args, nargout,
 Query or set the internal variable that specifies the options used for\n\
 saving the workspace data if Octave aborts.  The value of\n\
 @code{octave_core_file_options} should follow the same format as the\n\
-options for the @code{save} function. The default value is Octave's binary\n\
+options for the @code{save} function.  The default value is Octave's binary\n\
 format.\n\
 @seealso{crash_dumps_octave_core, octave_core_file_name, octave_core_file_limit}\n\
 @end deftypefn")
@@ -1847,6 +1821,7 @@ passed to @code{strftime} and should begin with the character\n\
 the header comment is omitted from text-format data files.  The\n\
 default value is\n\
 \n\
+@c Set example in small font to prevent overfull line\n\
 @smallexample\n\
 \"# Created by Octave VERSION, %a %b %d %H:%M:%S %Y %Z <USER@@HOST>\"\n\
 @end smallexample\n\
