@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2003, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 2009 Benjamin Lindner
 
 This file is part of Octave.
 
@@ -26,15 +26,19 @@ along with Octave; see the file COPYING.  If not, see
 #include <iosfwd>
 #include <string>
 
-#include "oct-dlldefs.h"
+extern OCTINTERP_API void
+skip_until_newline (std::istream& is, bool keep_newline = false);
 
 extern OCTINTERP_API void
-skip_until_newline( std::istream& is, bool keep_newline = false );
-
-extern OCTINTERP_API void
-skip_preceeding_newline( std::istream& is );
+skip_preceeding_newline (std::istream& is);
 
 extern OCTINTERP_API std::string
-read_until_newline( std::istream& is, bool keep_newline = false );
+read_until_newline (std::istream& is, bool keep_newline = false);
 
-#endif  // !defined (octave_ls_ascii_helper_h)
+#endif
+
+/*
+;;; Local Variables: ***
+;;; mode: C++ ***
+;;; End: ***
+*/
