@@ -1,6 +1,7 @@
 /*
 
-Copyright (C) 1996, 1997, 2003, 2004, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 1996, 1997, 2003, 2004, 2005, 2006, 2007, 2008,
+              2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -38,8 +39,6 @@ along with Octave; see the file COPYING.  If not, see
 #include <string>
 #include <vector>
 
-#include <hdf5.h>
-
 #include "byte-swap.h"
 #include "data-conv.h"
 #include "file-ops.h"
@@ -50,6 +49,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-time.h"
 #include "quit.h"
 #include "str-vec.h"
+#include "oct-locbuf.h"
 
 #include "Cell.h"
 #include "defun.h"
@@ -61,7 +61,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "ov-cell.h"
 #include "pager.h"
 #include "pt-exp.h"
-#include "symtab.h"
 #include "sysdep.h"
 #include "unwind-prot.h"
 #include "utils.h"
@@ -747,7 +746,13 @@ add_hdf5_data (hid_t loc_id, const octave_value& tc,
   hid_t type_id = -1, space_id = -1, data_id = -1, data_type_id = -1;
   bool retval = false;
   octave_value val = tc;
-  std::string t = tc.type_name();
+  // FIXME: diagonal & permutation matrices currently don't know how to save
+  // themselves, so we convert them first to normal matrices using A = A(:,:).
+  // This is a temporary hack.
+  if (val.is_diag_matrix () || val.is_perm_matrix ())
+    val = val.full_value ();
+
+  std::string t = val.type_name();
 
   data_id = H5Gcreate (loc_id, name.c_str (), 0);
   if (data_id < 0)
