@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1996, 1997, 2003, 2004, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 1996, 1997, 2003, 2004, 2005, 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -51,12 +51,13 @@ along with Octave; see the file COPYING.  If not, see
 #include "gripes.h"
 #include "lex.h"
 #include "load-save.h"
+#include "ls-ascii-helper.h"
+#include "ls-mat-ascii.h"
 #include "oct-obj.h"
 #include "oct-map.h"
 #include "ov-cell.h"
 #include "pager.h"
 #include "pt-exp.h"
-#include "symtab.h"
 #include "sysdep.h"
 #include "unwind-prot.h"
 #include "utils.h"
@@ -64,9 +65,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "version.h"
 #include "dMatrix.h"
 
-#include "ls-mat-ascii.h"
-#include "ls-ascii-helper.h"
-
 static std::string
 get_mat_data_input_line (std::istream& is)
 {
@@ -83,7 +81,6 @@ get_mat_data_input_line (std::istream& is)
 	{
 	  if (c == '\n' || c == '\r')
 	    {
-	      // Let skip_until_newline handle CR/LF issues...
 	      is.putback (c);
 	      skip_preceeding_newline (is);
 	      break;
@@ -91,9 +88,7 @@ get_mat_data_input_line (std::istream& is)
 
 	  if (c == '%' || c == '#')
 	    {
-	      // skip to end of line
 	      skip_until_newline (is, false);
-
 	      break;
 	    }
 
@@ -135,26 +130,26 @@ get_lines_and_columns (std::istream& is, const std::string& filename, octave_idx
       // CRLF pair as the line separator.  Any other CR in the text
       // will not be considered as whitespace.
 
-      if (beg != NPOS && buf[beg] == '\r' && beg == buf.length () - 1)
+      if (beg != std::string::npos && buf[beg] == '\r' && beg == buf.length () - 1)
 	{
 	  // We had a blank line ending with a CRLF.  Handle it the
 	  // same as an empty line.
-	  beg = NPOS;
+	  beg = std::string::npos;
 	}
 
       octave_idx_type tmp_nc = 0;
 
-      while (beg != NPOS)
+      while (beg != std::string::npos)
 	{
 	  tmp_nc++;
 
 	  size_t end = buf.find_first_of (", \t", beg);
 
-	  if (end != NPOS)
+	  if (end != std::string::npos)
 	    {
 	      beg = buf.find_first_not_of (", \t", end);
 
-	      if (beg == NPOS || (buf[beg] == '\r' && 
+	      if (beg == std::string::npos || (buf[beg] == '\r' && 
 				  beg == buf.length () - 1))
 		{
 		  // We had a line with trailing spaces and
@@ -210,14 +205,14 @@ read_mat_ascii_data (std::istream& is, const std::string& filename,
 
   size_t pos = filename.rfind ('/');
 
-  if (pos != NPOS)
+  if (pos != std::string::npos)
     varname = filename.substr (pos+1);
   else
     varname = filename;
 
   pos = varname.rfind ('.');
 
-  if (pos != NPOS)
+  if (pos != std::string::npos)
     varname = varname.substr (0, pos);
 
   size_t len = varname.length ();
@@ -334,7 +329,7 @@ read_mat_ascii_data (std::istream& is, const std::string& filename,
 
 bool
 save_mat_ascii_data (std::ostream& os, const octave_value& val,
-		     int precision)
+		     int precision, bool tabs)
 {
   bool success = true;
 
@@ -358,7 +353,21 @@ save_mat_ascii_data (std::ostream& os, const octave_value& val,
       std::ios::fmtflags oflags
 	= os.flags (static_cast<std::ios::fmtflags> (std::ios::scientific));
 
-      os << m;
+      if (tabs)
+        {
+          for (octave_idx_type i = 0; i < m.rows (); i++)
+            {
+              for (octave_idx_type j = 0; j < m.cols (); j++)
+                {
+                  // Omit leading tabs.
+                  if (j != 0) os << '\t'; 
+                  octave_write_double (os, m (i, j));
+                }
+              os << "\n";
+            }
+        }
+      else
+        os << m;
 
       os.flags (oflags);
 
