@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-              2005, 2006, 2007, 2008 John W. Eaton
+              2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -30,15 +30,12 @@ along with Octave; see the file COPYING.  If not, see
 #include <cstring>
 
 #include <iomanip>
+#include <iostream>
 #include <fstream>
 #include <sstream>
 #include <string>
 
 #include <Array.h>
-#include <Array2.h>
-#include <Array3.h>
-
-#include <Array.cc>
 
 #include "byte-swap.h"
 #include "lo-ieee.h"
@@ -48,6 +45,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "quit.h"
 
 #include "error.h"
+#include "gripes.h"
 #include "input.h"
 #include "oct-stdstrm.h"
 #include "oct-stream.h"
@@ -963,6 +961,14 @@ octave_base_stream::do_gets (octave_idx_type max_len, bool& err,
 {
   std::string retval;
 
+  if ((interactive || forced_interactive) && file_number () == 0)
+    {
+      ::error ("%s: unable to read from stdin while running interactively",
+	       who.c_str ());
+	     
+      return retval;
+    }
+
   err = false;
 
   std::istream *isp = input_stream ();
@@ -1581,14 +1587,14 @@ do_scanf_conv (std::istream&, const scanf_format_elt&, double*,
         { \
 	  int chars_read = 0; \
 	  while (is && chars_read++ < width && (c = is.get ()) != EOF \
-	         && char_class.find (c) != NPOS) \
+	         && char_class.find (c) != std::string::npos) \
 	    buf << static_cast<char> (c); \
 	} \
       else \
 	{ \
 	  int chars_read = 0; \
 	  while (is && chars_read++ < width && (c = is.get ()) != EOF \
-	         && char_class.find (c) == NPOS) \
+	         && char_class.find (c) == std::string::npos) \
 	    buf << static_cast<char> (c); \
 	} \
  \
@@ -1650,14 +1656,22 @@ octave_base_stream::do_scanf (scanf_format_list& fmt_list,
 			      octave_idx_type nr, octave_idx_type nc, bool one_elt_size_spec,
 			      octave_idx_type& conversion_count, const std::string& who)
 {
+  octave_value retval = Matrix ();
+
+  if ((interactive || forced_interactive) && file_number () == 0)
+    {
+      ::error ("%s: unable to read from stdin while running interactively",
+	       who.c_str ());
+	     
+      return retval;
+    }
+
   conversion_count = 0;
 
   int nconv = fmt_list.num_conversions ();
 
   octave_idx_type data_index = 0;
 
-  octave_value retval = Matrix ();
-
   if (nr == 0 || nc == 0)
     {
       if (one_elt_size_spec)
@@ -2289,10 +2303,23 @@ public:
 
   enum state { ok, conversion_error };
 
-  printf_value_cache (const octave_value_list& args)
+  printf_value_cache (const octave_value_list& args, const std::string& who)
     : values (args), val_idx (0), elt_idx (0),
       n_vals (values.length ()), n_elts (0), data (0),
-      curr_state (ok) { }
+      curr_state (ok)
+  {
+    for (octave_idx_type i = 0; i < values.length (); i++)
+      {
+	octave_value val = values(i);
+
+	if (val.is_map () || val.is_cell () || val.is_object ()
+	    || val.is_list ())
+	  {
+	    gripe_wrong_type_arg (who, val);
+	    break;
+	  }
+      }
+  }
 
   ~printf_value_cache (void) { }
 
@@ -2540,7 +2567,10 @@ octave_base_stream::do_printf (printf_format_list& fmt_list,
 
       const printf_format_elt *elt = fmt_list.first ();
 
-      printf_value_cache val_cache (args);
+      printf_value_cache val_cache (args, who);
+
+      if (error_state)
+	return retval;
 
       for (;;)
 	{
@@ -2609,7 +2639,7 @@ octave_base_stream::do_printf (printf_format_list& fmt_list,
 			  tfmt.replace ((i1 = tfmt.rfind (elt->type)),
 					1, 1, 's');
 
-			  if ((i2 = tfmt.rfind ('.')) != NPOS && i2 < i1)
+			  if ((i2 = tfmt.rfind ('.')) != std::string::npos && i2 < i1)
 			    {
 			      tfmt.erase (i2, i1-i2);
 			      if (elt->prec < 0)
@@ -3033,7 +3063,7 @@ octave_stream::close (void)
 template <class RET_T, class READ_T>
 octave_value
 do_read (octave_stream& strm, octave_idx_type nr, octave_idx_type nc, octave_idx_type block_size,
-	 octave_idx_type skip, bool do_float_fmt_conv,
+	 octave_idx_type skip, bool do_float_fmt_conv, bool do_NA_conv,
 	 oct_mach_info::float_format from_flt_fmt, octave_idx_type& count)
 {
   octave_value retval;
@@ -3146,6 +3176,9 @@ do_read (octave_stream& strm, octave_idx_type nr, octave_idx_type nc, octave_idx
 		      dat = nda.fortran_vec ();
 		    }
 
+		  if (do_NA_conv && __lo_ieee_is_old_NA (tmp))
+		    tmp = __lo_ieee_replace_old_NA (tmp);
+
 		  dat[count++] = tmp;
 
 		  elts_read++;
@@ -3197,7 +3230,7 @@ do_read (octave_stream& strm, octave_idx_type nr, octave_idx_type nc, octave_idx
 
 #define DO_READ_VAL_TEMPLATE(RET_T, READ_T) \
   template octave_value \
-  do_read<RET_T, READ_T> (octave_stream&, octave_idx_type, octave_idx_type, octave_idx_type, octave_idx_type, bool, \
+  do_read<RET_T, READ_T> (octave_stream&, octave_idx_type, octave_idx_type, octave_idx_type, octave_idx_type, bool, bool, \
 			  oct_mach_info::float_format, octave_idx_type&)
 
 // FIXME -- should we only have float if it is a different
@@ -3226,32 +3259,29 @@ INSTANTIATE_DO_READ (int32NDArray);
 INSTANTIATE_DO_READ (uint32NDArray);
 INSTANTIATE_DO_READ (int64NDArray);
 INSTANTIATE_DO_READ (uint64NDArray);
-// INSTANTIATE_DO_READ (floatNDArray);
+INSTANTIATE_DO_READ (FloatNDArray);
 INSTANTIATE_DO_READ (NDArray);
 INSTANTIATE_DO_READ (charNDArray);
 INSTANTIATE_DO_READ (boolNDArray);
 
-typedef octave_value (*read_fptr) (octave_stream&, octave_idx_type, octave_idx_type, octave_idx_type, octave_idx_type, bool,
+typedef octave_value (*read_fptr) (octave_stream&, octave_idx_type, octave_idx_type, octave_idx_type, octave_idx_type, bool, bool,
 				   oct_mach_info::float_format ffmt, octave_idx_type&);
 
-INSTANTIATE_ARRAY (read_fptr,);
-template class Array2<read_fptr>;
-
 #define FILL_TABLE_ROW(R, VAL_T) \
-  read_fptr_table(R,oct_data_conv::dt_int8) = do_read<VAL_T, octave_int8>; \
-  read_fptr_table(R,oct_data_conv::dt_uint8) = do_read<VAL_T, octave_uint8>; \
-  read_fptr_table(R,oct_data_conv::dt_int16) = do_read<VAL_T, octave_int16>; \
-  read_fptr_table(R,oct_data_conv::dt_uint16) = do_read<VAL_T, octave_uint16>; \
-  read_fptr_table(R,oct_data_conv::dt_int32) = do_read<VAL_T, octave_int32>; \
-  read_fptr_table(R,oct_data_conv::dt_uint32) = do_read<VAL_T, octave_uint32>; \
-  read_fptr_table(R,oct_data_conv::dt_int64) = do_read<VAL_T, octave_int64>; \
-  read_fptr_table(R,oct_data_conv::dt_uint64) = do_read<VAL_T, octave_uint64>; \
-  read_fptr_table(R,oct_data_conv::dt_single) = do_read<VAL_T, float>; \
-  read_fptr_table(R,oct_data_conv::dt_double) = do_read<VAL_T, double>; \
-  read_fptr_table(R,oct_data_conv::dt_char) = do_read<VAL_T, char>; \
-  read_fptr_table(R,oct_data_conv::dt_schar) = do_read<VAL_T, signed char>; \
-  read_fptr_table(R,oct_data_conv::dt_uchar) = do_read<VAL_T, unsigned char>; \
-  read_fptr_table(R,oct_data_conv::dt_logical) = do_read<VAL_T, unsigned char>
+  read_fptr_table[R][oct_data_conv::dt_int8] = do_read<VAL_T, octave_int8>; \
+  read_fptr_table[R][oct_data_conv::dt_uint8] = do_read<VAL_T, octave_uint8>; \
+  read_fptr_table[R][oct_data_conv::dt_int16] = do_read<VAL_T, octave_int16>; \
+  read_fptr_table[R][oct_data_conv::dt_uint16] = do_read<VAL_T, octave_uint16>; \
+  read_fptr_table[R][oct_data_conv::dt_int32] = do_read<VAL_T, octave_int32>; \
+  read_fptr_table[R][oct_data_conv::dt_uint32] = do_read<VAL_T, octave_uint32>; \
+  read_fptr_table[R][oct_data_conv::dt_int64] = do_read<VAL_T, octave_int64>; \
+  read_fptr_table[R][oct_data_conv::dt_uint64] = do_read<VAL_T, octave_uint64>; \
+  read_fptr_table[R][oct_data_conv::dt_single] = do_read<VAL_T, float>; \
+  read_fptr_table[R][oct_data_conv::dt_double] = do_read<VAL_T, double>; \
+  read_fptr_table[R][oct_data_conv::dt_char] = do_read<VAL_T, char>; \
+  read_fptr_table[R][oct_data_conv::dt_schar] = do_read<VAL_T, signed char>; \
+  read_fptr_table[R][oct_data_conv::dt_uchar] = do_read<VAL_T, unsigned char>; \
+  read_fptr_table[R][oct_data_conv::dt_logical] = do_read<VAL_T, unsigned char>
 
 octave_value
 octave_stream::read (const Array<double>& size, octave_idx_type block_size,
@@ -3264,10 +3294,14 @@ octave_stream::read (const Array<double>& size, octave_idx_type block_size,
 
   // Table function pointers for return types x read types.
 
-  static Array2<read_fptr> read_fptr_table (oct_data_conv::dt_unknown, 14, 0);
+  static read_fptr read_fptr_table[oct_data_conv::dt_unknown][14];
 
   if (! initialized)
     {
+      for (int i = 0; i < oct_data_conv::dt_unknown; i++)
+        for (int j = 0; j < 14; j++)
+          read_fptr_table[i][j] = 0;
+
       FILL_TABLE_ROW (oct_data_conv::dt_int8, int8NDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_uint8, uint8NDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_int16, int16NDArray);
@@ -3276,10 +3310,7 @@ octave_stream::read (const Array<double>& size, octave_idx_type block_size,
       FILL_TABLE_ROW (oct_data_conv::dt_uint32, uint32NDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_int64, int64NDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_uint64, uint64NDArray);
-      // FIXME -- the following line allows things like int8=>single
-      // to work, but they will actually return a double value.  We
-      // need a floatNDArray for this to work properly.
-      FILL_TABLE_ROW (oct_data_conv::dt_single, NDArray);
+      FILL_TABLE_ROW (oct_data_conv::dt_single, FloatNDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_double, NDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_char, charNDArray);
       FILL_TABLE_ROW (oct_data_conv::dt_schar, charNDArray);
@@ -3317,16 +3348,19 @@ octave_stream::read (const Array<double>& size, octave_idx_type block_size,
 	      if (ffmt == oct_mach_info::flt_fmt_unknown)
 		ffmt = float_format ();
 
-	      read_fptr fcn = read_fptr_table (output_type, input_type);
+	      read_fptr fcn = read_fptr_table[output_type][input_type];
 
 	      bool do_float_fmt_conv = ((input_type == oct_data_conv::dt_double
 					 || input_type == oct_data_conv::dt_single)
 					&& ffmt != float_format ());
 
+	      bool do_NA_conv = (output_type == oct_data_conv::dt_double);
+
 	      if (fcn)
 		{
 		  retval = (*fcn) (*this, nr, nc, block_size, skip,
-				   do_float_fmt_conv, ffmt, char_count);
+				   do_float_fmt_conv, do_NA_conv,
+				   ffmt, char_count);
 
 		  // FIXME -- kluge!
 
@@ -3558,17 +3592,36 @@ octave_stream::write (const Array<T>& data, octave_idx_type block_size,
 	{
 	  std::ostream& os = *osp;
 
-	  // It seems that Matlab writes zeros instead of actually
-	  // seeking.  Hmm...
-
 	  if (skip != 0 && (i % block_size) == 0)
 	    {
-	      // FIXME -- probably should try to write larger
-	      // blocks...
+	      // Seek to skip when inside bounds of existing file.
+	      // Otherwise, write NUL to skip.
 
-	      unsigned char zero = 0;
-	      for (int j = 0; j < skip; j++)
-		os.write (reinterpret_cast<const char *> (&zero), 1);
+	      long orig_pos = tell ();
+
+	      seek (0, SEEK_END);
+
+	      long eof_pos = tell ();
+
+	      // Is it possible for this to fail to return us to the
+	      // original position?
+	      seek (orig_pos, SEEK_SET);
+
+	      long remaining = eof_pos - orig_pos;
+
+	      if (remaining < skip)
+		{
+		  seek (0, SEEK_END);
+
+		  // FIXME -- probably should try to write larger
+		  // blocks...
+
+		  unsigned char zero = 0;
+		  for (octave_idx_type j = 0; j < skip - remaining; j++)
+		    os.write (reinterpret_cast<const char *> (&zero), 1);
+		}
+	      else
+		seek (skip, SEEK_CUR);
 	    }
 
 	  if (os)
@@ -3615,6 +3668,11 @@ octave_stream::write (const Array<double>&, octave_idx_type,
 		      oct_data_conv::data_type,
 		      octave_idx_type, oct_mach_info::float_format);
 
+template octave_idx_type
+octave_stream::write (const Array<float>&, octave_idx_type,
+		      oct_data_conv::data_type,
+		      octave_idx_type, oct_mach_info::float_format);
+
 template octave_idx_type
 octave_stream::write (const Array<octave_int8>&, octave_idx_type,
 		      oct_data_conv::data_type,
@@ -3941,10 +3999,10 @@ octave_stream_list::remove (const octave_value& fid, const std::string& who)
 }
 
 void
-octave_stream_list::clear (void)
+octave_stream_list::clear (bool flush)
 {
   if (instance)
-    instance->do_clear ();
+    instance->do_clear (flush);
 }
 
 string_vector
@@ -4026,12 +4084,20 @@ octave_stream_list::do_lookup (int fid, const std::string& who) const
 
   if (fid >= 0)
     {
-      ostrl_map::const_iterator iter = list.find (fid);
-
-      if (iter != list.end ())
-	retval = iter->second;
+      if (lookup_cache != list.end () && lookup_cache->first == fid)
+        retval = lookup_cache->second;
       else
-	gripe_invalid_file_id (fid, who);
+        {
+          ostrl_map::const_iterator iter = list.find (fid);
+
+          if (iter != list.end ())
+            {
+              retval = iter->second;
+              lookup_cache = iter;
+            }
+          else
+            gripe_invalid_file_id (fid, who);
+        }
     }
   else
     gripe_invalid_file_id (fid, who);
@@ -4068,11 +4134,13 @@ octave_stream_list::do_remove (int fid, const std::string& who)
       if (iter != list.end ())
 	{
 	  octave_stream os = iter->second;
+          list.erase (iter);
+          lookup_cache = list.end ();
 
+          // FIXME: is this check redundant?
 	  if (os.is_valid ())
 	    {
 	      os.close ();
-	      iter->second = octave_stream ();
 	      retval = 0;
 	    }
 	  else
@@ -4094,18 +4162,7 @@ octave_stream_list::do_remove (const octave_value& fid, const std::string& who)
 
   if (fid.is_string () && fid.string_value () == "all")
     {
-      for (ostrl_map::iterator p = list.begin (); p != list.end (); p++)
-	{
-	  // Skip stdin, stdout, and stderr.
-
-	  if (p->first > 2)
-	    {
-	      octave_stream os = p->second;
-
-	      if (os.is_valid ())
-		do_remove (p->first, who);
-	    }
-	}
+      do_clear (false);
 
       retval = 0;
     }
@@ -4121,22 +4178,30 @@ octave_stream_list::do_remove (const octave_value& fid, const std::string& who)
 }
 
 void
-octave_stream_list::do_clear (void)
+octave_stream_list::do_clear (bool flush)
 {
-  // Do flush stdout and stderr.
-
-  list[0].flush ();
-  list[1].flush ();
-
-  // But don't delete them or stdin.
-
-  for (ostrl_map::iterator p = list.begin (); p != list.end (); p++)
+  if (flush)
     {
-      // Skip stdin, stdout, and stderr.
+      // Do flush stdout and stderr.
 
-      if (p->first > 2)
-	p->second = octave_stream ();
+      list[0].flush ();
+      list[1].flush ();
     }
+
+  octave_stream saved_os[3];
+  // But don't delete them or stdin.
+  for (ostrl_map::iterator iter = list.begin (); iter != list.end (); iter++)
+    {
+      int fid = iter->first;
+      octave_stream os = iter->second;
+      if (fid < 3)
+        saved_os[fid] = os;
+      else if (os.is_valid ())
+        os.close ();
+    }
+  list.clear ();
+  for (int fid = 0; fid < 3; fid++) list[fid] = saved_os[fid];
+  lookup_cache = list.end ();
 }
 
 string_vector
