@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2003, 2004, 2005, 2006, 2007 John W. Eaton
+              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -107,7 +107,7 @@ static bool read_init_files = true;
 static bool read_site_files = true;
 
 // TRUE means we set the initial path to configured defaults.
-// (--no-initial-path)
+// (--no-init-path)
 static bool set_initial_path = true;
 
 // TRUE means we don't print the usual startup message.
@@ -143,22 +143,24 @@ static bool persist = false;
 
 // Long options.  See the comments in getopt.h for the meanings of the
 // fields in this structure.
-#define EVAL_OPTION 1
-#define EXEC_PATH_OPTION 2
-#define IMAGE_PATH_OPTION 3
-#define INFO_FILE_OPTION 4
-#define INFO_PROG_OPTION 5
-#define NO_INIT_FILE_OPTION 6
-#define NO_LINE_EDITING_OPTION 7
-#define NO_SITE_FILE_OPTION 8
-#define NO_INITIAL_PATH_OPTION 9
-#define PERSIST_OPTION 10
-#define TRADITIONAL_OPTION 11
-#define LINE_EDITING_OPTION 12
+#define DOC_CACHE_FILE_OPTION 1
+#define EVAL_OPTION 2
+#define EXEC_PATH_OPTION 3
+#define IMAGE_PATH_OPTION 4
+#define INFO_FILE_OPTION 5
+#define INFO_PROG_OPTION 6
+#define NO_INIT_FILE_OPTION 7
+#define NO_LINE_EDITING_OPTION 8
+#define NO_SITE_FILE_OPTION 9
+#define NO_INIT_PATH_OPTION 10
+#define PERSIST_OPTION 11
+#define TRADITIONAL_OPTION 12
+#define LINE_EDITING_OPTION 13
 long_options long_opts[] =
   {
     { "debug",            prog_args::no_arg,       0, 'd' },
     { "braindead",        prog_args::no_arg,       0, TRADITIONAL_OPTION },
+    { "doc-cache-file",   prog_args::required_arg, 0, DOC_CACHE_FILE_OPTION },
     { "echo-commands",    prog_args::no_arg,       0, 'x' },
     { "eval",             prog_args::required_arg, 0, EVAL_OPTION },
     { "exec-path",        prog_args::required_arg, 0, EXEC_PATH_OPTION },
@@ -172,7 +174,7 @@ long_options long_opts[] =
     { "no-init-file",     prog_args::no_arg,       0, NO_INIT_FILE_OPTION },
     { "no-line-editing",  prog_args::no_arg,       0, NO_LINE_EDITING_OPTION },
     { "no-site-file",     prog_args::no_arg,       0, NO_SITE_FILE_OPTION },
-    { "no-initial-path",  prog_args::no_arg,       0, NO_INITIAL_PATH_OPTION },
+    { "no-init-path",     prog_args::no_arg,       0, NO_INIT_PATH_OPTION },
     { "norc",             prog_args::no_arg,       0, 'f' },
     { "path",             prog_args::required_arg, 0, 'p' },
     { "persist",          prog_args::no_arg,       0, PERSIST_OPTION },
@@ -189,15 +191,15 @@ long_options long_opts[] =
 static void
 intern_argv (int argc, char **argv)
 {
-  symbol_record *nargin_sr = top_level_sym_tab->lookup ("__nargin__", true);
+  assert (symbol_table::at_top_level ());
 
-  nargin_sr->mark_as_static ();
+  symbol_table::varref (".nargin.") = argc - 1;
 
-  nargin_sr->define (argc-1);
+  symbol_table::mark_hidden (".nargin.");
 
   if (argc > 1)
     {
-      octave_argv.resize (argc-1);
+      octave_argv.resize (argc - 1);
 
       // Skip program name in argv.
       int i = argc;
@@ -220,8 +222,8 @@ initialize_pathsearch (void)
     odb = octave_env::getenv ("OCTAVE_DB_DIR");
 
   if (odb.empty ())
-    odb = Vdata_dir + file_ops::dir_sep_str + "octave:"
-      + Vlibexec_dir + file_ops::dir_sep_str + "octave";
+    odb = Vdata_dir + file_ops::dir_sep_str () + "octave:"
+      + Vlibexec_dir + file_ops::dir_sep_str () + "octave";
 }
 
 DEFUN (__version_info__, args, ,
@@ -291,7 +293,11 @@ execute_startup_files (void)
 
   input_from_startup_file = true;
 
-  int verbose = (verbose_flag && ! inhibit_startup_message);
+  std::string context;
+
+  bool verbose = (verbose_flag && ! inhibit_startup_message);
+
+  bool require_file = false;
 
   if (read_site_files)
     {
@@ -300,9 +306,9 @@ execute_startup_files (void)
       // (if it exists), then from the file
       // $(prefix)/share/octave/$(version)/m/octaverc (if it exists).
 
-      parse_and_execute (Vlocal_site_defaults_file, verbose);
+      source_file (Vlocal_site_defaults_file, context, verbose, require_file);
 
-      parse_and_execute (Vsite_defaults_file, verbose);
+      source_file (Vsite_defaults_file, context, verbose, require_file);
     }
 
   if (read_init_files)
@@ -326,7 +332,7 @@ execute_startup_files (void)
 
       if (! home_rc.empty ())
 	{
-	  parse_and_execute (home_rc, verbose);
+	  source_file (home_rc, context, verbose, require_file);
 
 	  // Names alone are not enough.
 
@@ -354,7 +360,7 @@ execute_startup_files (void)
 	      local_rc = octave_env::make_absolute (initfile, curr_dir);
 	    }
 
-	  parse_and_execute (local_rc, verbose);
+	  source_file (local_rc, context, verbose, require_file);
 	}
     }
 
@@ -388,6 +394,11 @@ execute_eval_option_code (const std::string& code)
     {
       eval_string (code, false, parse_status, 0);
     }
+  catch (octave_quit_exception e)
+    {
+      unwind_protect::run_frame ("execute_eval_option_code");
+      clean_up_and_exit (e.status);
+    }
   catch (octave_interrupt_exception)
     {
       recover_from_exception ();
@@ -440,16 +451,25 @@ execute_command_line_file (const std::string& fname)
 
   octave_program_invocation_name = curr_fcn_file_name;
 
-  size_t pos = curr_fcn_file_name.find_last_of (file_ops::dir_sep_chars);
+  size_t pos = curr_fcn_file_name.find_last_of (file_ops::dir_sep_chars ());
   
-  std::string tmp = (pos != NPOS)
+  std::string tmp = (pos != std::string::npos)
     ? curr_fcn_file_name.substr (pos+1) : curr_fcn_file_name;
 
   octave_program_name = tmp;
 
   try
     {
-      parse_and_execute (fname, false, "octave");
+      std::string context;
+      bool verbose = false;
+      bool require_file = true;
+
+      source_file (fname, context, verbose, require_file, "octave");
+    }
+  catch (octave_quit_exception e)
+    {
+      unwind_protect::run_frame ("execute_command_line_file");
+      clean_up_and_exit (e.status);
     }
   catch (octave_interrupt_exception)
     {
@@ -477,24 +497,26 @@ Usage: octave [options]\n\
 Options:\n\
 \n\
   --debug, -d             Enter parser debugging mode.\n\
+  --doc-cache-file FILE   Use doc cache file FILE.\n\
   --echo-commands, -x     Echo commands as they are executed.\n\
   --eval CODE             Evaluate CODE.  Exit when done unless --persist.\n\
   --exec-path PATH        Set path for executing subprograms.\n\
   --help, -h, -?          Print short help message and exit.\n\
-  --image-path PATH       Set initial IMAGE_PATH to PATH.\n\
+  --image-path PATH       Add PATH to head of image search path.\n\
   --info-file FILE        Use top-level info file FILE.\n\
   --info-program PROGRAM  Use PROGRAM for reading info files.\n\
   --interactive, -i       Force interactive behavior.\n\
   --line-editing          Force readline use for command-line editing.\n\
   --no-history, -H        Don't save commands to the history list\n\
   --no-init-file          Don't read the ~/.octaverc or .octaverc files.\n\
+  --no-init-path          Don't initialize function search path.\n\
   --no-line-editing       Don't use readline for command-line editing.\n\
   --no-site-file          Don't read the site-wide octaverc file.\n\
   --norc, -f              Don't read any initialization files.\n\
-  --path PATH, -p PATH    Set initial function search path to PATH.\n\
+  --path PATH, -p PATH    Add PATH to head of function search path.\n\
   --persist               Go interactive after --eval or reading from FILE.\n\
   --silent, -q            Don't print message at startup.\n\
-  --traditional           Set compatibility variables.\n\
+  --traditional           Set variables for closer MATLAB compatibility.\n\
   --verbose, -V           Enable verbose output in some cases.\n\
   --version, -v           Print version number and exit.\n\
 \n\
@@ -526,10 +548,21 @@ print_version_and_exit (void)
   exit (0);
 }
 
+static void
+lo_error_handler (const char *fmt, ...)
+{
+  va_list args;
+  va_start (args, fmt);
+  verror (fmt, args);
+  va_end (args);
+
+  octave_throw_execution_exception ();
+}
+
 static void
 initialize_error_handlers ()
 {
-  set_liboctave_error_handler (error);
+  set_liboctave_error_handler (lo_error_handler);
   set_liboctave_warning_handler (warning);
   set_liboctave_warning_with_id_handler (warning_with_id);
 }
@@ -544,6 +577,7 @@ maximum_braindamage (void)
   bind_internal_variable ("PS1", ">> ");
   bind_internal_variable ("PS2", "");
   bind_internal_variable ("beep_on_error", true);
+  bind_internal_variable ("confirm_recursive_rmdir", false);
   bind_internal_variable ("crash_dumps_octave_core", false);
   bind_internal_variable ("default_save_options", "-mat-binary");
   bind_internal_variable ("fixed_point_format", true);
@@ -596,11 +630,11 @@ octave_main (int argc, char **argv, int embedded)
 
   if (! embedded)
     install_signal_handlers ();
+  else
+    quit_allowed = false;
 
   initialize_file_io ();
 
-  initialize_symbol_tables ();
-
   install_types ();
 
   install_ops ();
@@ -611,12 +645,15 @@ octave_main (int argc, char **argv, int embedded)
 
   bool forced_line_editing = false;
 
+  bool read_history_file = true;
+
   int optc;
   while ((optc = args.getopt ()) != EOF)
     {
       switch (optc)
 	{
 	case 'H':
+	  read_history_file = false;
 	  bind_internal_variable ("saving_history", false);
 	  break;
 
@@ -663,6 +700,11 @@ octave_main (int argc, char **argv, int embedded)
 	  print_version_and_exit ();
 	  break;
 
+	case DOC_CACHE_FILE_OPTION:
+	  if (args.optarg ())
+	    bind_internal_variable ("doc_cache_file", args.optarg ());
+	  break;
+
 	case EVAL_OPTION:
 	  if (args.optarg ())
 	    {
@@ -693,9 +735,9 @@ octave_main (int argc, char **argv, int embedded)
 	    bind_internal_variable ("info_program", args.optarg ());
 	  break;
 
-        case LINE_EDITING_OPTION:
-          forced_line_editing = true;
-          break;
+	case LINE_EDITING_OPTION:
+	  forced_line_editing = true;
+	  break;
 
 	case NO_INIT_FILE_OPTION:
 	  read_init_files = false;
@@ -709,7 +751,7 @@ octave_main (int argc, char **argv, int embedded)
 	  read_site_files = 0;
 	  break;
 
-	case NO_INITIAL_PATH_OPTION:
+	case NO_INIT_PATH_OPTION:
 	  set_initial_path = false;
 	  break;
 
@@ -759,7 +801,7 @@ octave_main (int argc, char **argv, int embedded)
 
   execute_startup_files ();
 
-  initialize_history ();
+  initialize_history (read_history_file);
 
   if (! inhibit_startup_message && reading_startup_message_printed)
     std::cout << std::endl;
@@ -897,7 +939,7 @@ how to create an executable Octave script.\n\
 DEFUN (program_name, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} program_name ()\n\
-Return the last component of of the value returned by\n\
+Return the last component of the value returned by\n\
 @code{program_invocation_name}.\n\
 @seealso{program_invocation_name}\n\
 @end deftypefn")
