@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-              2006, 2007 John W. Eaton
+              2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -26,7 +26,7 @@ along with Octave; see the file COPYING.  If not, see
 
 #include <cstdlib>
 
-#include <iostream>
+#include <iosfwd>
 #include <string>
 
 #include "lo-mappers.h"
@@ -62,15 +62,14 @@ public:
 
   octave_value squeeze (void) const { return scalar; }
 
+  octave_value full_value (void) const { return scalar; }
+
   octave_value subsref (const std::string& type,
 			const std::list<octave_value_list>& idx);
 
-  octave_value_list subsref (const std::string&,
-			     const std::list<octave_value_list>&, int)
-    {
-      panic_impossible ();
-      return octave_value_list ();
-    }
+  octave_value_list subsref (const std::string& type,
+			     const std::list<octave_value_list>& idx, int)
+    { return subsref (type, idx); }
 
   octave_value subsasgn (const std::string& type,
 			 const std::list<octave_value_list>& idx,
@@ -87,12 +86,37 @@ public:
   octave_value permute (const Array<int>&, bool = false) const
     { return scalar; }
 
+  octave_value reshape (const dim_vector& new_dims) const
+    { return array_value ().reshape (new_dims); }
+
   size_t byte_size (void) const { return sizeof (ST); }
 
   octave_value all (int = 0) const { return (scalar != ST ()); }
 
   octave_value any (int = 0) const { return (scalar != ST ()); }
 
+  octave_value diag (octave_idx_type k = 0) const 
+    { return octave_value (matrix_value (). diag (k)); }
+
+  octave_value sort (octave_idx_type, sortmode) const
+    { return octave_value (scalar); }
+  octave_value sort (Array<octave_idx_type> &sidx, octave_idx_type,
+		     sortmode) const
+    { 
+      sidx.resize (dim_vector (1, 1)); 
+      sidx(0) = 0; 
+      return octave_value (scalar); 
+    }
+
+  sortmode is_sorted (sortmode mode = UNSORTED) const
+    { return mode ? mode : ASCENDING; }
+
+  Array<octave_idx_type> sort_rows_idx (sortmode) const
+    { return Array<octave_idx_type> (1, 0); }
+
+  sortmode is_sorted_rows (sortmode mode = UNSORTED) const
+    { return mode ? mode : ASCENDING; }
+
   MatrixType matrix_type (void) const { return typ; }
   MatrixType matrix_type (const MatrixType& _typ) const
     { MatrixType ret = typ; typ = _typ; return ret; }
@@ -101,7 +125,7 @@ public:
 
   bool is_numeric_type (void) const { return true; }
 
-  bool is_true (void) const { return (scalar != ST ()); }
+  bool is_true (void) const;
 
   void print (std::ostream& os, bool pr_as_read_syntax = false) const;
 
