@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007 David Bateman
+Copyright (C) 2004, 2005, 2006, 2007, 2008 David Bateman
 
 This file is part of Octave.
 
@@ -23,7 +23,7 @@ along with Octave; see the file COPYING.  If not, see
 #if !defined (octave_fcn_inline_h)
 #define octave_fcn_inline_h 1
 
-#include <iostream>
+#include <iosfwd>
 #include <string>
 
 #include "oct-alloc.h"
@@ -38,6 +38,7 @@ along with Octave; see the file COPYING.  If not, see
 // Inline functions.
 
 class
+OCTINTERP_API
 octave_fcn_inline : public octave_fcn_handle
 {
 public:
