@@ -1,12 +1,14 @@
 /*
 
-Copyright (C) 2004 David Bateman
-Copyright (C) 1998-2004 Andy Adler
+Copyright (C) 2004, 2005, 2006, 2007, 2008 David Bateman
+Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Andy Adler
+
+This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -14,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -29,6 +30,10 @@ Boston, MA 02110-1301, USA.
 #include <iostream>
 #include <vector>
 
+#include "lo-specfun.h"
+#include "lo-mappers.h"
+#include "oct-locbuf.h"
+
 #include "ov-base.h"
 #include "ov-scalar.h"
 #include "gripes.h"
@@ -42,16 +47,16 @@ Boston, MA 02110-1301, USA.
 
 #include "ov-bool-sparse.h"
 
-template class octave_base_sparse<SparseMatrix>;
+template class OCTINTERP_API octave_base_sparse<SparseMatrix>;
 
 DEFINE_OCTAVE_ALLOCATOR (octave_sparse_matrix);
 
-DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_sparse_matrix, "sparse matrix", "sparse");
+DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_sparse_matrix, "sparse matrix", "double");
 
 idx_vector
 octave_sparse_matrix::index_vector (void) const
 {
-  if (matrix.numel () == matrix.nonzero ())
+  if (matrix.numel () == matrix.nnz ())
     return idx_vector (array_value ());
   else
     {
@@ -61,36 +66,32 @@ octave_sparse_matrix::index_vector (void) const
     }
 }
 
-octave_value *
+octave_base_value *
 octave_sparse_matrix::try_narrowing_conversion (void)
 {
-  octave_value *retval = 0;
+  octave_base_value *retval = 0;
 
-  // Don't use numel, since it can overflow for very large matrices
-  // Note that for the second test, this means it becomes approximative
-  // since it involves a cast to double to avoid issues of overflow
-  if (matrix.rows () == 1 && matrix.cols () == 1)
+  if (Vsparse_auto_mutate)
     {
-      // Const copy of the matrix, so the right version of () operator used
-      const SparseMatrix tmp (matrix);
+      // Don't use numel, since it can overflow for very large matrices
+      // Note that for the second test, this means it becomes approximative
+      // since it involves a cast to double to avoid issues of overflow
+      if (matrix.rows () == 1 && matrix.cols () == 1)
+	{
+	  // Const copy of the matrix, so the right version of () operator used
+	  const SparseMatrix tmp (matrix);
 
-      retval = new octave_scalar (tmp (0));
+	  retval = new octave_scalar (tmp (0));
+	}
+      else if (matrix.cols () > 0 && matrix.rows () > 0 && 
+	       double (matrix.byte_size ()) > double (matrix.rows ()) *
+	       double (matrix.cols ()) * sizeof (double))
+	retval = new octave_matrix (matrix.matrix_value ());
     }
-  else if (matrix.cols () > 0 && matrix.rows () > 0 && 
-	   double (matrix.byte_size ()) > double (matrix.rows ()) *
-	   double (matrix.cols ()) * sizeof (double))
-    retval = new octave_matrix (matrix.matrix_value ());
 
   return retval;
 }
 
-bool
-octave_sparse_matrix::valid_as_scalar_index (void) const
-{
-  // XXX FIXME XXX
-  return false;
-}
-
 double
 octave_sparse_matrix::double_value (bool) const
 {
@@ -98,9 +99,9 @@ octave_sparse_matrix::double_value (bool) const
 
   if (numel () > 0)
     {
-      // XXX FIXME XXX -- is warn_fortran_indexing the right variable here?
-      if (Vwarn_fortran_indexing)
-	gripe_implicit_conversion ("real sparse matrix", "real scalar");
+      if (numel () > 1)
+	gripe_implicit_conversion ("Octave:array-as-scalar",
+				   "real sparse matrix", "real scalar");
 
       retval = matrix (0, 0);
     }
@@ -117,12 +118,12 @@ octave_sparse_matrix::complex_value (bool) const
 
   Complex retval (tmp, tmp);
 
-  // XXX FIXME XXX -- maybe this should be a function, valid_as_scalar()
+  // FIXME -- maybe this should be a function, valid_as_scalar()
   if (rows () > 0 && columns () > 0)
     {
-      // XXX FIXME XXX -- is warn_fortran_indexing the right variable here?
-      if (Vwarn_fortran_indexing)
-	gripe_implicit_conversion ("real sparse matrix", "complex scalar");
+      if (numel () > 1)
+	gripe_implicit_conversion ("Octave:array-as-scalar",
+				   "real sparse matrix", "complex scalar");
 
       retval = matrix (0, 0);
     }
@@ -138,6 +139,33 @@ octave_sparse_matrix::matrix_value (bool) const
   return matrix.matrix_value ();
 }
 
+boolNDArray
+octave_sparse_matrix::bool_array_value (bool warn) const
+{
+  NDArray m = matrix.matrix_value ();
+
+  if (m.any_element_is_nan ())
+    error ("invalid conversion from NaN to logical");
+  else if (warn && m.any_element_not_one_or_zero ())
+    gripe_logical_conversion ();
+
+  return boolNDArray (m);
+}
+
+charNDArray
+octave_sparse_matrix::char_array_value (bool) const
+{
+  charNDArray retval (dims (), 0);
+  octave_idx_type nc = matrix.cols ();
+  octave_idx_type nr = matrix.rows ();
+
+  for (octave_idx_type j = 0; j < nc; j++)
+    for (octave_idx_type i = matrix.cidx(j); i < matrix.cidx(j+1); i++)
+      retval(matrix.ridx(i) + nr * j) = static_cast<char>(matrix.data (i));
+
+  return retval;
+}
+  
 ComplexMatrix
 octave_sparse_matrix::complex_matrix_value (bool) const
 {
@@ -156,33 +184,6 @@ octave_sparse_matrix::array_value (bool) const
   return NDArray (matrix.matrix_value ());
 }
 
-streamoff_array
-octave_sparse_matrix::streamoff_array_value (void) const
-{
-  streamoff_array retval (dims ());
-  octave_idx_type nc = matrix.cols ();
-  octave_idx_type nr = matrix.rows ();
-
-  for (octave_idx_type j = 0; j < nc; j++)
-    for (octave_idx_type i = matrix.cidx(j); i < matrix.cidx(j+1); i++)
-      {
-	double d = matrix.data(i);
-
-	if (D_NINT (d) == d)
-	  {
-	    retval(matrix.ridx(i) + nr * j) = 
-	      std::streamoff (static_cast<long> (d));
-	  }
-	else
-	  {
-	    error ("conversion to streamoff_array value failed");
-	    break;
-	  }
-      }
-
-  return retval;
-}
-
 octave_value
 octave_sparse_matrix::convert_to_str_internal (bool, bool, char type) const
 {
@@ -222,7 +223,7 @@ octave_sparse_matrix::convert_to_str_internal (bool, bool, char type) const
 
 		  if (ival < 0 || ival > UCHAR_MAX)
 		    {
-		      // XXX FIXME XXX -- is there something
+		      // FIXME -- is there something
 		      // better we could do?
 
 		      ival = 0;
@@ -256,21 +257,21 @@ octave_sparse_matrix::save_binary (std::ostream& os, bool&save_as_floats)
 
   int nr = d(0);
   int nc = d(1);
-  int nz = nnz ();
+  int nz = nzmax ();
 
-  FOUR_BYTE_INT itmp;
+  int32_t itmp;
   // Use negative value for ndims to be consistent with other formats
   itmp= -2;        
-  os.write (X_CAST (char *, &itmp), 4);
+  os.write (reinterpret_cast<char *> (&itmp), 4);
   
   itmp= nr;    
-  os.write (X_CAST (char *, &itmp), 4);
+  os.write (reinterpret_cast<char *> (&itmp), 4);
   
   itmp= nc;
-  os.write (X_CAST (char *, &itmp), 4);
+  os.write (reinterpret_cast<char *> (&itmp), 4);
   
   itmp= nz;
-  os.write (X_CAST (char *, &itmp), 4);
+  os.write (reinterpret_cast<char *> (&itmp), 4);
 
   save_type st = LS_DOUBLE;
   if (save_as_floats)
@@ -283,7 +284,7 @@ octave_sparse_matrix::save_binary (std::ostream& os, bool&save_as_floats)
       else
 	st = LS_FLOAT;
     }
-  else if (matrix.nnz () > 8192) // XXX FIXME XXX -- make this configurable.
+  else if (matrix.nzmax () > 8192) // FIXME -- make this configurable.
     {
       double max_val, min_val;
       if (matrix.all_integers (max_val, min_val))
@@ -296,14 +297,14 @@ octave_sparse_matrix::save_binary (std::ostream& os, bool&save_as_floats)
      {
        OCTAVE_QUIT;
        itmp = matrix.cidx(i);
-       os.write (X_CAST (char *, &itmp), 4);
+       os.write (reinterpret_cast<char *> (&itmp), 4);
      }
 
    for (int i = 0; i < nz; i++) 
      {
        OCTAVE_QUIT;
        itmp = matrix.ridx(i); 
-       os.write (X_CAST (char *, &itmp), 4);
+       os.write (reinterpret_cast<char *> (&itmp), 4);
      }
 
    write_doubles (os, matrix.data(), st, nz);
@@ -315,10 +316,10 @@ bool
 octave_sparse_matrix::load_binary (std::istream& is, bool swap,
 				   oct_mach_info::float_format fmt)
 {
-  FOUR_BYTE_INT nz, nc, nr, tmp;
+  int32_t nz, nc, nr, tmp;
   char ctmp;
 
-  if (! is.read (X_CAST (char *, &tmp), 4))
+  if (! is.read (reinterpret_cast<char *> (&tmp), 4))
     return false;
 
   if (swap)
@@ -329,11 +330,11 @@ octave_sparse_matrix::load_binary (std::istream& is, bool swap,
     return false;
   }
 
-  if (! is.read (X_CAST (char *, &nr), 4))
+  if (! is.read (reinterpret_cast<char *> (&nr), 4))
     return false;
-  if (! is.read (X_CAST (char *, &nc), 4))
+  if (! is.read (reinterpret_cast<char *> (&nc), 4))
     return false;
-  if (! is.read (X_CAST (char *, &nz), 4))
+  if (! is.read (reinterpret_cast<char *> (&nz), 4))
     return false;
 
   if (swap)
@@ -350,7 +351,7 @@ octave_sparse_matrix::load_binary (std::istream& is, bool swap,
   for (int i = 0; i < nc+1; i++) 
     {
       OCTAVE_QUIT;
-      if (! is.read (X_CAST (char *, &tmp), 4))
+      if (! is.read (reinterpret_cast<char *> (&tmp), 4))
 	return false;
       if (swap)
 	swap_bytes<4> (&tmp);
@@ -360,17 +361,17 @@ octave_sparse_matrix::load_binary (std::istream& is, bool swap,
   for (int i = 0; i < nz; i++) 
     {
       OCTAVE_QUIT;
-      if (! is.read (X_CAST (char *, &tmp), 4))
+      if (! is.read (reinterpret_cast<char *> (&tmp), 4))
 	return false;
       if (swap)
 	swap_bytes<4> (&tmp);
       m.xridx(i) = tmp;
     }
 
-  if (! is.read (X_CAST (char *, &ctmp), 1))
+  if (! is.read (reinterpret_cast<char *> (&ctmp), 1))
     return false;
   
-  read_doubles (is, m.xdata (), X_CAST (save_type, ctmp), nz, swap, fmt);
+  read_doubles (is, m.xdata (), static_cast<save_type> (ctmp), nz, swap, fmt);
 
   if (error_state || ! is)
     return false;
@@ -380,6 +381,7 @@ octave_sparse_matrix::load_binary (std::istream& is, bool swap,
 }
 
 #if defined (HAVE_HDF5)
+
 bool
 octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name, 
 				 bool save_as_floats)
@@ -402,7 +404,7 @@ octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name,
   octave_idx_type tmp;
   hsize_t hdims[2];
 
-  space_hid = H5Screate_simple (0, hdims, (hsize_t*) 0);
+  space_hid = H5Screate_simple (0, hdims, 0);
   if (space_hid < 0) 
     {
       H5Gclose (group_hid);
@@ -420,7 +422,7 @@ octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name,
   
   tmp = m.rows ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-		     (void*) &tmp) >= 0;
+		     &tmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -440,7 +442,7 @@ octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name,
   
   tmp = m.cols ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-		     (void*) &tmp) >= 0;
+		     &tmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -458,9 +460,9 @@ octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name,
       return false;
     }
   
-  tmp = m.nnz ();
+  tmp = m.nzmax ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-		     (void*) &tmp) >= 0;
+		     &tmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -493,7 +495,7 @@ octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name,
   
   octave_idx_type * itmp = m.xcidx ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-		     (void*) itmp) >= 0;
+		     itmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -504,7 +506,7 @@ octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name,
 
   H5Sclose (space_hid);
 
-  hdims[0] = m.nnz();
+  hdims[0] = m.nzmax ();
   hdims[1] = 1;
 
   space_hid = H5Screate_simple (2, hdims, 0);
@@ -526,7 +528,7 @@ octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name,
   
   itmp = m.xridx ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-		     (void*) itmp) >= 0;
+		     itmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
@@ -570,7 +572,7 @@ octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name,
   
   double * dtmp = m.xdata ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL,
-		     H5P_DEFAULT, (void*) dtmp) >= 0;
+		     H5P_DEFAULT, dtmp) >= 0;
   H5Dclose (data_hid);
   H5Sclose (space_hid);
   H5Gclose (group_hid);
@@ -580,7 +582,7 @@ octave_sparse_matrix::save_hdf5 (hid_t loc_id, const char *name,
 
 bool
 octave_sparse_matrix::load_hdf5 (hid_t loc_id, const char *name,
-				      bool /* have_h5giterate_bug */)
+				 bool /* have_h5giterate_bug */)
 {
   octave_idx_type nr, nc, nz;
   hid_t group_hid, data_hid, space_hid;
@@ -594,7 +596,7 @@ octave_sparse_matrix::load_hdf5 (hid_t loc_id, const char *name,
     return (empty > 0);
   
   group_hid = H5Gopen (loc_id, name);
-  if (group_hid < 0 ) return false;
+  if (group_hid < 0) return false;
 
   data_hid = H5Dopen (group_hid, "nr");
   space_hid = H5Dget_space (data_hid);
@@ -608,7 +610,7 @@ octave_sparse_matrix::load_hdf5 (hid_t loc_id, const char *name,
     }
 
   if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) &nr) < 0)
+	       H5P_DEFAULT, &nr) < 0)
     { 
       H5Dclose (data_hid);
       H5Gclose (group_hid);
@@ -629,7 +631,7 @@ octave_sparse_matrix::load_hdf5 (hid_t loc_id, const char *name,
     }
 
   if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) &nc) < 0)
+	       H5P_DEFAULT, &nc) < 0)
     { 
       H5Dclose (data_hid);
       H5Gclose (group_hid);
@@ -650,7 +652,7 @@ octave_sparse_matrix::load_hdf5 (hid_t loc_id, const char *name,
     }
 
   if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) &nz) < 0)
+	       H5P_DEFAULT, &nz) < 0)
     { 
       H5Dclose (data_hid);
       H5Gclose (group_hid);
@@ -691,7 +693,7 @@ octave_sparse_matrix::load_hdf5 (hid_t loc_id, const char *name,
 
   octave_idx_type *itmp = m.xcidx ();
   if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) itmp) < 0) 
+	       H5P_DEFAULT, itmp) < 0) 
     {
       H5Sclose (space_hid);
       H5Dclose (data_hid);
@@ -726,7 +728,7 @@ octave_sparse_matrix::load_hdf5 (hid_t loc_id, const char *name,
 
   itmp = m.xridx ();
   if (H5Dread (data_hid, H5T_NATIVE_IDX, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) itmp) < 0) 
+	       H5P_DEFAULT, itmp) < 0) 
     {
       H5Sclose (space_hid);
       H5Dclose (data_hid);
@@ -761,7 +763,7 @@ octave_sparse_matrix::load_hdf5 (hid_t loc_id, const char *name,
 
   double *dtmp = m.xdata ();
   if (H5Dread (data_hid, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) dtmp) < 0) 
+	       H5P_DEFAULT, dtmp) < 0) 
     {
       H5Sclose (space_hid);
       H5Dclose (data_hid);
@@ -777,8 +779,138 @@ octave_sparse_matrix::load_hdf5 (hid_t loc_id, const char *name,
 
   return true;
 }
+
 #endif
 
+mxArray *
+octave_sparse_matrix::as_mxArray (void) const
+{
+  mwSize nz = nzmax();
+  mwSize nr = rows();
+  mwSize nc = columns();
+  mxArray *retval = new mxArray (mxDOUBLE_CLASS, nr, nc, nz, mxREAL);
+  double *pr = static_cast<double *> (retval->get_data ());
+  mwIndex *ir = retval->get_ir();
+  mwIndex *jc = retval->get_jc();
+
+  for (mwIndex i = 0; i < nz; i++)
+    {
+      pr[i] = matrix.data(i);
+      ir[i] = matrix.ridx(i);
+    }
+
+  for (mwIndex i = 0; i < nc + 1; i++)
+    jc[i] = matrix.cidx(i);
+
+  return retval;
+}
+
+static bool
+any_element_less_than (const SparseMatrix& a, double val)
+{
+  octave_idx_type len = a.nnz ();
+
+  if (val > 0. && len != a.numel ())
+    return true;
+
+  for (octave_idx_type i = 0; i < len; i++)
+    {
+      OCTAVE_QUIT;
+
+      if (a.data(i) < val)
+	return true;
+    }
+
+  return false;
+}
+
+static bool
+any_element_greater_than (const SparseMatrix& a, double val)
+{
+  octave_idx_type len = a.nnz ();
+
+  if (val < 0. && len != a.numel ())
+    return true;
+
+  for (octave_idx_type i = 0; i < len; i++)
+    {
+      OCTAVE_QUIT;
+
+      if (a.data(i) > val)
+	return true;
+    }
+
+  return false;
+}
+
+#define SPARSE_MAPPER(MAP, AMAP, FCN) \
+  octave_value \
+  octave_sparse_matrix::MAP (void) const \
+  { \
+    static AMAP dmap = FCN; \
+    return matrix.map (dmap); \
+  }
+
+#define CD_SPARSE_MAPPER(MAP, RFCN, CFCN, L1, L2) \
+  octave_value \
+  octave_sparse_matrix::MAP (void) const \
+  { \
+    static SparseMatrix::dmapper dmap = RFCN; \
+    static SparseMatrix::cmapper cmap = CFCN; \
+ \
+    return (any_element_less_than (matrix, L1) \
+            ? octave_value (matrix.map (cmap)) \
+            : (any_element_greater_than (matrix, L2) \
+               ? octave_value (matrix.map (cmap)) \
+	       : octave_value (matrix.map (dmap)))); \
+  }
+
+static double
+xconj (double x)
+{
+  return x;
+}
+
+SPARSE_MAPPER (erf, SparseMatrix::dmapper, ::erf)
+SPARSE_MAPPER (erfc, SparseMatrix::dmapper, ::erfc)
+SPARSE_MAPPER (gamma, SparseMatrix::dmapper, xgamma)
+CD_SPARSE_MAPPER (lgamma, xlgamma, xlgamma, 0.0, octave_Inf)
+SPARSE_MAPPER (abs, SparseMatrix::dmapper, ::fabs)
+SPARSE_MAPPER (acos, SparseMatrix::dmapper, ::acos)
+CD_SPARSE_MAPPER (acosh, ::acosh, ::acosh, 1.0, octave_Inf)
+SPARSE_MAPPER (angle, SparseMatrix::dmapper, ::arg)
+SPARSE_MAPPER (arg, SparseMatrix::dmapper, ::arg)
+CD_SPARSE_MAPPER (asin, ::asin, ::asin, -1.0, 1.0)
+SPARSE_MAPPER (asinh, SparseMatrix::dmapper, ::asinh)
+SPARSE_MAPPER (atan, SparseMatrix::dmapper, ::atan)
+CD_SPARSE_MAPPER (atanh, ::atanh, ::atanh, -1.0, 1.0)
+SPARSE_MAPPER (ceil, SparseMatrix::dmapper, ::ceil)
+SPARSE_MAPPER (conj, SparseMatrix::dmapper, xconj)
+SPARSE_MAPPER (cos, SparseMatrix::dmapper, ::cos)
+SPARSE_MAPPER (cosh, SparseMatrix::dmapper, ::cosh)
+SPARSE_MAPPER (exp, SparseMatrix::dmapper, ::exp)
+SPARSE_MAPPER (expm1, SparseMatrix::dmapper, ::expm1)
+SPARSE_MAPPER (fix, SparseMatrix::dmapper, ::fix)
+SPARSE_MAPPER (floor, SparseMatrix::dmapper, ::floor)
+SPARSE_MAPPER (imag, SparseMatrix::dmapper, ::imag)
+CD_SPARSE_MAPPER (log, ::log, std::log, 0.0, octave_Inf)
+CD_SPARSE_MAPPER (log2, xlog2, xlog2, 0.0, octave_Inf)
+CD_SPARSE_MAPPER (log10, ::log10, std::log10, 0.0, octave_Inf)
+CD_SPARSE_MAPPER (log1p, ::log1p, ::log1p, 0.0, octave_Inf)
+SPARSE_MAPPER (real, SparseMatrix::dmapper, ::real)
+SPARSE_MAPPER (round, SparseMatrix::dmapper, xround)
+SPARSE_MAPPER (roundb, SparseMatrix::dmapper, xroundb)
+SPARSE_MAPPER (signum, SparseMatrix::dmapper, ::signum)
+SPARSE_MAPPER (sin, SparseMatrix::dmapper, ::sin)
+SPARSE_MAPPER (sinh, SparseMatrix::dmapper, ::sinh)
+CD_SPARSE_MAPPER (sqrt, ::sqrt, std::sqrt, 0.0, octave_Inf)
+SPARSE_MAPPER (tan, SparseMatrix::dmapper, ::tan)
+SPARSE_MAPPER (tanh, SparseMatrix::dmapper, ::tanh)
+SPARSE_MAPPER (finite, SparseMatrix::bmapper, xfinite)
+SPARSE_MAPPER (isinf, SparseMatrix::bmapper, xisinf)
+SPARSE_MAPPER (isna, SparseMatrix::bmapper, octave_is_NA)
+SPARSE_MAPPER (isnan, SparseMatrix::bmapper, xisnan)
+
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
