@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1998, 2000, 2002, 2003, 2004, 2005, 2006,
+              2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -29,7 +29,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include "defun.h"
 #include "error.h"
-#include "oct-obj.h"
 
 const int
 octave_value_typeinfo::init_tab_sz (16);
@@ -41,34 +40,6 @@ octave_value_typeinfo::instance (0);
 #include <Array2.h>
 #include <Array3.h>
 
-#include <Array.cc>
-
-#undef OCTAVE_API
-#define OCTAVE_API
-
-INSTANTIATE_ARRAY (octave_value_typeinfo::unary_op_fcn);
-template class Array2<octave_value_typeinfo::unary_op_fcn>;
-
-INSTANTIATE_ARRAY (octave_value_typeinfo::non_const_unary_op_fcn);
-template class Array2<octave_value_typeinfo::non_const_unary_op_fcn>;
-
-INSTANTIATE_ARRAY (octave_value_typeinfo::binary_op_fcn);
-template class Array2<octave_value_typeinfo::binary_op_fcn>;
-template class Array3<octave_value_typeinfo::binary_op_fcn>;
-
-INSTANTIATE_ARRAY (octave_value_typeinfo::cat_op_fcn);
-template class Array2<octave_value_typeinfo::cat_op_fcn>;
-
-INSTANTIATE_ARRAY (octave_value_typeinfo::assign_op_fcn);
-template class Array2<octave_value_typeinfo::assign_op_fcn>;
-template class Array3<octave_value_typeinfo::assign_op_fcn>;
-
-INSTANTIATE_ARRAY (octave_value_typeinfo::assignany_op_fcn);
-template class Array2<octave_value_typeinfo::assignany_op_fcn>;
-
-INSTANTIATE_ARRAY (octave_base_value::type_conv_fcn);
-template class Array2<octave_base_value::type_conv_fcn>;
-
 bool
 octave_value_typeinfo::instance_ok (void)
 {
@@ -95,6 +66,14 @@ octave_value_typeinfo::register_type (const std::string& t_name,
     ? instance->do_register_type (t_name, c_name, val) : -1;
 }
 
+bool
+octave_value_typeinfo::register_unary_class_op (octave_value::unary_op op,
+					        octave_value_typeinfo::unary_class_op_fcn f)
+{
+  return (instance_ok ())
+    ? instance->do_register_unary_class_op (op, f) : false;
+}
+
 bool
 octave_value_typeinfo::register_unary_op (octave_value::unary_op op,
 					   int t, octave_value_typeinfo::unary_op_fcn f)
@@ -112,6 +91,14 @@ octave_value_typeinfo::register_non_const_unary_op (octave_value::unary_op op,
     ? instance->do_register_non_const_unary_op (op, t, f) : false;
 }
 
+bool
+octave_value_typeinfo::register_binary_class_op (octave_value::binary_op op,
+						 octave_value_typeinfo::binary_class_op_fcn f)
+{
+  return (instance_ok ())
+    ? instance->do_register_binary_class_op (op, f) : false;
+}
+
 bool
 octave_value_typeinfo::register_binary_op (octave_value::binary_op op,
 					   int t1, int t2,
@@ -121,6 +108,23 @@ octave_value_typeinfo::register_binary_op (octave_value::binary_op op,
     ? instance->do_register_binary_op (op, t1, t2, f) : false;
 }
 
+bool
+octave_value_typeinfo::register_binary_class_op (octave_value::compound_binary_op op,
+						 octave_value_typeinfo::binary_class_op_fcn f)
+{
+  return (instance_ok ())
+    ? instance->do_register_binary_class_op (op, f) : false;
+}
+
+bool
+octave_value_typeinfo::register_binary_op (octave_value::compound_binary_op op,
+					   int t1, int t2,
+					   octave_value_typeinfo::binary_op_fcn f)
+{
+  return (instance_ok ())
+    ? instance->do_register_binary_op (op, t1, t2, f) : false;
+}
+
 bool
 octave_value_typeinfo::register_cat_op (int t1, int t2, octave_value_typeinfo::cat_op_fcn f)
 {
@@ -174,7 +178,7 @@ octave_value_typeinfo::register_widening_op (int t, int t_result,
 
 int
 octave_value_typeinfo::do_register_type (const std::string& t_name,
-					 const std::string& c_name,
+					 const std::string& /* c_name */,
 					 const octave_value& val)
 {
   int i = 0;
@@ -193,29 +197,28 @@ octave_value_typeinfo::do_register_type (const std::string& t_name,
 
       vals.resize (len, octave_value ());
 
-      unary_ops.resize (static_cast<int> (octave_value::num_unary_ops),
-			len, static_cast<octave_value_typeinfo::unary_op_fcn> (0));
+      unary_ops.resize (static_cast<int> (octave_value::num_unary_ops), len, 0);
 
       non_const_unary_ops.resize
-	(static_cast<int> (octave_value::num_unary_ops),
-	 len, static_cast<octave_value_typeinfo::non_const_unary_op_fcn> (0));
+	(static_cast<int> (octave_value::num_unary_ops), len, 0);
 
       binary_ops.resize (static_cast<int> (octave_value::num_binary_ops),
-			 len, len, static_cast<octave_value_typeinfo::binary_op_fcn> (0));
+			 len, len, 0);
 
-      cat_ops.resize (len, len, static_cast<octave_value_typeinfo::cat_op_fcn> (0));
+      compound_binary_ops.resize (static_cast<int> (octave_value::num_compound_binary_ops),
+                                  len, len, 0);
 
-      assign_ops.resize (static_cast<int> (octave_value::num_assign_ops),
-			 len, len, static_cast<octave_value_typeinfo::assign_op_fcn> (0));
+      cat_ops.resize (len, len, 0);
 
-      assignany_ops.resize (static_cast<int> (octave_value::num_assign_ops),
-			    len, static_cast<octave_value_typeinfo::assignany_op_fcn> (0));
+      assign_ops.resize (static_cast<int> (octave_value::num_assign_ops), len, len, 0);
+
+      assignany_ops.resize (static_cast<int> (octave_value::num_assign_ops), len, 0);
 
       pref_assign_conv.resize (len, len, -1);
 
-      type_conv_ops.resize (len, len, static_cast<octave_base_value::type_conv_fcn> (0));
+      type_conv_ops.resize (len, len, 0);
 
-      widening_ops.resize (len, len, static_cast<octave_base_value::type_conv_fcn> (0));
+      widening_ops.resize (len, len, 0);
     }
 
   types (i) = t_name;
@@ -227,6 +230,23 @@ octave_value_typeinfo::do_register_type (const std::string& t_name,
   return i;
 }
 
+bool
+octave_value_typeinfo::do_register_unary_class_op (octave_value::unary_op op,
+					           octave_value_typeinfo::unary_class_op_fcn f)
+{
+  if (lookup_unary_class_op (op))
+    {
+      std::string op_name = octave_value::unary_op_as_string (op);
+
+      warning ("duplicate unary operator `%s' for class dispatch",
+	       op_name.c_str ());
+    }
+
+  unary_class_ops.checkelem (static_cast<int> (op)) = reinterpret_cast<void *> (f);
+
+  return false;
+}
+
 bool
 octave_value_typeinfo::do_register_unary_op (octave_value::unary_op op,
 					     int t, octave_value_typeinfo::unary_op_fcn f)
@@ -240,7 +260,7 @@ octave_value_typeinfo::do_register_unary_op (octave_value::unary_op op,
 	       op_name.c_str (), type_name.c_str ());
     }
 
-  unary_ops.checkelem (static_cast<int> (op), t) = f;
+  unary_ops.checkelem (static_cast<int> (op), t) = reinterpret_cast<void *> (f);
 
   return false;
 }
@@ -258,7 +278,24 @@ octave_value_typeinfo::do_register_non_const_unary_op
 	       op_name.c_str (), type_name.c_str ());
     }
 
-  non_const_unary_ops.checkelem (static_cast<int> (op), t) = f;
+  non_const_unary_ops.checkelem (static_cast<int> (op), t) = reinterpret_cast<void *> (f);
+
+  return false;
+}
+
+bool
+octave_value_typeinfo::do_register_binary_class_op (octave_value::binary_op op,
+						    octave_value_typeinfo::binary_class_op_fcn f)
+{
+  if (lookup_binary_class_op (op))
+    {
+      std::string op_name = octave_value::binary_op_as_string (op);
+
+      warning ("duplicate binary operator `%s' for class dispatch",
+	       op_name.c_str ());
+    }
+
+  binary_class_ops.checkelem (static_cast<int> (op)) = reinterpret_cast<void *> (f);
 
   return false;
 }
@@ -278,7 +315,44 @@ octave_value_typeinfo::do_register_binary_op (octave_value::binary_op op,
 	       op_name.c_str (), t1_name.c_str (), t1_name.c_str ());
     }
 
-  binary_ops.checkelem (static_cast<int> (op), t1, t2) = f;
+  binary_ops.checkelem (static_cast<int> (op), t1, t2) = reinterpret_cast<void *> (f);
+
+  return false;
+}
+
+bool
+octave_value_typeinfo::do_register_binary_class_op (octave_value::compound_binary_op op,
+						    octave_value_typeinfo::binary_class_op_fcn f)
+{
+  if (lookup_binary_class_op (op))
+    {
+      std::string op_name = octave_value::binary_op_fcn_name (op);
+
+      warning ("duplicate compound binary operator `%s' for class dispatch",
+	       op_name.c_str ());
+    }
+
+  compound_binary_class_ops.checkelem (static_cast<int> (op)) = reinterpret_cast<void *> (f);
+
+  return false;
+}
+
+bool
+octave_value_typeinfo::do_register_binary_op (octave_value::compound_binary_op op,
+					      int t1, int t2,
+					      octave_value_typeinfo::binary_op_fcn f)
+{
+  if (lookup_binary_op (op, t1, t2))
+    {
+      std::string op_name = octave_value::binary_op_fcn_name (op);
+      std::string t1_name = types(t1);
+      std::string t2_name = types(t2);
+
+      warning ("duplicate compound binary operator `%s' for types `%s' and `%s'",
+	       op_name.c_str (), t1_name.c_str (), t1_name.c_str ());
+    }
+
+  compound_binary_ops.checkelem (static_cast<int> (op), t1, t2) = reinterpret_cast<void *> (f);
 
   return false;
 }
@@ -295,7 +369,7 @@ octave_value_typeinfo::do_register_cat_op (int t1, int t2, octave_value_typeinfo
 	       t1_name.c_str (), t1_name.c_str ());
     }
 
-  cat_ops.checkelem (t1, t2) = f;
+  cat_ops.checkelem (t1, t2) = reinterpret_cast<void *> (f);
 
   return false;
 }
@@ -315,7 +389,7 @@ octave_value_typeinfo::do_register_assign_op (octave_value::assign_op op,
 	       op_name.c_str (), t_lhs_name.c_str (), t_rhs_name.c_str ());
     }
 
-  assign_ops.checkelem (static_cast<int> (op), t_lhs, t_rhs) = f;
+  assign_ops.checkelem (static_cast<int> (op), t_lhs, t_rhs) = reinterpret_cast<void *> (f);
 
   return false;
 }
@@ -333,7 +407,7 @@ octave_value_typeinfo::do_register_assignany_op (octave_value::assign_op op,
 	       op_name.c_str (), t_lhs_name.c_str ());
     }
 
-  assignany_ops.checkelem (static_cast<int> (op), t_lhs) = f;
+  assignany_ops.checkelem (static_cast<int> (op), t_lhs) = reinterpret_cast<void *> (f);
 
   return false;
 }
@@ -369,7 +443,7 @@ octave_value_typeinfo::do_register_type_conv_op
 	       t_name.c_str (), t_result_name.c_str ());
     }
 
-  type_conv_ops.checkelem (t, t_result) = f;
+  type_conv_ops.checkelem (t, t_result) = reinterpret_cast<void *> (f);
 
   return false;
 }
@@ -387,7 +461,7 @@ octave_value_typeinfo::do_register_widening_op
 	       t_name.c_str (), t_result_name.c_str ());
     }
 
-  widening_ops.checkelem (t, t_result) = f;
+  widening_ops.checkelem (t, t_result) = reinterpret_cast<void *> (f);
 
   return false;
 }
@@ -410,44 +484,79 @@ octave_value_typeinfo::do_lookup_type (const std::string& nm)
   return retval;
 }
 
+octave_value_typeinfo::unary_class_op_fcn
+octave_value_typeinfo::do_lookup_unary_class_op (octave_value::unary_op op)
+{
+  void *f = unary_class_ops.checkelem (static_cast<int> (op));
+  return reinterpret_cast<octave_value_typeinfo::unary_class_op_fcn> (f);
+}
+
 octave_value_typeinfo::unary_op_fcn
 octave_value_typeinfo::do_lookup_unary_op (octave_value::unary_op op, int t)
 {
-  return unary_ops.checkelem (static_cast<int> (op), t);
+  void *f = unary_ops.checkelem (static_cast<int> (op), t);
+  return reinterpret_cast<octave_value_typeinfo::unary_op_fcn> (f);
 }
 
 octave_value_typeinfo::non_const_unary_op_fcn
 octave_value_typeinfo::do_lookup_non_const_unary_op
   (octave_value::unary_op op, int t)
 {
-  return non_const_unary_ops.checkelem (static_cast<int> (op), t);
+  void *f = non_const_unary_ops.checkelem (static_cast<int> (op), t);
+  return reinterpret_cast<octave_value_typeinfo::non_const_unary_op_fcn> (f);
+}
+
+octave_value_typeinfo::binary_class_op_fcn
+octave_value_typeinfo::do_lookup_binary_class_op (octave_value::binary_op op)
+{
+  void *f = binary_class_ops.checkelem (static_cast<int> (op));
+  return reinterpret_cast<octave_value_typeinfo::binary_class_op_fcn> (f);
 }
 
 octave_value_typeinfo::binary_op_fcn
 octave_value_typeinfo::do_lookup_binary_op (octave_value::binary_op op,
 					    int t1, int t2)
 {
-  return binary_ops.checkelem (static_cast<int> (op), t1, t2);
+  void *f = binary_ops.checkelem (static_cast<int> (op), t1, t2);
+  return reinterpret_cast<octave_value_typeinfo::binary_op_fcn> (f);
+}
+
+octave_value_typeinfo::binary_class_op_fcn
+octave_value_typeinfo::do_lookup_binary_class_op (octave_value::compound_binary_op op)
+{
+  void *f = compound_binary_class_ops.checkelem (static_cast<int> (op));
+  return reinterpret_cast<octave_value_typeinfo::binary_class_op_fcn> (f);
+}
+
+octave_value_typeinfo::binary_op_fcn
+octave_value_typeinfo::do_lookup_binary_op (octave_value::compound_binary_op op,
+					    int t1, int t2)
+{
+  void *f = compound_binary_ops.checkelem (static_cast<int> (op), t1, t2);
+  return reinterpret_cast<octave_value_typeinfo::binary_op_fcn> (f);
 }
 
 octave_value_typeinfo::cat_op_fcn
 octave_value_typeinfo::do_lookup_cat_op (int t1, int t2)
 {
-  return cat_ops.checkelem (t1, t2);
+  void *f = cat_ops.checkelem (t1, t2);
+  return reinterpret_cast<octave_value_typeinfo::cat_op_fcn> (f);
 }
 
 octave_value_typeinfo::assign_op_fcn
 octave_value_typeinfo::do_lookup_assign_op (octave_value::assign_op op,
 					    int t_lhs, int t_rhs)
 {
-  return assign_ops.checkelem (static_cast<int> (op), t_lhs, t_rhs);
+  void *f = assign_ops.checkelem (static_cast<int> (op), t_lhs, t_rhs);
+  return reinterpret_cast<octave_value_typeinfo::assign_op_fcn> (f);
 }
 
 octave_value_typeinfo::assignany_op_fcn
 octave_value_typeinfo::do_lookup_assignany_op (octave_value::assign_op op,
 					       int t_lhs)
 {
-  return assignany_ops.checkelem (static_cast<int> (op), t_lhs);
+  void *f = assignany_ops.checkelem (static_cast<int> (op), t_lhs);
+  return reinterpret_cast<octave_value_typeinfo::assignany_op_fcn> (f);
 }
 
 int
@@ -459,13 +568,15 @@ octave_value_typeinfo::do_lookup_pref_assign_conv (int t_lhs, int t_rhs)
 octave_base_value::type_conv_fcn
 octave_value_typeinfo::do_lookup_type_conv_op (int t, int t_result)
 {
-  return type_conv_ops.checkelem (t, t_result);
+  void *f = type_conv_ops.checkelem (t, t_result);
+  return reinterpret_cast<octave_base_value::type_conv_fcn> (f);
 }
 
 octave_base_value::type_conv_fcn
 octave_value_typeinfo::do_lookup_widening_op (int t, int t_result)
 {
-  return widening_ops.checkelem (t, t_result);
+  void *f = widening_ops.checkelem (t, t_result);
+  return reinterpret_cast<octave_base_value::type_conv_fcn> (f);
 }
 
 string_vector
@@ -484,7 +595,7 @@ DEFUN (typeinfo, args, ,
 @deftypefn {Built-in Function} {} typeinfo (@var{expr})\n\
 \n\
 Return the type of the expression @var{expr}, as a string.  If\n\
-@var{EXPR}  is omitted, return an array of strings containing all the\n\
+@var{expr} is omitted, return an array of strings containing all the\n\
 currently installed data types.\n\
 @end deftypefn")
 {
@@ -502,25 +613,6 @@ currently installed data types.\n\
   return retval;
 }
 
-DEFUN (class, args, ,
-  "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} class (@var{expr})\n\
-\n\
-Return the class of the expression @var{expr}, as a string.\n\
-@end deftypefn")
-{
-  octave_value retval;
-
-  int nargin = args.length ();
-
-  if (nargin == 1)
-    retval = args(0).class_name ();
-  else
-    print_usage ();
-
-  return retval;
-}
-
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
