@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 1999, 2000, 2002, 2003, 2004, 2005,
-              2006, 2007 John W. Eaton
+              2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -26,14 +26,10 @@ along with Octave; see the file COPYING.  If not, see
 
 #include <cstdlib>
 
-#include <iostream>
+#include <iosfwd>
 #include <string>
 #include <list>
 
-#if defined (HAVE_HDF5)
-#include <hdf5.h>
-#endif
-
 #include "Range.h"
 #include "data-conv.h"
 #include "idx-vector.h"
@@ -44,11 +40,12 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-time.h"
 #include "str-vec.h"
 
+#include "oct-hdf5.h"
+#include "oct-sort.h"
+
 class Cell;
-class streamoff_array;
 class Octave_map;
 class octave_stream;
-class octave_streamoff;
 class octave_function;
 class octave_user_function;
 class octave_fcn_handle;
@@ -108,6 +105,21 @@ public:
     unknown_binary_op
   };
 
+  enum compound_binary_op
+  {
+    // ** compound operations **
+    op_trans_mul,      
+    op_mul_trans,
+    op_herm_mul,      
+    op_mul_herm,
+    op_el_not_and,
+    op_el_not_or,
+    op_el_and_not,
+    op_el_or_not,
+    num_compound_binary_ops,
+    unknown_compound_binary_op
+  };
+
   enum assign_op
   {
     op_asn_eq,
@@ -130,8 +142,12 @@ public:
   };
 
   static std::string unary_op_as_string (unary_op);
+  static std::string unary_op_fcn_name (unary_op);
 
   static std::string binary_op_as_string (binary_op);
+  static std::string binary_op_fcn_name (binary_op);
+
+  static std::string binary_op_fcn_name (compound_binary_op);
 
   static std::string assign_op_as_string (assign_op);
 
@@ -140,7 +156,13 @@ public:
 
   enum magic_colon { magic_colon_t };
 
-  octave_value (void);
+  octave_value (void)
+    {
+      static octave_base_value nil_rep;
+      rep = &nil_rep;
+      rep->count++;
+    }
+
   octave_value (short int i);
   octave_value (unsigned short int i);
   octave_value (int i);
@@ -163,24 +185,40 @@ public:
 
   octave_value (octave_time t);
   octave_value (double d);
+  octave_value (float d);
   octave_value (const ArrayN<octave_value>& a, bool is_cs_list = false);
   octave_value (const Cell& c, bool is_cs_list = false);
   octave_value (const Matrix& m, const MatrixType& t = MatrixType());
+  octave_value (const FloatMatrix& m, const MatrixType& t = MatrixType());
   octave_value (const NDArray& nda);
+  octave_value (const FloatNDArray& nda);
   octave_value (const ArrayN<double>& m);
+  octave_value (const ArrayN<float>& m);
   octave_value (const DiagMatrix& d);
+  octave_value (const FloatDiagMatrix& d);
   octave_value (const RowVector& v);
+  octave_value (const FloatRowVector& v);
   octave_value (const ColumnVector& v);
+  octave_value (const FloatColumnVector& v);
   octave_value (const Complex& C);
+  octave_value (const FloatComplex& C);
   octave_value (const ComplexMatrix& m, const MatrixType& t = MatrixType());
+  octave_value (const FloatComplexMatrix& m, const MatrixType& t = MatrixType());
   octave_value (const ComplexNDArray& cnda);
+  octave_value (const FloatComplexNDArray& cnda);
   octave_value (const ArrayN<Complex>& m);
+  octave_value (const ArrayN<FloatComplex>& m);
   octave_value (const ComplexDiagMatrix& d);
+  octave_value (const FloatComplexDiagMatrix& d);
   octave_value (const ComplexRowVector& v);
+  octave_value (const FloatComplexRowVector& v);
   octave_value (const ComplexColumnVector& v);
+  octave_value (const FloatComplexColumnVector& v);
+  octave_value (const PermMatrix& p);
   octave_value (bool b);
   octave_value (const boolMatrix& bm, const MatrixType& t = MatrixType());
   octave_value (const boolNDArray& bnda);
+  octave_value (const ArrayN<bool>& bnda);
   octave_value (char c, char type = '"');
   octave_value (const char *s, char type = '"');
   octave_value (const std::string& s, char type = '"');
@@ -198,6 +236,7 @@ public:
   octave_value (const Sparse<Complex>& m, const MatrixType& t = MatrixType ());
   octave_value (const SparseBoolMatrix& bm, 
 		const MatrixType& t = MatrixType ());
+  octave_value (const Sparse<bool>& m, const MatrixType& t = MatrixType ());
   octave_value (const octave_int8& i);
   octave_value (const octave_int16& i);
   octave_value (const octave_int32& i);
@@ -225,11 +264,12 @@ public:
   octave_value (double base, double limit, double inc);
   octave_value (const Range& r);
   octave_value (const Octave_map& m);
-  octave_value (const streamoff_array& off);
+  octave_value (const Octave_map& m, const std::string& id);
   octave_value (const octave_value_list& m, bool is_cs_list = false);
   octave_value (octave_value::magic_colon);
 
   octave_value (octave_base_value *new_rep);
+  octave_value (octave_base_value *new_rep, int xcount);
 
   // Copy constructor.
 
@@ -265,6 +305,19 @@ public:
 	}
     }
 
+  // This uniquifies the value if it is referenced by more than a certain
+  // number of shallow copies. This is useful for optimizations where we
+  // know a certain copy, typically within a cell array, to be obsolete.
+  void make_unique (int obsolete_copies)
+    {
+      if (rep->count > obsolete_copies + 1)
+	{
+	  --rep->count;
+	  rep = rep->clone ();
+	  rep->count = 1;
+	}
+    }
+
   // Simple assignment.
 
   octave_value& operator = (const octave_value& a)
@@ -283,14 +336,21 @@ public:
 
   int get_count (void) const { return rep->count; }
 
-  octave_base_value::type_conv_fcn numeric_conversion_function (void) const
+  octave_base_value::type_conv_info numeric_conversion_function (void) const
     { return rep->numeric_conversion_function (); }
 
+  octave_base_value::type_conv_info numeric_demotion_function (void) const
+    { return rep->numeric_demotion_function (); }
+
   void maybe_mutate (void);
 
   octave_value squeeze (void) const
     { return rep->squeeze (); }
 
+  // The result of full().
+  octave_value full_value (void) const
+    { return rep->full_value (); }
+
   octave_base_value *try_narrowing_conversion (void)
     { return rep->try_narrowing_conversion (); }
 
@@ -298,12 +358,17 @@ public:
 			       const octave_value_list& idx);
 
   octave_value subsref (const std::string& type,
-				const std::list<octave_value_list>& idx)
+                        const std::list<octave_value_list>& idx)
     { return rep->subsref (type, idx); }
 
+  octave_value subsref (const std::string& type,
+                        const std::list<octave_value_list>& idx,
+                        bool auto_add)
+    { return rep->subsref (type, idx, auto_add); }
+
   octave_value_list subsref (const std::string& type,
-				     const std::list<octave_value_list>& idx,
-    				     int nargout);
+                             const std::list<octave_value_list>& idx,
+                             int nargout);
 
   octave_value next_subsref (const std::string& type, const
 			     std::list<octave_value_list>& idx,
@@ -314,6 +379,10 @@ public:
 				  std::list<octave_value_list>& idx,
 				  size_t skip = 1);
 
+  octave_value next_subsref (bool auto_add, const std::string& type, const
+			     std::list<octave_value_list>& idx,
+			     size_t skip = 1);
+
   octave_value do_index_op (const octave_value_list& idx,
 			    bool resize_ok = false)
     { return rep->do_index_op (idx, resize_ok); }
@@ -427,6 +496,12 @@ public:
   bool is_char_matrix (void) const
     { return rep->is_char_matrix (); }
 
+  bool is_diag_matrix (void) const
+    { return rep->is_diag_matrix (); }
+
+  bool is_perm_matrix (void) const
+    { return rep->is_perm_matrix (); }
+
   bool is_string (void) const
     { return rep->is_string (); }
 
@@ -442,8 +517,8 @@ public:
   bool is_map (void) const
     { return rep->is_map (); }
 
-  bool is_streamoff (void) const
-    { return rep->is_streamoff (); }
+  bool is_object (void) const
+    { return rep->is_object (); }
 
   bool is_cs_list (void) const
     { return rep->is_cs_list (); }
@@ -454,6 +529,9 @@ public:
   bool is_magic_colon (void) const
     { return rep->is_magic_colon (); }
 
+  bool is_null_value (void) const
+    { return rep->is_null_value (); }
+
   // Are any or all of the elements in this constant nonzero?
 
   octave_value all (int dim = 0) const
@@ -470,6 +548,9 @@ public:
   bool is_single_type (void) const
     { return rep->is_single_type (); }
 
+  bool is_float_type (void) const
+    { return rep->is_float_type (); }
+
   // Integer types.
 
   bool is_int8_type (void) const
@@ -522,17 +603,15 @@ public:
   bool is_sparse_type (void) const
     { return rep->is_sparse_type (); }
 
-  bool valid_as_scalar_index (void) const
-    { return rep->valid_as_scalar_index (); }
-
-  bool valid_as_zero_index (void) const
-    { return rep->valid_as_zero_index (); }
-
   // Does this constant correspond to a truth value?
 
   bool is_true (void) const
     { return rep->is_true (); }
 
+  // Do two constants match (in a switch statement)?
+  
+  bool is_equal (const octave_value&) const;
+
   // Are the dimensions of this constant zero by zero?
 
   bool is_zero_by_zero (void) const
@@ -550,6 +629,15 @@ public:
   bool is_function (void) const
     { return rep->is_function (); }
 
+  bool is_user_script (void) const
+    { return rep->is_user_script (); }
+
+  bool is_user_function (void) const
+    { return rep->is_user_function (); }
+
+  bool is_user_code (void) const
+    { return rep->is_user_code (); }
+
   bool is_builtin_function (void) const
     { return rep->is_builtin_function (); }
 
@@ -559,6 +647,8 @@ public:
   bool is_mex_function (void) const
     { return rep->is_mex_function (); }
 
+  void erase_subfunctions (void) { rep->erase_subfunctions (); }
+
   // Values.
 
   octave_value eval (void) { return *this; }
@@ -590,40 +680,52 @@ public:
     { return rep->ulong_value (req_int, frc_str_conv); }
 
   octave_idx_type
-  idx_type_value (bool req_int = false, bool frc_str_conv = false) const
-    {
-#if SIZEOF_OCTAVE_IDX_TYPE == SIZEOF_LONG
-      return long_value (req_int, frc_str_conv);
-#elif SIZEOF_OCTAVE_IDX_TYPE == SIZEOF_INT
-      return int_value (req_int, frc_str_conv);
-#else
-#error "no octave_value extractor for octave_idx_type"
-#endif
-    }
+  idx_type_value (bool req_int = false, bool frc_str_conv = false) const;
 
   double double_value (bool frc_str_conv = false) const
     { return rep->double_value (frc_str_conv); }
 
+  float float_value (bool frc_str_conv = false) const
+    { return rep->float_value (frc_str_conv); }
+
   double scalar_value (bool frc_str_conv = false) const
     { return rep->scalar_value (frc_str_conv); }
 
+  float float_scalar_value (bool frc_str_conv = false) const
+    { return rep->float_scalar_value (frc_str_conv); }
+
   Cell cell_value (void) const;
 
   Matrix matrix_value (bool frc_str_conv = false) const
     { return rep->matrix_value (frc_str_conv); }
 
+  FloatMatrix float_matrix_value (bool frc_str_conv = false) const
+    { return rep->float_matrix_value (frc_str_conv); }
+
   NDArray array_value (bool frc_str_conv = false) const
     { return rep->array_value (frc_str_conv); }
 
+  FloatNDArray float_array_value (bool frc_str_conv = false) const
+    { return rep->float_array_value (frc_str_conv); }
+
   Complex complex_value (bool frc_str_conv = false) const
     { return rep->complex_value (frc_str_conv); }
 
+  FloatComplex float_complex_value (bool frc_str_conv = false) const
+    { return rep->float_complex_value (frc_str_conv); }
+
   ComplexMatrix complex_matrix_value (bool frc_str_conv = false) const
     { return rep->complex_matrix_value (frc_str_conv); }
 
+  FloatComplexMatrix float_complex_matrix_value (bool frc_str_conv = false) const
+    { return rep->float_complex_matrix_value (frc_str_conv); }
+
   ComplexNDArray complex_array_value (bool frc_str_conv = false) const
     { return rep->complex_array_value (frc_str_conv); }
 
+  FloatComplexNDArray float_complex_array_value (bool frc_str_conv = false) const
+    { return rep->float_complex_array_value (frc_str_conv); }
+
   bool bool_value (bool warn = false) const
     { return rep->bool_value (warn); }
 
@@ -648,6 +750,21 @@ public:
   SparseBoolMatrix sparse_bool_matrix_value (bool frc_str_conv = false) const
     { return rep->sparse_bool_matrix_value (frc_str_conv); }
 
+  DiagMatrix diag_matrix_value (bool force = false) const
+    { return rep->diag_matrix_value (force); }
+  
+  FloatDiagMatrix float_diag_matrix_value (bool force = false) const
+    { return rep->float_diag_matrix_value (force); }
+  
+  ComplexDiagMatrix complex_diag_matrix_value (bool force = false) const
+    { return rep->complex_diag_matrix_value (force); }
+  
+  FloatComplexDiagMatrix float_complex_diag_matrix_value (bool force = false) const
+    { return rep->float_complex_diag_matrix_value (force); }
+
+  PermMatrix perm_matrix_value (void) const
+    { return rep->perm_matrix_value (); }
+  
   octave_int8 int8_scalar_value (void) const
     { return rep->int8_scalar_value (); }
 
@@ -702,6 +819,9 @@ public:
   std::string string_value (bool force = false) const
     { return rep->string_value (force); }
 
+  Array<std::string> cellstr_value (void) const
+    { return rep->cellstr_value (); }
+
   Range range_value (void) const
     { return rep->range_value (); }
 
@@ -710,14 +830,29 @@ public:
   string_vector map_keys (void) const
     { return rep->map_keys (); }
 
-  std::streamoff streamoff_value (void) const;
+  size_t nparents (void) const
+    { return rep->nparents (); }
 
-  streamoff_array streamoff_array_value (void) const;
+  std::list<std::string> parent_class_name_list (void) const
+    { return rep->parent_class_name_list (); }
+
+  string_vector parent_class_names (void) const
+    { return rep->parent_class_names (); }
+
+  octave_base_value *
+  find_parent_class (const std::string& parent_class_name)
+    { return rep->find_parent_class (parent_class_name); }
 
   octave_function *function_value (bool silent = false);
 
+  const octave_function *function_value (bool silent = false) const;
+
   octave_user_function *user_function_value (bool silent = false);
 
+  octave_user_script *user_script_value (bool silent = false);
+
+  octave_user_code *user_code_value (bool silent = false);
+
   octave_fcn_handle *fcn_handle_value (bool silent = false);
 
   octave_fcn_inline *fcn_inline_value (bool silent = false);
@@ -738,16 +873,61 @@ public:
   complex_row_vector_value (bool frc_str_conv = false,
 			    bool frc_vec_conv = false) const;
 
+
+  FloatColumnVector float_column_vector_value (bool frc_str_conv = false,
+			     bool frc_vec_conv = false) const;
+
+  FloatComplexColumnVector
+  float_complex_column_vector_value (bool frc_str_conv = false,
+			bool frc_vec_conv = false) const;
+
+  FloatRowVector float_row_vector_value (bool frc_str_conv = false,
+			      bool frc_vec_conv = false) const;
+
+  FloatComplexRowVector
+  float_complex_row_vector_value (bool frc_str_conv = false,
+			    bool frc_vec_conv = false) const;
+
+
+
+
   Array<int> int_vector_value (bool req_int = false,
 			       bool frc_str_conv = false,
 			       bool frc_vec_conv = false) const;
 
+  Array<octave_idx_type>
+  octave_idx_type_vector_value (bool req_int = false,
+				bool frc_str_conv = false,
+				bool frc_vec_conv = false) const;
+
   Array<double> vector_value (bool frc_str_conv = false,
 			      bool frc_vec_conv = false) const;
 
   Array<Complex> complex_vector_value (bool frc_str_conv = false,
 				       bool frc_vec_conv = false) const;
 
+  Array<float> float_vector_value (bool frc_str_conv = false,
+			      bool frc_vec_conv = false) const;
+
+  Array<FloatComplex> float_complex_vector_value (bool frc_str_conv = false,
+				       bool frc_vec_conv = false) const;
+
+  // Possibly economize a lazy-indexed value.
+
+  void maybe_economize (void)
+    { rep->maybe_economize (); }
+
+  // The following two hook conversions are called on any octave_value prior to
+  // storing it to a "permanent" location, like a named variable, a cell or a
+  // struct component, or a return value of a function. 
+
+  octave_value storable_value (void) const;
+
+  // Ditto, but in place, i.e. equivalent to *this = this->storable_value (),
+  // but possibly more efficient.
+
+  void make_storable_value (void);
+
   // Conversions.  These should probably be private.  If a user of this
   // class wants a certain kind of constant, he should simply ask for
   // it, and we should convert it if possible.
@@ -802,12 +982,18 @@ public:
 				    const octave_value& a,
 				    const octave_value& b);
 
+  friend OCTINTERP_API octave_value do_binary_op (compound_binary_op op,
+                                                  const octave_value& a,
+                                                  const octave_value& b);
+
   friend OCTINTERP_API octave_value do_cat_op (const octave_value& a,
 				 const octave_value& b,
 				 const Array<octave_idx_type>& ra_idx);
 
   const octave_base_value& get_rep (void) const { return *rep; }
 
+  bool is_copy_of (const octave_value &val) const { return rep == val.rep; }
+
   void print_info (std::ostream& os,
 			   const std::string& prefix = std::string ()) const;
 
@@ -847,6 +1033,96 @@ public:
 
   mxArray *as_mxArray (void) const { return rep->as_mxArray (); }
 
+  octave_value diag (octave_idx_type k = 0) const
+    { return rep->diag (k); }
+
+  octave_value sort (octave_idx_type dim = 0, sortmode mode = ASCENDING) const
+    { return rep->sort (dim, mode); } 
+  octave_value sort (Array<octave_idx_type> &sidx, octave_idx_type dim = 0,
+		 sortmode mode = ASCENDING) const
+    { return rep->sort (sidx, dim, mode); } 
+
+  sortmode is_sorted (sortmode mode = UNSORTED) const
+    { return rep->is_sorted (mode); }
+
+  Array<octave_idx_type> sort_rows_idx (sortmode mode = ASCENDING) const
+    { return rep->sort_rows_idx (mode); }
+
+  sortmode is_sorted_rows (sortmode mode = UNSORTED) const
+    { return rep->is_sorted_rows (mode); }
+
+  void lock (void) { rep->lock (); }
+
+  void unlock (void) { rep->unlock (); }
+
+  bool islocked (void) const { return rep->islocked (); }
+
+  void dump (std::ostream& os) const { rep->dump (os); }
+
+#define MAPPER_FORWARD(F) \
+  octave_value F (void) const { return rep->F (); }
+
+  MAPPER_FORWARD (abs)
+  MAPPER_FORWARD (acos)
+  MAPPER_FORWARD (acosh)
+  MAPPER_FORWARD (angle)
+  MAPPER_FORWARD (arg)
+  MAPPER_FORWARD (asin)
+  MAPPER_FORWARD (asinh)
+  MAPPER_FORWARD (atan)
+  MAPPER_FORWARD (atanh)
+  MAPPER_FORWARD (ceil)
+  MAPPER_FORWARD (conj)
+  MAPPER_FORWARD (cos)
+  MAPPER_FORWARD (cosh)
+  MAPPER_FORWARD (erf)
+  MAPPER_FORWARD (erfc)
+  MAPPER_FORWARD (exp)
+  MAPPER_FORWARD (expm1)
+  MAPPER_FORWARD (finite)
+  MAPPER_FORWARD (fix)
+  MAPPER_FORWARD (floor)
+  MAPPER_FORWARD (gamma)
+  MAPPER_FORWARD (imag)
+  MAPPER_FORWARD (isinf)
+  MAPPER_FORWARD (isna)
+  MAPPER_FORWARD (isnan)
+  MAPPER_FORWARD (lgamma)
+  MAPPER_FORWARD (log)
+  MAPPER_FORWARD (log2)
+  MAPPER_FORWARD (log10)
+  MAPPER_FORWARD (log1p)
+  MAPPER_FORWARD (real)
+  MAPPER_FORWARD (round)
+  MAPPER_FORWARD (roundb)
+  MAPPER_FORWARD (signum)
+  MAPPER_FORWARD (sin)
+  MAPPER_FORWARD (sinh)
+  MAPPER_FORWARD (sqrt)
+  MAPPER_FORWARD (tan)
+  MAPPER_FORWARD (tanh)
+
+  // These functions are prefixed with X to avoid potential macro
+  // conflicts.
+
+  MAPPER_FORWARD (xisalnum)
+  MAPPER_FORWARD (xisalpha)
+  MAPPER_FORWARD (xisascii)
+  MAPPER_FORWARD (xiscntrl)
+  MAPPER_FORWARD (xisdigit)
+  MAPPER_FORWARD (xisgraph)
+  MAPPER_FORWARD (xislower)
+  MAPPER_FORWARD (xisprint)
+  MAPPER_FORWARD (xispunct)
+  MAPPER_FORWARD (xisspace)
+  MAPPER_FORWARD (xisupper)
+  MAPPER_FORWARD (xisxdigit)
+  MAPPER_FORWARD (xtoascii)
+  MAPPER_FORWARD (xtolower)
+  MAPPER_FORWARD (xtoupper)
+
+#undef MAPPER_FORWARD
+
 protected:
 
   // The real representation.
@@ -870,6 +1146,10 @@ extern OCTINTERP_API octave_value
 do_binary_op (octave_value::binary_op op,
 	      const octave_value& a, const octave_value& b);
 
+extern OCTINTERP_API octave_value
+do_binary_op (octave_value::compound_binary_op op,
+              const octave_value& a, const octave_value& b);
+
 #define OV_UNOP_FN(name) \
   inline octave_value \
   name (const octave_value& a) \
@@ -944,6 +1224,18 @@ OV_BINOP_FN (op_el_or)
 
 OV_BINOP_FN (op_struct_ref)
 
+#define OV_COMP_BINOP_FN(name) \
+  inline octave_value \
+  name (const octave_value& a1, const octave_value& a2) \
+  { \
+    return do_binary_op (octave_value::name, a1, a2); \
+  }
+
+OV_COMP_BINOP_FN (op_trans_mul)
+OV_COMP_BINOP_FN (op_mul_trans)
+OV_COMP_BINOP_FN (op_herm_mul)
+OV_COMP_BINOP_FN (op_mul_herm)
+
 extern OCTINTERP_API void install_types (void);
 
 // FIXME -- these trait classes probably belong somehwere else...
@@ -1001,6 +1293,7 @@ OCTAVE_ARRAY_TYPE_TRAIT (uint32NDArray, octave_uint32);
 OCTAVE_ARRAY_TYPE_TRAIT (int64NDArray, octave_int64);
 OCTAVE_ARRAY_TYPE_TRAIT (uint64NDArray, octave_uint64);
 OCTAVE_ARRAY_TYPE_TRAIT (NDArray, double);
+OCTAVE_ARRAY_TYPE_TRAIT (FloatNDArray, float);
 
 // This will eventually go away, but for now it can be used to
 // simplify the transition to the new octave_value class hierarchy,
