@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2003, 2004, 2005, 2006, 2007, 2008 John W. Eaton
+              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -1088,7 +1088,7 @@ set_format (const ComplexMatrix& cm)
 }
 
 static void
-set_range_format (bool sign, int x_max, int x_min, int all_ints, int& fw)
+set_range_format (int x_max, int x_min, int all_ints, int& fw)
 {
   static float_format fmt;
 
@@ -1104,7 +1104,7 @@ set_range_format (bool sign, int x_max, int x_min, int all_ints, int& fw)
   else if (bank_format)
     {
       int digits = x_max > x_min ? x_max : x_min;
-      fw = sign + digits < 0 ? 4 : digits + 3;
+      fw = digits < 0 ? 5 : digits + 4;
       rd = 2;
     }
   else if (hex_format)
@@ -1120,13 +1120,13 @@ set_range_format (bool sign, int x_max, int x_min, int all_ints, int& fw)
   else if (all_ints)
     {
       int digits = x_max > x_min ? x_max : x_min;
-      fw = sign + digits;
+      fw = digits + 1;
       rd = fw;
     }
   else if (Vfixed_point_format && ! print_g)
     {
       rd = prec;
-      fw = rd + 2 + sign;
+      fw = rd + 3;
     }
   else
     {
@@ -1161,7 +1161,7 @@ set_range_format (bool sign, int x_max, int x_min, int all_ints, int& fw)
       ld = ld_max > ld_min ? ld_max : ld_min;
       rd = rd_max > rd_min ? rd_max : rd_min;
 
-      fw = sign + 1 + ld + 1 + rd;
+      fw = ld + rd + 3;
     }
 
   if (! (rat_format || bank_format || hex_format || bit_format)
@@ -1177,7 +1177,7 @@ set_range_format (bool sign, int x_max, int x_min, int all_ints, int& fw)
 	  if (x_max > 100 || x_min > 100)
 	    exp_field++;
 
-	  fw = sign + 2 + prec + exp_field;
+	  fw = 3 + prec + exp_field;
 
 	  fmt = float_format (fw, prec - 1, std::ios::scientific);
 	}
@@ -1212,8 +1212,6 @@ set_format (const Range& r, int& fw, double& scale)
       r_min = tmp;
     }
 
-  bool sign = (r_min < 0.0);
-
   bool all_ints = r.all_elements_are_ints ();
 
   double max_abs = r_max < 0.0 ? -r_max : r_max;
@@ -1227,7 +1225,7 @@ set_format (const Range& r, int& fw, double& scale)
 
   scale = (x_max == 0 || all_ints) ? 1.0 : std::pow (10.0, x_max - 1);
 
-  set_range_format (sign, x_max, x_min, all_ints, fw);
+  set_range_format (x_max, x_min, all_ints, fw);
 }
 
 static inline void
@@ -1696,6 +1694,148 @@ octave_print_internal (std::ostream& os, const Matrix& m,
     }
 }
 
+void
+octave_print_internal (std::ostream& os, const DiagMatrix& m,
+		       bool pr_as_read_syntax, int extra_indent)
+{
+  octave_idx_type nr = m.rows ();
+  octave_idx_type nc = m.columns ();
+
+  if (nr == 0 || nc == 0)
+    print_empty_matrix (os, nr, nc, pr_as_read_syntax);
+  else if (plus_format && ! pr_as_read_syntax)
+    {
+      for (octave_idx_type i = 0; i < nr; i++)
+	{
+	  for (octave_idx_type j = 0; j < nc; j++)
+	    {
+	      OCTAVE_QUIT;
+
+	      pr_plus_format (os, m(i,j));
+	    }
+
+	  if (i < nr - 1)
+	    os << "\n";
+	}
+    }
+  else
+    {
+      int fw;
+      double scale = 1.0;
+      set_format (Matrix (m.diag ()), fw, scale);
+      int column_width = fw + 2;
+      octave_idx_type total_width = nc * column_width;
+      octave_idx_type max_width = command_editor::terminal_cols ();
+
+      if (pr_as_read_syntax)
+	max_width -= 4;
+      else
+	max_width -= extra_indent;
+
+      if (max_width < 0)
+	max_width = 0;
+
+      if (free_format)
+	{
+	  if (pr_as_read_syntax)
+	    os << "[\n";
+
+	  os << Matrix (m);
+
+	  if (pr_as_read_syntax)
+	    os << "]";
+
+	  return;
+	}
+
+      octave_idx_type inc = nc;
+      if (total_width > max_width && Vsplit_long_rows)
+	{
+	  inc = max_width / column_width;
+	  if (inc == 0)
+	    inc++;
+	}
+
+      if (pr_as_read_syntax)
+	{
+          os << "diag (";
+
+          octave_idx_type col = 0;
+          while (col < nc)
+            {
+              octave_idx_type lim = col + inc < nc ? col + inc : nc;
+
+              for (octave_idx_type j = col; j < lim; j++)
+                {
+                  OCTAVE_QUIT;
+
+                  if (j == 0)
+                    os << "[ ";
+                  else
+                    {
+                      if (j > col && j < lim)
+                        os << ", ";
+                      else
+                        os << "  ";
+                    }
+
+                  pr_float (os, m(j,j));
+                }
+
+              col += inc;
+
+              if (col >= nc)
+                  os << " ]";
+              else
+                os << " ...\n";
+            }
+          os << ")";
+	}
+      else
+	{
+          os << "Diagonal Matrix\n\n";
+	  pr_scale_header (os, scale);
+
+          // kluge. Get the true width of a number.
+          int zero_fw;
+
+            { 
+              std::ostringstream tmp_oss;
+              pr_float (tmp_oss, 0.0, fw, scale);
+              zero_fw = tmp_oss.str ().length ();
+            }
+
+	  for (octave_idx_type col = 0; col < nc; col += inc)
+	    {
+	      octave_idx_type lim = col + inc < nc ? col + inc : nc;
+
+	      pr_col_num_header (os, total_width, max_width, lim, col,
+				 extra_indent);
+
+	      for (octave_idx_type i = 0; i < nr; i++)
+		{
+		  os << std::setw (extra_indent) << "";
+
+		  for (octave_idx_type j = col; j < lim; j++)
+		    {
+		      OCTAVE_QUIT;
+
+		      os << "  ";
+
+                      if (i == j)
+                        pr_float (os, m(i,j), fw, scale);
+                      else
+                        os << std::setw (zero_fw) << '0';
+
+		    }
+
+		  if (i < nr - 1)
+		    os << "\n";
+		}
+	    }
+	}
+    }
+}
 #define PRINT_ND_ARRAY(os, nda, NDA_T, ELT_T, MAT_T) \
   do \
     { \
@@ -1749,7 +1889,7 @@ octave_print_internal (std::ostream& os, const Matrix& m,
               idx(1) = idx_vector (':'); \
  \
               for (int k = 2; k < ndims; k++) \
-                idx(k) = idx_vector (ra_idx(k) + 1); \
+                idx(k) = idx_vector (ra_idx(k)); \
  \
               octave_value page \
                 = MAT_T (Array2<ELT_T> (nda.index (idx), nr, nc)); \
@@ -1955,6 +2095,282 @@ octave_print_internal (std::ostream& os, const ComplexMatrix& cm,
     }
 }
 
+void
+octave_print_internal (std::ostream& os, const ComplexDiagMatrix& cm,
+		       bool pr_as_read_syntax, int extra_indent)
+{
+  octave_idx_type nr = cm.rows ();
+  octave_idx_type nc = cm.columns ();
+
+ if (nr == 0 || nc == 0)
+    print_empty_matrix (os, nr, nc, pr_as_read_syntax);
+  else if (plus_format && ! pr_as_read_syntax)
+    {
+      for (octave_idx_type i = 0; i < nr; i++)
+	{
+	  for (octave_idx_type j = 0; j < nc; j++)
+	    {
+	      OCTAVE_QUIT;
+
+	      pr_plus_format (os, cm(i,j));
+	    }
+
+	  if (i < nr - 1)
+	    os << "\n";
+	}
+    }
+  else
+    {
+      int r_fw, i_fw;
+      double scale = 1.0;
+      set_format (ComplexMatrix (cm.diag ()), r_fw, i_fw, scale);
+      int column_width = i_fw + r_fw;
+      column_width += (rat_format || bank_format || hex_format 
+		       || bit_format) ? 2 : 7;
+      octave_idx_type total_width = nc * column_width;
+      octave_idx_type max_width = command_editor::terminal_cols ();
+
+      if (pr_as_read_syntax)
+	max_width -= 4;
+      else
+	max_width -= extra_indent;
+
+      if (max_width < 0)
+	max_width = 0;
+
+      if (free_format)
+	{
+	  if (pr_as_read_syntax)
+	    os << "[\n";
+
+	  os << ComplexMatrix (cm);
+
+	  if (pr_as_read_syntax)
+	    os << "]";
+
+	  return;
+	}
+
+      octave_idx_type inc = nc;
+      if (total_width > max_width && Vsplit_long_rows)
+	{
+	  inc = max_width / column_width;
+	  if (inc == 0)
+	    inc++;
+	}
+
+      if (pr_as_read_syntax)
+	{
+          os << "diag (";
+
+          octave_idx_type col = 0;
+          while (col < nc)
+            {
+              octave_idx_type lim = col + inc < nc ? col + inc : nc;
+
+              for (octave_idx_type j = col; j < lim; j++)
+                {
+                  OCTAVE_QUIT;
+
+                  if (j == 0)
+                    os << "[ ";
+                  else
+                    {
+                      if (j > col && j < lim)
+                        os << ", ";
+                      else
+                        os << "  ";
+                    }
+
+                  pr_complex (os, cm(j,j));
+                }
+
+              col += inc;
+
+              if (col >= nc)
+                  os << " ]";
+              else
+                os << " ...\n";
+            }
+          os << ")";
+	}
+      else
+	{
+          os << "Diagonal Matrix\n\n";
+	  pr_scale_header (os, scale);
+
+          // kluge. Get the true width of a number.
+          int zero_fw;
+
+            { 
+              std::ostringstream tmp_oss;
+              pr_complex (tmp_oss, Complex (0.0), r_fw, i_fw, scale);
+              zero_fw = tmp_oss.str ().length ();
+            }
+
+	  for (octave_idx_type col = 0; col < nc; col += inc)
+	    {
+	      octave_idx_type lim = col + inc < nc ? col + inc : nc;
+
+	      pr_col_num_header (os, total_width, max_width, lim, col,
+				 extra_indent);
+
+	      for (octave_idx_type i = 0; i < nr; i++)
+		{
+		  os << std::setw (extra_indent) << "";
+
+		  for (octave_idx_type j = col; j < lim; j++)
+		    {
+		      OCTAVE_QUIT;
+
+		      os << "  ";
+
+                      if (i == j)
+                        pr_complex (os, cm(i,j), r_fw, i_fw, scale);
+                      else
+                        os << std::setw (zero_fw) << '0';
+		    }
+
+		  if (i < nr - 1) 
+		    os << "\n";
+		}
+	    }
+	}
+    }
+}
+
+void
+octave_print_internal (std::ostream& os, const PermMatrix& m,
+		       bool pr_as_read_syntax, int extra_indent)
+{
+  octave_idx_type nr = m.rows ();
+  octave_idx_type nc = m.columns ();
+
+  if (nr == 0 || nc == 0)
+    print_empty_matrix (os, nr, nc, pr_as_read_syntax);
+  else if (plus_format && ! pr_as_read_syntax)
+    {
+      for (octave_idx_type i = 0; i < nr; i++)
+	{
+	  for (octave_idx_type j = 0; j < nc; j++)
+	    {
+	      OCTAVE_QUIT;
+
+	      pr_plus_format (os, m(i,j));
+	    }
+
+	  if (i < nr - 1)
+	    os << "\n";
+	}
+    }
+  else
+    {
+      int fw = 2;
+      int column_width = fw + 2;
+      octave_idx_type total_width = nc * column_width;
+      octave_idx_type max_width = command_editor::terminal_cols ();
+
+      if (pr_as_read_syntax)
+	max_width -= 4;
+      else
+	max_width -= extra_indent;
+
+      if (max_width < 0)
+	max_width = 0;
+
+      if (free_format)
+	{
+	  if (pr_as_read_syntax)
+	    os << "[\n";
+
+	  os << Matrix (m);
+
+	  if (pr_as_read_syntax)
+	    os << "]";
+
+	  return;
+	}
+
+      octave_idx_type inc = nc;
+      if (total_width > max_width && Vsplit_long_rows)
+	{
+	  inc = max_width / column_width;
+	  if (inc == 0)
+	    inc++;
+	}
+
+      if (pr_as_read_syntax)
+        {
+          Array<octave_idx_type> pvec = m.pvec ();
+          bool colp = m.is_col_perm ();
+
+          os << "eye (";
+          if (colp) os << ":, ";
+
+          octave_idx_type col = 0;
+          while (col < nc)
+            {
+              octave_idx_type lim = col + inc < nc ? col + inc : nc;
+
+              for (octave_idx_type j = col; j < lim; j++)
+                {
+                  OCTAVE_QUIT;
+
+                  if (j == 0)
+                    os << "[ ";
+                  else
+                    {
+                      if (j > col && j < lim)
+                        os << ", ";
+                      else
+                        os << "  ";
+                    }
+
+                  os << pvec (j);
+                }
+
+              col += inc;
+
+              if (col >= nc)
+                  os << " ]";
+              else
+                os << " ...\n";
+            }
+          if (! colp) os << ", :";
+          os << ")";
+	}
+      else
+	{
+          os << "Permutation Matrix\n\n";
+
+	  for (octave_idx_type col = 0; col < nc; col += inc)
+	    {
+	      octave_idx_type lim = col + inc < nc ? col + inc : nc;
+
+	      pr_col_num_header (os, total_width, max_width, lim, col,
+				 extra_indent);
+
+	      for (octave_idx_type i = 0; i < nr; i++)
+		{
+		  os << std::setw (extra_indent) << "";
+
+		  for (octave_idx_type j = col; j < lim; j++)
+		    {
+		      OCTAVE_QUIT;
+
+		      os << "  ";
+
+                      os << std::setw (fw) << m(i,j);
+		    }
+
+		  if (i < nr - 1)
+		    os << "\n";
+		}
+	    }
+	}
+    }
+}
+
 void
 octave_print_internal (std::ostream& os, const ComplexNDArray& nda,
 		       bool pr_as_read_syntax, int extra_indent)
@@ -1973,6 +2389,69 @@ octave_print_internal (std::ostream& os, const ComplexNDArray& nda,
     }
 }
 
+void
+octave_print_internal (std::ostream& os, bool d, bool pr_as_read_syntax)
+{ 
+  octave_print_internal (os, double (d), pr_as_read_syntax); 
+}
+
+// FIXME -- write single precision versions of the printing functions.
+
+void
+octave_print_internal (std::ostream& os, float d, bool pr_as_read_syntax)
+{ 
+  octave_print_internal (os, double (d), pr_as_read_syntax); 
+}
+
+void
+octave_print_internal (std::ostream& os, const FloatMatrix& m,
+		       bool pr_as_read_syntax, int extra_indent)
+{ 
+  octave_print_internal (os, Matrix (m), pr_as_read_syntax, extra_indent); 
+}
+
+void
+octave_print_internal (std::ostream& os, const FloatDiagMatrix& m,
+		       bool pr_as_read_syntax, int extra_indent)
+{ 
+  octave_print_internal (os, DiagMatrix (m), pr_as_read_syntax, extra_indent); 
+}
+
+void
+octave_print_internal (std::ostream& os, const FloatNDArray& nda,
+		       bool pr_as_read_syntax, int extra_indent)
+{
+  octave_print_internal (os, NDArray (nda), pr_as_read_syntax, extra_indent); 
+}
+
+void
+octave_print_internal (std::ostream& os, const FloatComplex& c,
+		       bool pr_as_read_syntax)
+{
+  octave_print_internal (os, Complex (c), pr_as_read_syntax); 
+}
+
+void
+octave_print_internal (std::ostream& os, const FloatComplexMatrix& cm,
+		       bool pr_as_read_syntax, int extra_indent)
+{
+  octave_print_internal (os, ComplexMatrix (cm), pr_as_read_syntax, extra_indent);
+}
+
+void
+octave_print_internal (std::ostream& os, const FloatComplexDiagMatrix& cm,
+		       bool pr_as_read_syntax, int extra_indent)
+{
+  octave_print_internal (os, ComplexDiagMatrix (cm), pr_as_read_syntax, extra_indent);
+}
+
+void
+octave_print_internal (std::ostream& os, const FloatComplexNDArray& nda,
+		       bool pr_as_read_syntax, int extra_indent)
+{
+  octave_print_internal (os, ComplexNDArray (nda), pr_as_read_syntax, extra_indent);
+}
+
 void
 octave_print_internal (std::ostream& os, const Range& r,
 		       bool pr_as_read_syntax, int extra_indent)
@@ -2244,7 +2723,7 @@ octave_print_internal (std::ostream& os, const ArrayN<std::string>& nda,
 	  idx(1) = idx_vector (':');
 
 	  for (int k = 2; k < ndims; k++)
-	    idx(k) = idx_vector (ra_idx(k) + 1);
+	    idx(k) = idx_vector (ra_idx(k));
 
 	  Array2<std::string> page (nda.index (idx), nr, nc);
 
@@ -2511,7 +2990,7 @@ octave_print_internal_template (std::ostream& os, const intNDArray<T>& nda,
 	  idx(1) = idx_vector (':');
 
 	  for (int k = 2; k < ndims; k++)
-	    idx(k) = idx_vector (ra_idx(k) + 1);
+	    idx(k) = idx_vector (ra_idx(k));
 
 	  Array2<T> page (nda.index (idx), nr, nc);
 
@@ -2616,7 +3095,7 @@ octave_print_internal_template (std::ostream& os, const intNDArray<T>& nda,
 	  idx(1) = idx_vector (':');
 
 	  for (int k = 2; k < ndims; k++)
-	    idx(k) = idx_vector (ra_idx(k) + 1);
+	    idx(k) = idx_vector (ra_idx(k));
 
 	  Array2<T> page (nda.index (idx), nr, nc);
 
@@ -2693,7 +3172,7 @@ PRINT_INT_ARRAY_INTERNAL (octave_uint32)
 PRINT_INT_ARRAY_INTERNAL (octave_int64)
 PRINT_INT_ARRAY_INTERNAL (octave_uint64)
 
-extern void
+void
 octave_print_internal (std::ostream&, const Cell&, bool, int, bool)
 {
   panic_impossible ();
@@ -2706,12 +3185,14 @@ Convert @var{x} into a rational approximation represented as a string.\n\
 You can convert the string back into a matrix as follows:\n\
 \n\
 @example\n\
+@group\n\
    r = rats(hilb(4));\n\
    x = str2num(r)\n\
+@end group\n\
 @end example\n\
 \n\
 The optional second argument defines the maximum length of the string\n\
-representing the elements of @var{x}. By default @var{len} is 9.\n\
+representing the elements of @var{x}.  By default @var{len} is 9.\n\
 @seealso{format, rat}\n\
 @end deftypefn")
 {
@@ -2719,18 +3200,20 @@ representing the elements of @var{x}. By default @var{len} is 9.\n\
 
   int nargin = args.length ();
 
-  unwind_protect::begin_frame ("Frats");
-
-  unwind_protect_int (rat_string_len);
-
-  rat_string_len = 9;
-
-  if (nargin == 2)
-    rat_string_len = args(1).nint_value ();
-
-  if (! error_state)
+  if (nargin < 1 || nargin > 2 || nargout > 1)
+    print_usage ();
+  else
     {
-      if (nargin < 3 && nargout < 2)
+      unwind_protect::begin_frame ("Frats");
+
+      unwind_protect_int (rat_string_len);
+
+      rat_string_len = 9;
+
+      if (nargin == 2)
+	rat_string_len = args(1).nint_value ();
+
+      if (! error_state)
 	{
 	  octave_value arg = args(0);
 
@@ -2753,7 +3236,7 @@ representing the elements of @var{x}. By default @var{len} is 9.\n\
 		{
 		  size_t m = s.find ('\n',  n);
 
-		  if (m == NPOS)
+		  if (m == std::string::npos)
 		    {
 		      lst.push_back (s.substr (n));
 		      break;
@@ -2770,11 +3253,9 @@ representing the elements of @var{x}. By default @var{len} is 9.\n\
 	  else
 	    error ("rats: expecting numeric input");
 	}
-      else
-	print_usage ();
-    }
 
-  unwind_protect::run_frame ("Frats");
+      unwind_protect::run_frame ("Frats");
+    }
 
   return retval;
 }
@@ -2785,10 +3266,12 @@ DEFUN (disp, args, nargout,
 Display the value of @var{x}.  For example,\n\
 \n\
 @example\n\
+@group\n\
 disp (\"The value of pi is:\"), disp (pi)\n\
 \n\
      @print{} the value of pi is:\n\
      @print{} 3.1416\n\
+@end group\n\
 @end example\n\
 \n\
 @noindent\n\
@@ -2827,10 +3310,12 @@ DEFUN (fdisp, args, ,
 Display the value of @var{x} on the stream @var{fid}.  For example,\n\
 \n\
 @example\n\
+@group\n\
 fdisp (stdout, \"The value of pi is:\"), fdisp (stdout, pi)\n\
 \n\
      @print{} the value of pi is:\n\
      @print{} 3.1416\n\
+@end group\n\
 @end example\n\
 \n\
 @noindent\n\
@@ -3053,7 +3538,7 @@ set_format_style (int argc, const string_vector& argv)
     }
 }
 
-DEFCMD (format, args, ,
+DEFUN (format, args, ,
   "-*- texinfo -*-\n\
 @deffn {Command} format options\n\
 Control the format of the output produced by @code{disp} and Octave's\n\
@@ -3196,7 +3681,7 @@ Remove extra blank space around column number labels.\n\
 Insert blank lines above and below column number labels (this is the\n\
 default).\n\
 @item rat\n\
-Print a rational approximation. That is the values are approximated\n\
+Print a rational approximation.  That is the values are approximated\n\
 by one small integer divided by another.\n\
 @end table\n\
 \n\
