@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2004, 2005,
-              2006, 2007 John W. Eaton
+              2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -149,6 +149,8 @@ maybe_warn_former_built_in_variable (const std::string& nm)
 
       while (*p)
 	vars.insert (*p++);
+
+      initialized = true;
     }
 
   if (vars.find (nm) != vars.end ())
@@ -176,21 +178,16 @@ tree_simple_assignment::rvalue (int nargout)
 {
   octave_value_list retval;
 
-  MAYBE_DO_BREAKPOINT;
-
   if (nargout > 1)
     error ("invalid number of output arguments for expression X = RHS");
   else
-    retval = rvalue ();
+    retval = rvalue1 (nargout);
 
   return retval;
 }
 
-// FIXME -- this works, but it would look a little better if
-// it were broken up into a couple of separate functions.
-
 octave_value
-tree_simple_assignment::rvalue (void)
+tree_simple_assignment::rvalue1 (int)
 {
   octave_value retval;
 
@@ -202,29 +199,33 @@ tree_simple_assignment::rvalue (void)
 
   if (rhs)
     {
-      octave_value_list tmp = rhs->rvalue ();
+      octave_value rhs_val = rhs->rvalue1 ();
 
-      if (! (error_state || tmp.empty ()))
+      if (! error_state)
 	{
-	  octave_value rhs_val = tmp(0);
-
 	  if (rhs_val.is_undefined ())
 	    {
 	      error ("value on right hand side of assignment is undefined");
-	      eval_error ();
-	    }
-	  else if (rhs_val.is_cs_list ())
-	    {
-	      error ("invalid assignment of comma-separated list");
-	      eval_error ();
+	      return retval;
 	    }
 	  else
 	    {
+	      if (rhs_val.is_cs_list ())
+		{
+		  const octave_value_list lst = rhs_val.list_value ();
+
+		  if (! lst.empty ())
+		    rhs_val = lst(0);
+		  else
+		    {
+		      error ("invalid number of elements on RHS of assignment");
+		      return retval;
+		    }
+		}
+
 	      octave_lvalue ult = lhs->lvalue ();
 
-	      if (error_state)
-		eval_error ();
-	      else
+	      if (! error_state)
 		{
 		  ult.assign (etype, rhs_val);
 
@@ -252,13 +253,9 @@ tree_simple_assignment::rvalue (void)
 						     lhs->name ());
 			}
 		    }
-		  else
-		    eval_error ();
 		}
 	    }
 	}
-      else
-	eval_error ();
     }
 
   first_execution = false;
@@ -266,17 +263,6 @@ tree_simple_assignment::rvalue (void)
   return retval;
 }
 
-void
-tree_simple_assignment::eval_error (void)
-{
-  int l = line ();
-  int c = column ();
-
-  if (l != -1 && c != -1)
-    ::error ("evaluating assignment expression near line %d, column %d",
-	     l, c);
-}
-
 std::string
 tree_simple_assignment::oper (void) const
 {
@@ -284,11 +270,12 @@ tree_simple_assignment::oper (void) const
 }
 
 tree_expression *
-tree_simple_assignment::dup (symbol_table *sym_tab)
+tree_simple_assignment::dup (symbol_table::scope_id scope,
+			     symbol_table::context_id context) const
 {
   tree_simple_assignment *new_sa
-    = new tree_simple_assignment (lhs ? lhs->dup (sym_tab) : 0,
-				  rhs ? rhs->dup (sym_tab) : 0,
+    = new tree_simple_assignment (lhs ? lhs->dup (scope, context) : 0,
+				  rhs ? rhs->dup (scope, context) : 0,
 				  preserve, etype);
 
   new_sa->copy_base (*this);
@@ -319,11 +306,11 @@ tree_multi_assignment::~tree_multi_assignment (void)
 }
 
 octave_value
-tree_multi_assignment::rvalue (void)
+tree_multi_assignment::rvalue1 (int nargout)
 {
   octave_value retval;
 
-  octave_value_list tmp = rvalue (1);
+  const octave_value_list tmp = rvalue (nargout);
 
   if (! tmp.empty ())
     retval = tmp(0);
@@ -367,7 +354,10 @@ tree_multi_assignment::rvalue (int)
 	   p++)
 	n_out += p->numel ();
 
-      octave_value_list rhs_val = rhs->rvalue (n_out);
+      // The following trick is used to keep rhs_val constant.
+      const octave_value_list rhs_val1 = rhs->rvalue (n_out);
+      const octave_value_list rhs_val = (rhs_val1.length () == 1 && rhs_val1(0).is_cs_list ()
+                                         ? rhs_val1(0).list_value () : rhs_val1);
 
       if (error_state)
 	return retval;
@@ -377,7 +367,6 @@ tree_multi_assignment::rvalue (int)
 	  if (n_out > 0)
 	    {
 	      error ("value on right hand side of assignment is undefined");
-	      eval_error ();
 	      return retval;
 	    }
 	}
@@ -387,19 +376,9 @@ tree_multi_assignment::rvalue (int)
 
 	  octave_idx_type n = rhs_val.length ();
 
-	  if (n == 1)
-	    {
-	      octave_value tmp = rhs_val(0);
-
-	      if (tmp.is_cs_list ())
-		{
-		  error ("invalid assignment of comma-separated list");
-		  eval_error ();
-		  return retval;
-		}
-	    }
-
-	  retval.resize (n, octave_value ());
+          // To avoid copying per elements and possible optimizations, we
+          // postpone joining the final values.
+          std::list<octave_value_list> retval_list;
 
 	  tree_argument_list::iterator q = lhs->begin ();
 
@@ -419,17 +398,14 @@ tree_multi_assignment::rvalue (int)
 		    {
 		      if (etype == octave_value::op_asn_eq)
 			{
-			  octave_value_list ovl (nel, octave_value ());
-
-			  for (octave_idx_type j = 0; j < nel; j++)
-			    ovl(j) = rhs_val(k+j);
+                          // This won't do a copy.
+			  octave_value_list ovl  = rhs_val.slice (k, nel);
 
 			  ult.assign (etype, octave_value (ovl, true));
 
 			  if (! error_state)
 			    {
-			      for (octave_idx_type j = 0; j < nel; j++)
-				retval(k+j) = rhs_val(k+j);
+                              retval_list.push_back (ovl);
 
 			      k += nel;
 			    }
@@ -444,7 +420,7 @@ tree_multi_assignment::rvalue (int)
 		  else
 		    error ("some elements undefined in return list");
 		}
-	      else
+	      else if (nel == 1)
 		{
 		  if (k < n)
 		    {
@@ -453,9 +429,9 @@ tree_multi_assignment::rvalue (int)
 		      if (! error_state)
 			{
 			  if (etype == octave_value::op_asn_eq)
-			    retval(k) = rhs_val(k);
+                            retval_list.push_back (rhs_val(k));
 			  else
-			    retval(k) = ult.value ();
+                            retval_list.push_back (ult.value ());
 
 			  k++;
 			}
@@ -465,10 +441,7 @@ tree_multi_assignment::rvalue (int)
 		}
 
 	      if (error_state)
-		{
-		  eval_error ();
-		  break;
-		}
+		break;
 	      else if (print_result ())
 		{
 		  // We clear any index here so that we can get
@@ -489,27 +462,17 @@ tree_multi_assignment::rvalue (int)
 		break;
 
 	    }
+          
+          // Concatenate return values.
+          retval = retval_list;
 	}
     }
-  else
-    eval_error ();
 
   first_execution = false;
 
   return retval;
 }
 
-void
-tree_multi_assignment::eval_error (void)
-{
-  int l = line ();
-  int c = column ();
-
-  if (l != -1 && c != -1)
-    ::error ("evaluating assignment expression near line %d, column %d",
-	     l, c);
-}
-
 std::string
 tree_multi_assignment::oper (void) const
 {
@@ -517,11 +480,12 @@ tree_multi_assignment::oper (void) const
 }
 
 tree_expression *
-tree_multi_assignment::dup (symbol_table *sym_tab)
+tree_multi_assignment::dup (symbol_table::scope_id scope,
+			    symbol_table::context_id context) const
 {
   tree_multi_assignment *new_ma
-    = new tree_multi_assignment (lhs ? lhs->dup (sym_tab) : 0,
-				 rhs ? rhs->dup (sym_tab) : 0,
+    = new tree_multi_assignment (lhs ? lhs->dup (scope, context) : 0,
+				 rhs ? rhs->dup (scope, context) : 0,
 				 preserve, etype);
 
   new_ma->copy_base (*this);
