@@ -1,7 +1,7 @@
 /*
 
-Copyright (C) 1996, 1997, 2000, 2002, 2003, 2004, 2005, 2006, 2007
-              John W. Eaton
+Copyright (C) 1996, 1997, 2000, 2002, 2003, 2004, 2005, 2006, 2007,
+              2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -34,6 +34,7 @@ class octave_lvalue;
 
 #include "ov.h"
 #include "pt-exp.h"
+#include "symtab.h"
 
 // Binary expressions.
 
@@ -69,12 +70,10 @@ public:
 
   bool rvalue_ok (void) const { return true; }
 
-  octave_value rvalue (void);
+  octave_value rvalue1 (int nargout = 1);
 
   octave_value_list rvalue (int nargout);
 
-  void eval_error (void);
-
   std::string oper (void) const;
 
   octave_value::binary_op op_type (void) const { return etype; }
@@ -82,7 +81,8 @@ public:
   tree_expression *lhs (void) { return op_lhs; }
   tree_expression *rhs (void) { return op_rhs; }
 
-  tree_expression *dup (symbol_table *sym_tab);
+  tree_expression *dup (symbol_table::scope_id scope,
+			symbol_table::context_id context) const;
 
   void accept (tree_walker& tw);
 
@@ -131,7 +131,7 @@ public:
 
   bool rvalue_ok (void) const { return true; }
 
-  octave_value rvalue (void);
+  octave_value rvalue1 (int nargout = 1);
 
   octave_value_list rvalue (int nargout);
 
@@ -139,7 +139,8 @@ public:
 
   type op_type (void) const { return etype; }
 
-  tree_expression *dup (symbol_table *sym_tab);
+  tree_expression *dup (symbol_table::scope_id scope,
+			symbol_table::context_id context) const;
 
 private:
 
