@@ -1,7 +1,7 @@
 /*
 
-Copyright (C) 1996, 1997, 2000, 2002, 2003, 2004, 2005, 2006, 2007
-              John W. Eaton
+Copyright (C) 1996, 1997, 2000, 2002, 2003, 2004, 2005, 2006, 2007,
+              2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -26,6 +26,8 @@ along with Octave; see the file COPYING.  If not, see
 
 #include "pt-walk.h"
 
+class tree_decl_command;
+
 // How to check the semantics of the code that the parse trees represent.
 
 class
@@ -48,7 +50,9 @@ public:
 
   void visit_continue_command(tree_continue_command&);
 
-  void visit_decl_command (tree_decl_command&);
+  void visit_global_command (tree_global_command&);
+
+  void visit_static_command (tree_static_command&);
 
   void visit_decl_elt (tree_decl_elt&);
 
@@ -58,8 +62,12 @@ public:
 
   void visit_complex_for_command (tree_complex_for_command&);
 
+  void visit_octave_user_script (octave_user_script&);
+
   void visit_octave_user_function (octave_user_function&);
 
+  void visit_function_def (tree_function_def&);
+
   void visit_identifier (tree_identifier&);
 
   void visit_if_clause (tree_if_clause&);
@@ -118,6 +126,8 @@ private:
 
   bool do_lvalue_check;
 
+  void do_decl_command (tree_decl_command&);
+
   void gripe (const std::string& msg, int line);
 
   // No copying!
