@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1996, 1997, 2001, 2002, 2004, 2005, 2006, 2007
+Copyright (C) 1996, 1997, 2001, 2002, 2004, 2005, 2006, 2007, 2008, 2009
               John W. Eaton
 
 This file is part of Octave.
@@ -38,18 +38,9 @@ class octave_value_list;
 // Nonzero means we're breaking out of a loop or function body.
 int tree_break_command::breaking = 0;
 
-void
-tree_break_command::eval (void)
-{
-  // Even if we have an error we should still enter debug mode.
-  MAYBE_DO_BREAKPOINT;
-
-  if (! error_state)
-    breaking = 1;
-}
-
 tree_command *
-tree_break_command::dup (symbol_table *)
+tree_break_command::dup (symbol_table::scope_id,
+			 symbol_table::context_id) const
 {
   return new tree_break_command (line (), column ());
 }
@@ -65,17 +56,9 @@ tree_break_command::accept (tree_walker& tw)
 // Nonzero means we're jumping to the end of a loop.
 int tree_continue_command::continuing = 0;
 
-void
-tree_continue_command::eval (void)
-{
-  MAYBE_DO_BREAKPOINT;
-
-  if (! error_state)
-    continuing = 1;
-}
-
 tree_command *
-tree_continue_command::dup (symbol_table *)
+tree_continue_command::dup (symbol_table::scope_id,
+			    symbol_table::context_id) const
 {
   return new tree_continue_command (line (), column ());
 }
@@ -91,17 +74,9 @@ tree_continue_command::accept (tree_walker& tw)
 // Nonzero means we're returning from a function.
 int tree_return_command::returning = 0;
 
-void
-tree_return_command::eval (void)
-{
-  MAYBE_DO_BREAKPOINT;
-
-  if (! error_state)
-    returning = 1;
-}
-
 tree_command *
-tree_return_command::dup (symbol_table *)
+tree_return_command::dup (symbol_table::scope_id,
+			  symbol_table::context_id) const
 {
   return new tree_return_command (line (), column ());
 }
