@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-              2005, 2006, 2007 John W. Eaton
+              2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -67,14 +67,14 @@ private:
       : count (1), dv (0, 0), all_str (false),
 	all_sq_str (false), all_dq_str (false),
 	some_str (false), all_real (false), all_cmplx (false),
-	all_mt (true), any_sparse (false),
+	all_mt (true), any_sparse (false), any_class (false),
 	class_nm (octave_base_value::static_class_name ()), ok (false)
     { }
 
     tm_row_const_rep (const tree_argument_list& row)
       : count (1), dv (0, 0), all_str (false), all_sq_str (false),
 	some_str (false), all_real (false), all_cmplx (false),
-	all_mt (true), any_sparse (false),
+	all_mt (true), any_sparse (false), any_class (false),
 	class_nm (octave_base_value::static_class_name ()), ok (false)
     { init (row); }
 
@@ -92,6 +92,7 @@ private:
     bool all_cmplx;
     bool all_mt;
     bool any_sparse;
+    bool any_class;
 
     std::string class_nm;
 
@@ -158,6 +159,8 @@ public:
 
   bool empty (void) const { return rep->empty (); }
 
+  size_t length (void) const { return rep->length (); }
+
   dim_vector dims (void) { return rep->dv; }
 
   bool all_strings_p (void) const { return rep->all_str; }
@@ -168,6 +171,7 @@ public:
   bool all_complex_p (void) const { return rep->all_cmplx; }
   bool all_empty_p (void) const { return rep->all_mt; }
   bool any_sparse_p (void) const { return rep->any_sparse; }
+  bool any_class_p (void) const { return rep->any_class; }
 
   std::string class_name (void) const { return rep->class_nm; }
 
@@ -191,6 +195,8 @@ get_concat_class (const std::string& c1, const std::string& c2)
 
   if (c1 == c2)
     retval = c1;
+  else if (c1 == retval)
+    retval = c2;
   else
     {
       bool c1_is_int = (c1 == "int8" || c1 == "uint8"
@@ -259,6 +265,9 @@ tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
 
   dim_vector this_elt_dv = val.dims ();
 
+  class_nm = get_concat_class (class_nm, this_elt_class_nm);
+
+
   if (! this_elt_dv.all_zero ())
     {
       all_mt = false;
@@ -267,8 +276,6 @@ tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
 	{
 	  first_elem = false;
 
-	  class_nm = this_elt_class_nm;
-
 	  dv.resize (this_elt_dv.length ());
 	  for (int i = 2; i < dv.length (); i++)
 	    dv.elem (i) = this_elt_dv.elem (i);
@@ -279,8 +286,6 @@ tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
 	}
       else
 	{
-	  class_nm = get_concat_class (class_nm, this_elt_class_nm);
-
 	  int len = (this_elt_dv.length () < dv.length ()
 		     ? this_elt_dv.length () : dv.length ());
 
@@ -345,6 +350,9 @@ tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
   if (!any_sparse && val.is_sparse_type ())
     any_sparse = true;
 
+  if (!any_class && val.is_object ())
+    any_class = true;
+
   return true;
 }
 
@@ -357,6 +365,7 @@ tm_row_const::tm_row_const_rep::init (const tree_argument_list& row)
   all_real = true;
   all_cmplx = true;
   any_sparse = false;
+  any_class = false;
 
   bool first_elem = true;
 
@@ -368,7 +377,7 @@ tm_row_const::tm_row_const_rep::init (const tree_argument_list& row)
 
       tree_expression *elt = *p;
 
-      octave_value tmp = elt->rvalue ();
+      octave_value tmp = elt->rvalue1 ();
 
       if (error_state || tmp.is_undefined ())
 	break;
@@ -438,7 +447,7 @@ public:
   tm_const (const tree_matrix& tm)
     : dv (0, 0), all_str (false), all_sq_str (false), all_dq_str (false),
       some_str (false), all_real (false), all_cmplx (false),
-      all_mt (true), any_sparse (false),
+      all_mt (true), any_sparse (false), any_class (false),
       class_nm (octave_base_value::static_class_name ()), ok (false)
   { init (tm); }
 
@@ -457,6 +466,7 @@ public:
   bool all_complex_p (void) const { return all_cmplx; }
   bool all_empty_p (void) const { return all_mt; }
   bool any_sparse_p (void) const { return any_sparse; }
+  bool any_class_p (void) const { return any_class; }
 
   std::string class_name (void) const { return class_nm; }
 
@@ -474,6 +484,7 @@ private:
   bool all_cmplx;
   bool all_mt;
   bool any_sparse;
+  bool any_class;
 
   std::string class_nm;
 
@@ -497,6 +508,7 @@ tm_const::init (const tree_matrix& tm)
   all_real = true;
   all_cmplx = true;
   any_sparse = false;
+  any_class = false;
 
   bool first_elem = true;
 
@@ -539,6 +551,9 @@ tm_const::init (const tree_matrix& tm)
 	  if (!any_sparse && tmp.any_sparse_p ())
 	    any_sparse = true;
 
+	  if (!any_class && tmp.any_class_p ())
+	    any_class = true;
+
 	  append (tmp);
 	}
       else
@@ -689,12 +704,10 @@ tree_matrix::rvalue (int nargout)
 {
   octave_value_list retval;
 
-  MAYBE_DO_BREAKPOINT;
-
   if (nargout > 1)
     error ("invalid number of output arguments for matrix list");
   else
-    retval = rvalue ();
+    retval = rvalue1 (nargout);
 
   return retval;
 }
@@ -758,7 +771,7 @@ maybe_warn_string_concat (bool all_dq_strings_p, bool all_sq_strings_p)
   while (0)
 
 octave_value
-tree_matrix::rvalue (void)
+tree_matrix::rvalue1 (int)
 {
   octave_value retval = Matrix ();
 
@@ -769,6 +782,7 @@ tree_matrix::rvalue (void)
   bool all_real_p = false;
   bool all_complex_p = false;
   bool any_sparse_p = false;
+  bool any_class_p = false;
   bool frc_str_conv = false;
 
   tm_const tmp (*this);
@@ -783,13 +797,75 @@ tree_matrix::rvalue (void)
       all_real_p = tmp.all_real_p ();
       all_complex_p = tmp.all_complex_p ();
       any_sparse_p = tmp.any_sparse_p ();
+      any_class_p = tmp.any_class_p ();
       frc_str_conv = tmp.some_strings_p ();
 
       // Try to speed up the common cases.
 
       std::string result_type = tmp.class_name ();
 
-      if (result_type == "double")
+      if (any_class_p)
+	{
+	  octave_value_list tmp3 (tmp.length (), octave_value ());
+
+	  int j = 0;
+	  for (tm_const::iterator p = tmp.begin (); p != tmp.end (); p++)
+	    {
+	      OCTAVE_QUIT;
+
+	      tm_row_const row = *p;
+
+	      if (row.length () == 1)
+		tmp3 (j++) = *(row.begin ());
+	      else
+		{
+		  octave_value_list tmp1 (row.length (), octave_value ());
+
+		  int i = 0;
+		  for (tm_row_const::iterator q = row.begin (); 
+		       q != row.end (); q++)
+		    tmp1 (i++) = *q;
+
+		  octave_value_list tmp2;
+		  octave_value fcn = 
+		    symbol_table::find_function ("horzcat", tmp1);
+
+		  if (fcn.is_defined ())
+		    {
+		      tmp2 = fcn.do_multi_index_op (1, tmp1);
+		      
+		      if (error_state)
+			goto done;
+
+		      tmp3 (j++) = tmp2 (0);
+		    }
+		  else
+		    {
+		      ::error ("cat not find overloaded horzcat function");
+		      goto done;
+		    }
+		}
+	    }
+
+	  if (tmp.length () == 1)
+	    retval = tmp3 (0);
+	  else
+	    {
+	      octave_value_list tmp2;
+	      octave_value fcn = symbol_table::find_function ("vertcat", tmp3);
+
+	      if (fcn.is_defined ())
+		{
+		  tmp2 = fcn.do_multi_index_op (1, tmp3);
+		      
+		  if (! error_state)
+		    retval = tmp2 (0);
+		}
+	      else
+		::error ("cat not find overloaded vertcat function");
+	    }
+	}
+      else if (result_type == "double")
 	{
 	  if (any_sparse_p)
 	    {	    
@@ -806,9 +882,14 @@ tree_matrix::rvalue (void)
 		DO_SINGLE_TYPE_CONCAT (ComplexNDArray, complex_array_value);
 	    }
 	}
-#if 0
       else if (result_type == "single")
-#endif
+	{
+	  if (all_real_p)
+	    DO_SINGLE_TYPE_CONCAT (FloatNDArray, float_array_value);
+	  else
+	    DO_SINGLE_TYPE_CONCAT (FloatComplexNDArray, 
+				   float_complex_array_value);
+	}
       else if (result_type == "char")
 	{
 	  char type = all_dq_strings_p ? '"' : '\'';
@@ -951,15 +1032,16 @@ done:
 }
 
 tree_expression *
-tree_matrix::dup (symbol_table *sym_tab)
+tree_matrix::dup (symbol_table::scope_id scope,
+		  symbol_table::context_id context) const
 {
   tree_matrix *new_matrix = new tree_matrix (0, line (), column ());
 
-  for (iterator p = begin (); p != end (); p++)
+  for (const_iterator p = begin (); p != end (); p++)
     {
-      tree_argument_list *elt = *p;
+      const tree_argument_list *elt = *p;
 
-      new_matrix->append (elt ? elt->dup (sym_tab) : 0);
+      new_matrix->append (elt ? elt->dup (scope, context) : 0);
     }
 
   new_matrix->copy_base (*this);
