@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1996, 1997, 2000, 2002, 2004, 2005, 2006, 2007
+Copyright (C) 1996, 1997, 2000, 2002, 2004, 2005, 2006, 2007, 2008, 2009
               John W. Eaton
 
 This file is part of Octave.
@@ -45,25 +45,12 @@ tree_if_clause::~tree_if_clause (void)
   delete lead_comm;
 }
 
-int
-tree_if_clause::eval (void)
-{
-  if (is_else_clause () || expr->is_logically_true ("if"))
-    {
-      if (list)
-	list->eval ();
-
-      return 1;
-    }
-
-  return 0;
-}
-
 tree_if_clause *
-tree_if_clause::dup (symbol_table *sym_tab)
+tree_if_clause::dup (symbol_table::scope_id scope,
+		     symbol_table::context_id context) const
 {
-  return new tree_if_clause (expr ? expr->dup (sym_tab) : 0,
-			     list ? list->dup (sym_tab) : 0,
+  return new tree_if_clause (expr ? expr->dup (scope, context) : 0,
+			     list ? list->dup (scope, context) : 0,
 			     lead_comm ? lead_comm->dup () : 0);
 }
 
@@ -75,28 +62,17 @@ tree_if_clause::accept (tree_walker& tw)
 
 // List of if commands.
 
-void
-tree_if_command_list::eval (void)
-{
-  for (iterator p = begin (); p != end (); p++)
-    {
-      tree_if_clause *t = *p;
-
-      if (t->eval () || error_state)
-	break;
-    }
-}
-
 tree_if_command_list *
-tree_if_command_list::dup (symbol_table *sym_tab)
+tree_if_command_list::dup (symbol_table::scope_id scope,
+			   symbol_table::context_id context) const
 {
   tree_if_command_list *new_icl = new tree_if_command_list ();
 
-  for (iterator p = begin (); p != end (); p++)
+  for (const_iterator p = begin (); p != end (); p++)
     {
-      tree_if_clause *elt = *p;
+      const tree_if_clause *elt = *p;
 
-      new_icl->append (elt ? elt->dup (sym_tab) : 0);
+      new_icl->append (elt ? elt->dup (scope, context) : 0);
     }
 
   return new_icl;
@@ -118,20 +94,34 @@ tree_if_command::~tree_if_command (void)
 }
 
 void
-tree_if_command::eval (void)
+tree_if_command::set_breakpoint (void)
 {
   if (list)
-    list->eval ();
+    {
+      tree_if_clause *elt = list->front ();
 
-  if (error_state)
-    ::error ("evaluating if command near line %d, column %d",
-	     line (), column ());
+      if (elt)
+	elt->set_breakpoint ();
+    }
+}
+
+void
+tree_if_command::delete_breakpoint (void)
+{
+  if (list)
+    {
+      tree_if_clause *elt = list->front ();
+
+      if (elt)
+	elt->set_breakpoint ();
+    }
 }
 
 tree_command *
-tree_if_command::dup (symbol_table *sym_tab)
+tree_if_command::dup (symbol_table::scope_id scope,
+		      symbol_table::context_id context) const
 {
-  return new tree_if_command (list ? list->dup (sym_tab) : 0,
+  return new tree_if_command (list ? list->dup (scope, context) : 0,
 			      lead_comm ? lead_comm->dup () : 0,
 			      trail_comm ? trail_comm->dup () : 0,
 			      line (), column ());
@@ -153,31 +143,10 @@ tree_switch_case::~tree_switch_case (void)
 }
 
 
-// Compare two octave values, returning true if equal, false if not
-// FIXME --- should be member or friend of octave_value class.
-
-static bool
-equal (const octave_value& val, const octave_value& test)
-{
-  bool retval = false;
-
-  // If there is no op_eq for these types, we can't compare values.
-
-  if (val.rows () == test.rows () && val.columns () == test.columns ())
-    {
-      octave_value tmp = do_binary_op (octave_value::op_eq, val, test);
-
-      if (! error_state && tmp.is_defined ())
-	retval = tmp.is_true ();
-    }
-
-  return retval;
-}
-
 bool
 tree_switch_case::label_matches (const octave_value& val)
 {
-  octave_value label_value = label->rvalue ();
+  octave_value label_value = label->rvalue1 ();
 
   if (! error_state && label_value.is_defined() )
     {
@@ -189,13 +158,10 @@ tree_switch_case::label_matches (const octave_value& val)
 	    {
 	      for (octave_idx_type j = 0; j < cell.columns (); j++)
 		{
-		  bool match = equal (val, cell(i,j));
+		  bool match = val.is_equal (cell(i,j));
 
 		  if (error_state)
-		    {
-		      eval_error ();
-		      return false;
-		    }
+		    return false;
 		  else if (match)
 		    return true;
 		}
@@ -203,50 +169,24 @@ tree_switch_case::label_matches (const octave_value& val)
 	}
       else
 	{
-	  bool match = equal (val, label_value);
+	  bool match = val.is_equal (label_value);
 
 	  if (error_state)
-	    {
-	      eval_error ();
-	      return false;
-	    }
+	    return false;
 	  else
 	    return match;
 	}
     }
-  else
-    eval_error ();
 
   return false;
 }
 
-int
-tree_switch_case::eval (const octave_value& val)
-{
-  int retval = 0;
-
-  if (is_default_case () || label_matches (val))
-    {
-      if (list)
-	list->eval ();
-
-      retval = 1;
-    }
-
-  return retval;
-}
-
-void
-tree_switch_case::eval_error (void)
-{
-  ::error ("evaluating switch case label");
-}
-
 tree_switch_case *
-tree_switch_case::dup (symbol_table *sym_tab)
+tree_switch_case::dup (symbol_table::scope_id scope,
+		       symbol_table::context_id context) const
 {
-  return new tree_switch_case (label ? label->dup (sym_tab) : 0,
-			       list ? list->dup (sym_tab) : 0,
+  return new tree_switch_case (label ? label->dup (scope, context) : 0,
+			       list ? list->dup (scope, context) : 0,
 			       lead_comm ? lead_comm->dup () : 0);
 }
 
@@ -258,28 +198,17 @@ tree_switch_case::accept (tree_walker& tw)
 
 // List of switch cases.
 
-void
-tree_switch_case_list::eval (const octave_value& val)
-{
-  for (iterator p = begin (); p != end (); p++)
-    {
-      tree_switch_case *t = *p;
-
-      if (t->eval (val) || error_state)
-	break;
-    }
-}
-
 tree_switch_case_list *
-tree_switch_case_list::dup (symbol_table *sym_tab)
+tree_switch_case_list::dup (symbol_table::scope_id scope,
+			    symbol_table::context_id context) const
 {
   tree_switch_case_list *new_scl = new tree_switch_case_list ();
 
-  for (iterator p = begin (); p != end (); p++)
+  for (const_iterator p = begin (); p != end (); p++)
     {
-      tree_switch_case *elt = *p;
+      const tree_switch_case *elt = *p;
 
-      new_scl->append (elt ? elt->dup (sym_tab) : 0);
+      new_scl->append (elt ? elt->dup (scope, context) : 0);
     }
   
   return new_scl;
@@ -302,40 +231,35 @@ tree_switch_command::~tree_switch_command (void)
 }
 
 void
-tree_switch_command::eval (void)
+tree_switch_command::set_breakpoint (void)
 {
-  if (expr)
+  if (list)
     {
-      octave_value val = expr->rvalue ();
+      tree_switch_case *elt = list->front ();
 
-      if (! error_state)
-	{
-	  if (list)
-	    list->eval (val);
-
-	  if (error_state)
-	    eval_error ();
-	}
-      else
-	eval_error ();
+      if (elt)
+	elt->set_breakpoint ();
     }
-  else
-    ::error ("missing value in switch command near line %d, column %d",
-	     line (), column ());
 }
 
 void
-tree_switch_command::eval_error (void)
+tree_switch_command::delete_breakpoint (void)
 {
-  ::error ("evaluating switch command near line %d, column %d",
-	   line (), column ());
+  if (list)
+    {
+      tree_switch_case *elt = list->front ();
+
+      if (elt)
+	elt->set_breakpoint ();
+    }
 }
 
 tree_command *
-tree_switch_command::dup (symbol_table *sym_tab)
+tree_switch_command::dup (symbol_table::scope_id scope,
+			  symbol_table::context_id context) const
 {
-  return new tree_switch_command (expr ? expr->dup (sym_tab) : 0,
-				  list ? list->dup (sym_tab) : 0,
+  return new tree_switch_command (expr ? expr->dup (scope, context) : 0,
+				  list ? list->dup (scope, context) : 0,
 				  lead_comm ? lead_comm->dup () : 0,
 				  trail_comm ? trail_comm->dup () : 0,
 				  line (), column ());
