@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-              2004, 2005, 2006, 2007 John W. Eaton
+              2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -50,6 +50,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-env.h"
 #include "quit.h"
 #include "str-vec.h"
+#include "oct-locbuf.h"
 
 #include <defaults.h>
 #include "defun.h"
@@ -66,6 +67,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "pathsearch.h"
 #include "procstream.h"
 #include "ov.h"
+#include "pt-eval.h"
 #include "pt-jump.h"
 #include "pt-stmt.h"
 #include "sighandlers.h"
@@ -77,6 +79,11 @@ along with Octave; see the file COPYING.  If not, see
 #include "variables.h"
 #include <version.h>
 
+void (*octave_exit) (int) = ::exit;
+
+// TRUE means the quit() call is allowed.
+bool quit_allowed = true;
+
 // TRUE means we are exiting via the builtin exit or quit functions.
 static bool quitting_gracefully = false;
 
@@ -90,89 +97,421 @@ bool octave_initialized = false;
 // Current command to execute.
 tree_statement_list *global_command = 0;
 
-// Pointer to parent function that is currently being evaluated.
-octave_function *curr_parent_function = 0;
-
 octave_call_stack *octave_call_stack::instance = 0;
 
-octave_function *
-octave_call_stack::do_caller (void)
+int
+octave_call_stack::do_current_line (void) const
 {
-  octave_function *retval = 0;
+  tree_statement *stmt = do_top_statement ();
 
-  if (cs.size () > 1)
-    {
-      iterator p = cs.begin ();
-      retval = *++p;
-    }
-
-  return retval;
+  return stmt ? stmt->line () : -1;
 }
 
-octave_user_script *
-octave_call_stack::do_caller_user_script (void)
+int
+octave_call_stack::do_current_column (void) const
 {
-  octave_user_script *retval = 0;
+  tree_statement *stmt = do_top_statement ();
 
-  for (iterator p = cs.begin (); p != cs.end (); p++)
+  return stmt ? stmt->column () : -1;
+}
+
+int
+octave_call_stack::do_caller_user_code_line (void) const
+{
+  int retval = -1;
+
+  const_iterator p = cs.end ();
+
+  while (p != cs.begin ())
     {
-      octave_function *f = *p;
+      const call_stack_elt& elt = *(--p);
 
-      if (f && f->is_user_script ())
+      octave_function *f = elt.fcn;
+
+      if (f && f->is_user_code ())
 	{
-	  retval = dynamic_cast<octave_user_script *> (f);
-	  break;
+	  tree_statement *stmt = elt.stmt;
+
+	  if (stmt)
+	    {
+	      retval = stmt->line ();
+	      break;
+	    }
 	}
     }
 
   return retval;
 }
 
-octave_user_function *
-octave_call_stack::do_caller_user_function (void)
+int
+octave_call_stack::do_caller_user_code_column (void) const
 {
-  octave_user_function *retval = 0;
+  int retval = -1;
 
-  for (iterator p = cs.begin (); p != cs.end (); p++)
+  const_iterator p = cs.end ();
+
+  while (p != cs.begin ())
     {
-      octave_function *f = *p;
+      const call_stack_elt& elt = *(--p);
 
-      if (f && f->is_user_function ())
+      octave_function *f = elt.fcn;
+
+      if (f && f->is_user_code ())
 	{
-	  retval = dynamic_cast<octave_user_function *> (f);
-	  break;
+	  tree_statement *stmt = elt.stmt;
+
+	  if (stmt)
+	    {
+	      retval = stmt->column ();
+	      break;
+	    }
 	}
     }
 
   return retval;
 }
 
-octave_function *
-octave_call_stack::do_caller_user_script_or_function (void)
+size_t
+octave_call_stack::do_num_user_code_frames (octave_idx_type& curr_user_frame) const
 {
-  octave_function *retval = 0;
+  size_t retval = 0;
 
-  for (iterator p = cs.begin (); p != cs.end (); p++)
+  curr_user_frame = 0;
+
+  // Look for the caller of dbstack.
+  size_t frame = cs[curr_frame].prev;
+
+  bool found = false;
+
+  size_t k = cs.size ();
+
+  for (const_reverse_iterator p = cs.rbegin (); p != cs.rend (); p++)
     {
-      octave_function *f = *p;
+      octave_function *f = (*p).fcn;
 
-      if (f && (f->is_user_script () || f->is_user_function ()))
+      if (--k == frame)
+	found = true;
+
+      if (f && f->is_user_code ())
 	{
-	  retval = f;
-	  break;
+	  if (! found)
+	    curr_user_frame++;
+
+	  retval++;
 	}
     }
 
+  // We counted how many user frames were not the one, in reverse.
+  // Now set curr_user_frame to be the index in the other direction.
+  curr_user_frame = retval - curr_user_frame - 1;
+
+  return retval;
+}
+
+octave_user_code *
+octave_call_stack::do_caller_user_code (size_t nskip) const
+{
+  octave_user_code *retval = 0;
+
+  const_iterator p = cs.end ();
+
+  while (p != cs.begin ())
+    {
+      const call_stack_elt& elt = *(--p);
+
+      octave_function *f = elt.fcn;
+
+      if (f && f->is_user_code ())
+	{
+	  if (nskip > 0)
+	    nskip--;
+	  else
+	    {
+	      retval = dynamic_cast<octave_user_code *> (f);
+	      break;
+	    }
+	}
+    }
+
+  return retval;
+}
+
+Octave_map
+octave_call_stack::do_backtrace (size_t nskip,
+				 octave_idx_type& curr_user_frame) const
+{
+  Octave_map retval;
+
+  size_t user_code_frames = do_num_user_code_frames (curr_user_frame);
+
+  size_t nframes = nskip <= user_code_frames ? user_code_frames - nskip : 0;
+
+  // Our list is reversed.
+  curr_user_frame = nframes - curr_user_frame - 1;
+
+  Cell keys (6, 1);
+
+  keys(0) = "file";
+  keys(1) = "name";
+  keys(2) = "line";
+  keys(3) = "column";
+  keys(4) = "scope";
+  keys(5) = "context";
+
+  Cell file (nframes, 1);
+  Cell name (nframes, 1);
+  Cell line (nframes, 1);
+  Cell column (nframes, 1);
+  Cell scope (nframes, 1);
+  Cell context (nframes, 1);
+
+  if (nframes > 0)
+    {
+      int k = 0;
+
+      for (const_reverse_iterator p = cs.rbegin (); p != cs.rend (); p++)
+	{
+	  const call_stack_elt& elt = *p;
+
+	  octave_function *f = elt.fcn;
+
+	  if (f && f->is_user_code ())
+	    {
+	      if (nskip > 0)
+		nskip--;
+	      else
+		{
+		  scope(k) = elt.scope;
+		  context(k) = elt.context;
+
+		  file(k) = f->fcn_file_name ();
+		  std::string parent_fcn_name = f->parent_fcn_name ();
+		  if (parent_fcn_name == std::string ())
+		    name(k) = f->name ();
+		  else
+		    name(k) = f->parent_fcn_name () + Vfilemarker + f->name ();
+
+		  tree_statement *stmt = elt.stmt;
+
+		  if (stmt)
+		    {
+		      line(k) = stmt->line ();
+		      column(k) = stmt->column ();
+		    }
+		  else
+		    {
+		      line(k) = -1;
+		      column(k) = -1;
+		    }
+
+		  k++;
+		}
+	    }
+	}
+
+      retval.assign ("file", file);
+      retval.assign ("name", name);
+      retval.assign ("line", line);
+      retval.assign ("column", column);
+      retval.assign ("scope", scope);
+      retval.assign ("context", context);
+    }
+
+  return retval;
+}
+
+bool
+octave_call_stack::do_goto_frame (size_t n, bool verbose)
+{
+  bool retval = false;
+
+  if (n < cs.size ())
+    {
+      retval = true;
+
+      curr_frame = n;
+
+      const call_stack_elt& elt = cs[n];
+
+      symbol_table::set_scope_and_context (elt.scope, elt.context);
+
+      if (verbose)
+	{
+	  octave_function *f = elt.fcn;
+	  std::string nm = f ? f->name () : std::string ("<unknown>");
+
+	  tree_statement *s = elt.stmt;
+	  int l = -1;
+	  int c = -1;
+	  if (s)
+	    {
+	      l = s->line ();
+	      c = s->column ();
+	    }
+
+	  octave_stdout << "stopped in " << nm
+			<< " at line " << l << " column " << c
+			<< " (" << elt.scope << "[" << elt.context << "])"
+			<< std::endl;
+	}
+    }
+
+  return retval;
+}
+
+bool
+octave_call_stack::do_goto_frame_relative (int nskip, bool verbose)
+{
+  bool retval = false;
+
+  if (nskip == 0)
+    retval = true;
+  else
+    {
+      int incr = nskip < 0 ? -1 : 1;
+
+      // Start looking with the caller of dbup/dbdown.
+      size_t frame = cs[curr_frame].prev;
+
+      while (true)
+	{
+	  if ((incr < 0 && frame == 0) || (incr > 0 && frame == cs.size () - 1))
+	    break;
+
+	  frame += incr;
+
+	  const call_stack_elt& elt = cs[frame];
+
+	  octave_function *f = elt.fcn;
+
+	  if (f && f->is_user_code ())
+	    {
+	      if (nskip > 0)
+		nskip--;
+	      else if (nskip < 0)
+		nskip++;
+
+	      if (nskip == 0)
+		{
+		  curr_frame = frame;
+		  cs[cs.size () - 1].prev = curr_frame;
+
+		  if (verbose)
+		    {
+		      tree_statement *s = elt.stmt;
+		      int l = -1;
+		      int c = -1;
+		      if (s)
+			{
+			  l = s->line ();
+			  c = s->column ();
+			}
+
+		      std::ostringstream buf;
+		      buf << f->name () << ": " << " line " << l
+			  << ", column " << c << std::endl;
+
+		      octave_stdout << buf.str ();
+		    }
+
+		  retval = true;
+		  break;
+		}
+	    }
+	}
+
+      // There is no need to set scope and context here.  That will
+      // happen when the dbup/dbdown frame is popped and we jump to
+      // the new "prev" frame set above.
+    }
+
   return retval;
 }
 
 void
-octave_call_stack::unwind_pop_script (void *)
+octave_call_stack::do_goto_caller_frame (void)
 {
-  octave_function *f = top ();
-  pop ();
-  assert (f && f->is_user_script ());
-  delete f;
+  size_t frame = curr_frame;
+
+  bool skipped = false;
+
+  while (frame != 0)
+    {
+      frame = cs[frame].prev;
+
+      const call_stack_elt& elt = cs[frame];
+
+      octave_function *f = elt.fcn;
+
+      if (frame == 0 || (f && f->is_user_code ()))
+	{
+	  if (! skipped)
+	    // We found the current user code frame, so skip it.
+	    skipped = true;
+	  else
+	    {
+	      // We found the caller user code frame.
+	      call_stack_elt tmp (elt);
+	      tmp.prev = curr_frame;
+
+	      curr_frame = cs.size ();
+
+	      cs.push_back (tmp);
+
+	      symbol_table::set_scope_and_context (tmp.scope, tmp.context);
+
+	      break;
+	    }
+	}
+    }
+}
+
+void
+octave_call_stack::do_goto_base_frame (void)
+{
+  call_stack_elt tmp (cs[0]);
+  tmp.prev = curr_frame;
+
+  curr_frame = cs.size ();
+
+  cs.push_back (tmp);
+
+  symbol_table::set_scope_and_context (tmp.scope, tmp.context);
+}
+
+void
+octave_call_stack::do_backtrace_error_message (void) const
+{
+  if (error_state > 0)
+    {
+      error_state = -1;
+
+      error ("called from:");
+    }
+
+  if (! cs.empty ())
+    {
+      const call_stack_elt& elt = cs.back ();
+
+      octave_function *fcn = elt.fcn;
+      tree_statement *stmt = elt.stmt;
+
+      std::string fcn_name = "?unknown?";
+
+      if (fcn)
+	{
+	  fcn_name = fcn->fcn_file_name ();
+
+	  if (fcn_name.empty ())
+	    fcn_name = fcn->name ();
+	}
+
+      int line = stmt ? stmt->line () : -1;
+      int column = stmt ? stmt->column () : -1;
+
+      error ("  %s at line %d, column %d",
+	     fcn_name.c_str (), line, column);
+    }
 }
 
 void
@@ -182,7 +521,7 @@ recover_from_exception (void)
   octave_interrupt_immediately = 0;
   octave_interrupt_state = 0;
   octave_signal_caught = 0;
-  octave_allocation_error = 0;
+  octave_exception_state = octave_no_exception;
   octave_restore_signal_mask ();
   octave_catch_interrupts ();
 }
@@ -209,12 +548,18 @@ main_loop (void)
     {
       try
 	{
-	  curr_sym_tab = top_level_sym_tab;
+	  unwind_protect::begin_frame ("main_loop");
 
 	  reset_error_handler ();
 
 	  reset_parser ();
 
+	  // Do this with an unwind-protect cleanup function so that
+	  // the forced variables will be unmarked in the event of an
+	  // interrupt.
+	  symbol_table::scope_id scope = symbol_table::top_scope ();
+	  unwind_protect::add (symbol_table::unmark_forced_variables, &scope);
+
 	  // This is the same as yyparse in parse.y.
 	  retval = octave_parse ();
 
@@ -222,7 +567,7 @@ main_loop (void)
 	    {
 	      if (global_command)
 		{
-		  global_command->eval ();
+		  global_command->accept (*current_evaluator);
 
 		  delete global_command;
 
@@ -265,12 +610,26 @@ main_loop (void)
 	      else if (parser_end_of_input)
 		break;
 	    }
+
+	  unwind_protect::run_frame ("main_loop");
 	}
+      catch (octave_quit_exception e)
+        {
+          unwind_protect::run_all ();
+          clean_up_and_exit (e.status);
+        }
       catch (octave_interrupt_exception)
 	{
 	  recover_from_exception ();
 	  octave_stdout << "\n";
 	}
+      catch (octave_execution_exception)
+	{
+	  recover_from_exception ();
+	  std::cerr
+	    << "error: unhandled execution exception -- trying to return to prompt"
+	    << std::endl;
+	}
       catch (std::bad_alloc)
 	{
 	  recover_from_exception ();
@@ -293,7 +652,8 @@ clean_up_and_exit (int retval)
 
   sysdep_cleanup ();
 
-  exit (retval == EOF ? 0 : retval);
+  if (octave_exit)
+    (*octave_exit) (retval == EOF ? 0 : retval);
 }
 
 DEFUN (quit, args, nargout,
@@ -307,7 +667,9 @@ Octave's exit status.  The default value is zero.\n\
 {
   octave_value_list retval;
 
-  if (nargout == 0)
+  if (! quit_allowed)
+    error ("quit: not supported in embedded mode.");
+  else if (nargout == 0)
     {
       int exit_status = 0;
 
@@ -321,7 +683,7 @@ Octave's exit status.  The default value is zero.\n\
 	    exit_status = tmp;
 	}
 
-      clean_up_and_exit (exit_status);
+      throw octave_quit_exception (exit_status);
     }
   else
     error ("quit: invalid number of output arguments");
@@ -437,7 +799,7 @@ run_command_and_return_output (const std::string& cmd_str)
 	  else
 	    cmd_status = 127;
 
-	  retval(0) = (double) cmd_status;
+	  retval(0) = cmd_status;
 	  retval(1) = output_buf.str ();
 	}
 
@@ -458,7 +820,7 @@ Execute a shell command specified by @var{string}.  The second\n\
 argument is optional.  If @var{type} is @code{\"async\"}, the process\n\
 is started in the background and the process id of the child process\n\
 is returned immediately.  Otherwise, the process is started, and\n\
-Octave waits until it exits.  If @var{type} argument is omitted, a\n\
+Octave waits until it exits.  If the @var{type} argument is omitted, a\n\
 value of @code{\"sync\"} is assumed.\n\
 \n\
 If two input arguments are given (the actual value of\n\
@@ -477,7 +839,7 @@ disp (system (cmd, 1));\n\
 or\n\
 \n\
 @example\n\
-printf (\"%s\n\", system (cmd, 1));\n\
+printf (\"%s\\n\", system (cmd, 1));\n\
 @end example\n\
 \n\
 The @code{system} function can return two values.  The first is the\n\
@@ -643,7 +1005,8 @@ do_octave_atexit (void)
 
       octave_history_write_timestamp ();
 
-      command_history::clean_up_and_save ();
+      if (Vsaving_history)
+	command_history::clean_up_and_save ();
 
       close_files ();
 
@@ -663,36 +1026,63 @@ do_octave_atexit (void)
     }
 }
 
+void
+octave_add_atexit_function (const std::string& fname)
+{
+  octave_atexit_functions.push_front (fname);
+}
+
+bool
+octave_remove_atexit_function (const std::string& fname)
+{
+  bool found = false;
+
+  for (std::list<std::string>::iterator p = octave_atexit_functions.begin ();
+       p != octave_atexit_functions.end (); p++)
+    {
+      if (*p == fname)
+	{
+	  octave_atexit_functions.erase (p);
+	  found = true;
+	  break;
+	}
+    }
+
+  return found;
+}
+
+
 DEFUN (atexit, args, nargout,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} atexit (@var{fcn})\n\
+@deftypefnx {Built-in Function} {} atexit (@var{fcn}, @var{flag})\n\
 Register a function to be called when Octave exits.  For example,\n\
 \n\
 @example\n\
 @group\n\
-function bye_bye ()\n\
+function last_words ()\n\
   disp (\"Bye bye\");\n\
 endfunction\n\
-atexit (\"bye_bye\");\n\
+atexit (\"last_words\");\n\
 @end group\n\
 @end example\n\
 \n\
 @noindent\n\
 will print the message \"Bye bye\" when Octave exits.\n\
 \n\
-@deftypefnx {Built-in Function} {} atexit (@var{fcn}, @var{flag})\n\
-Register or unregister a function to be called when Octave exits,\n\
-depending on @var{flag}.  If @var{flag} is true, the function is\n\
-registered, if @var{flag} is false, it is unregistered.  For example,\n\
-after registering the function @code{bye_bye} as above,\n\
+The additional argument @var{flag} will register or unregister\n\
+@var{fcn} from the list of functions to be called when Octave\n\
+exits.  If @var{flag} is true, the function is registered, and if\n\
+@var{flag} is false, it is unregistered.  For example,\n\
+after registering the function @code{last_words} above,\n\
 \n\
 @example\n\
-atexit (\"bye_bye\", false);\n\
+atexit (\"last_words\", false);\n\
 @end example\n\
 \n\
 @noindent\n\
 will remove the function from the list and Octave will not call\n\
-the function @code{bye_by} when it exits.\n\
+@code{last_words} when it exits.\n\
 \n\
 Note that @code{atexit} only removes the first occurrence of a function\n\
 from the list, so if a function was placed in the list multiple\n\
@@ -723,22 +1113,10 @@ multiple times.\n\
           if (! error_state)
 	    {
 	      if (add_mode)
-		octave_atexit_functions.push_front (arg);
+		octave_add_atexit_function (arg);
 	      else
 		{
-		  bool found = false;
-		  std::list<std::string>::iterator it;
-
-		  for (std::list<std::string>::iterator p = octave_atexit_functions.begin ();
-		       p != octave_atexit_functions.end (); p++)
-		    {
-		      if (*p == arg)
-			{
-			  octave_atexit_functions.erase (p);
-			  found = true;
-			  break;
-			}
-		    }
+		  bool found = octave_remove_atexit_function (arg);
 
 		  if (nargout > 0)
 		    retval(0) = found;
@@ -792,6 +1170,7 @@ specified option.\n\
       { false, "AR", OCTAVE_CONF_AR },
       { false, "ARFLAGS", OCTAVE_CONF_ARFLAGS },
       { false, "BLAS_LIBS", OCTAVE_CONF_BLAS_LIBS },
+      { false, "CARBON_LIBS", OCTAVE_CONF_CARBON_LIBS },
       { false, "CC", OCTAVE_CONF_CC },
       { false, "CC_VERSION", OCTAVE_CONF_CC_VERSION },
       { false, "CFLAGS", OCTAVE_CONF_CFLAGS },
@@ -809,8 +1188,6 @@ specified option.\n\
       { false, "DL_LDFLAGS", OCTAVE_CONF_DL_LDFLAGS },
       { false, "ENABLE_DYNAMIC_LINKING", OCTAVE_CONF_ENABLE_DYNAMIC_LINKING },
       { false, "EXEEXT", OCTAVE_CONF_EXEEXT },
-      { false, "F2C", OCTAVE_CONF_F2C },
-      { false, "F2CFLAGS", OCTAVE_CONF_F2CFLAGS },
       { false, "F77", OCTAVE_CONF_F77 },
       { false, "F77_FLOAT_STORE_FLAG", OCTAVE_CONF_F77_FLOAT_STORE_FLAG },
       { false, "FC", OCTAVE_CONF_FC },
@@ -819,6 +1196,7 @@ specified option.\n\
       { false, "FLIBS", OCTAVE_CONF_FLIBS },
       { false, "FPICFLAG", OCTAVE_CONF_FPICFLAG },
       { false, "GLPK_LIBS", OCTAVE_CONF_GLPK_LIBS },
+      { false, "GNUPLOT", OCTAVE_CONF_GNUPLOT },
       { false, "INCFLAGS", OCTAVE_CONF_INCFLAGS },
       { false, "LDFLAGS", OCTAVE_CONF_LDFLAGS },
       { false, "LD_CXX", OCTAVE_CONF_LD_CXX },
@@ -834,6 +1212,8 @@ specified option.\n\
       { false, "LIBREADLINE", OCTAVE_CONF_LIBREADLINE },
       { false, "LIBS", OCTAVE_CONF_LIBS },
       { false, "LN_S", OCTAVE_CONF_LN_S },
+      { false, "MAGICK_INCFLAGS", OCTAVE_CONF_MAGICK_INCFLAGS },
+      { false, "MAGICK_LIBS", OCTAVE_CONF_MAGICK_LIBS },
       { false, "MKOCTFILE_DL_LDFLAGS", OCTAVE_CONF_MKOCTFILE_DL_LDFLAGS },
       { false, "RANLIB", OCTAVE_CONF_RANLIB },
       { false, "RDYNAMIC_FLAG", OCTAVE_CONF_RDYNAMIC_FLAG },
@@ -848,6 +1228,8 @@ specified option.\n\
       { false, "STATIC_LIBS", OCTAVE_CONF_STATIC_LIBS },
       { false, "UGLY_DEFS", OCTAVE_CONF_UGLY_DEFS },
       { false, "USE_64_BIT_IDX_T", OCTAVE_CONF_USE_64_BIT_IDX_T },
+      { false, "X11_INCFLAGS", OCTAVE_CONF_X11_INCFLAGS },
+      { false, "X11_LIBS", OCTAVE_CONF_X11_LIBS },
       { false, "XTRA_CFLAGS", OCTAVE_CONF_XTRA_CFLAGS },
       { false, "XTRA_CXXFLAGS", OCTAVE_CONF_XTRA_CXXFLAGS },
       { false, "YACC", OCTAVE_CONF_YACC },
@@ -881,6 +1263,7 @@ specified option.\n\
       { false, "man1ext", OCTAVE_MAN1EXT },
       { true, "mandir", OCTAVE_MANDIR },
       { true, "octfiledir", OCTAVE_OCTFILEDIR },
+      { true, "octetcdir", OCTAVE_OCTETCDIR },
       { true, "octincludedir", OCTAVE_OCTINCLUDEDIR },
       { true, "octlibdir", OCTAVE_OCTLIBDIR },
       { true, "prefix", OCTAVE_PREFIX },
@@ -933,7 +1316,7 @@ specified option.\n\
 #endif
 #endif
 
-#if defined (__APPLE__) && defined (__MACH__)
+#if defined (OCTAVE_USE_OS_X_API)
       mac_system = true;
 #endif
 
