@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2004,
-              2005, 2006, 2007 John W. Eaton
+              2005, 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -45,6 +45,7 @@ public:
   {
     boolean,
     integer,
+    size_type,
     string_type,
     generic_ptr,
     generic
@@ -56,6 +57,8 @@ public:
 
   saved_variable (int *p, int v);
 
+  saved_variable (size_t *p, size_t v);
+
   saved_variable (std::string *p, const std::string& v);
 
   saved_variable (void **p, void *v);
@@ -72,6 +75,7 @@ private:
     {
       bool *ptr_to_bool;
       int *ptr_to_int;
+      size_t *ptr_to_size_t;
       void *gen_ptr;
       void **ptr_to_gen_ptr;
     };
@@ -80,6 +84,7 @@ private:
     {
       bool bool_value;
       int int_value;
+      size_t size_t_value;
       std::string *str_value;
       void *gen_ptr_value;
     };
@@ -113,6 +118,14 @@ saved_variable::saved_variable (int *p, int v)
   size = sizeof (int);  // Is this necessary?
 }
 
+saved_variable::saved_variable (size_t *p, size_t v)
+{
+  type_tag = size_type;
+  ptr_to_size_t = p;
+  size_t_value = v;
+  size = sizeof (size_t);  // Is this necessary?
+}
+
 saved_variable::saved_variable (std::string *p, const std::string& v)
 {
   type_tag = string_type;
@@ -160,6 +173,10 @@ saved_variable::restore_value (void)
       *ptr_to_int = int_value;
       break;
 
+    case size_type:
+      *ptr_to_size_t = size_t_value;
+      break;
+
     case string_type:
       (static_cast<std::string *> (gen_ptr)) -> assign (*str_value);
       break;
@@ -289,6 +306,13 @@ unwind_protect::save_int (int *ptr, int value)
   add (saved_variable::restore, s);
 }
 
+void
+unwind_protect::save_size_t (size_t *ptr, size_t value)
+{
+  saved_variable *s = new saved_variable (ptr, value);
+  add (saved_variable::restore, s);
+}
+
 void
 unwind_protect::save_str (std::string *ptr, const std::string& value)
 {
