@@ -1,7 +1,8 @@
 /*
 
 Copyright (C) 1992, 1993, 1994, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2003, 2004, 2005, 2006, 2007, 2008 John W. Eaton
+              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
+              John W. Eaton
 
 This file is part of Octave.
 
@@ -24,13 +25,13 @@ along with Octave; see the file COPYING.  If not, see
 #if !defined (octave_version_h)
 #define octave_version_h 1
 
-#define OCTAVE_VERSION "3.0.3"
+#define OCTAVE_VERSION "3.2.0"
 
-#define OCTAVE_API_VERSION "api-v32"
+#define OCTAVE_API_VERSION "api-v37"
 
-#define OCTAVE_RELEASE_DATE "2008-09-24"
+#define OCTAVE_RELEASE_DATE "2009-06-03"
 
-#define OCTAVE_COPYRIGHT "Copyright (C) 2008 John W. Eaton and others."
+#define OCTAVE_COPYRIGHT "Copyright (C) 2009 John W. Eaton and others."
 
 // This is the first line printed by --version.  The GNU coding
 // standards say that the version number should follow the last space
@@ -45,7 +46,7 @@ along with Octave; see the file COPYING.  If not, see
   "This is free software; see the source code for copying conditions."
 
 #define X_OCTAVE_WARRANTY_STATEMENT(ARG) \
-  "There is ABSOLUTELY NO WARRANTY; not even for MERCHANTIBILITY or\n\
+  "There is ABSOLUTELY NO WARRANTY; not even for MERCHANTABILITY or\n\
 FITNESS FOR A PARTICULAR PURPOSE." ARG
 
 #define OCTAVE_WARRANTY_STATEMENT \
